/** 
 * \file mgui/formcoord.h 
 * \brief Definitions for MGUI::FORM_EDIT_COORD class.
 *
 * \if NODOC
 * $Id: formcoord.h_v 1.6 2011/11/23 13:58:32 mju Exp $
 *
 * $Log: formcoord.h_v $
 * Revision 1.6  2011/11/23 13:58:32  mju
 * Remove redundant hdrs and fix inclusion guard name.
 *
 * Revision 1.5  2010/11/12 16:03:05  mju
 * Add getLabelX/Y/Z.
 *
 * Revision 1.4  2009/04/02 21:38:52  bcolombe
 * Added FLAG_ShowCRS and support for setting crs via button
 *
 * Revision 1.3  2008/06/26 22:41:19  scowan
 * Added flag parm for create method.
 *
 * Revision 1.2  2005/06/24 15:20:45  mju
 * Add setDelegate... and remove obsolete template.
 *
 * Revision 1.1  2004/09/10 19:25:40  dwilliss
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORM_COORD_H
#define	INC_MGUI_FORM_COORD_H

#ifndef	INC_MGUI_FORMBTXT_H
#include <mgui/formbtxt.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif

namespace MGUI {

//===================================================================================================================
//! Form consisting of label and edit fields for editing a single coordinate
class FORM_EDIT_COORD : public MGUI::FORM_COMPOSITE {
	public:
		enum FLAGS {
			FLAG_Default = 	0x0000,
			FLAG_EditZCoord = 0x0001,
			FLAG_ShowCRS = 0x0002
			};

		//! Constructor.
		FORM_EDIT_COORD (
			);

		//! Destructor.
		virtual ~FORM_EDIT_COORD (
			);

		//! Clear range edit controls.
		void Clear (
			bool notify = true
			);

		//! Create form with label from resource lookup.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			FLAGS flags = FLAG_Default
			);

		//! Get X label control.
		CTRL_LABEL& GetLabelX ()
			{ return (m_XLabel); }

		//! Get Y label control.
		CTRL_LABEL& GetLabelY ()
			{ return (m_YLabel); }

		//! Get Z label control.
		CTRL_LABEL& GetLabelZ ()
			{ return (m_ZLabel); }

		//! Get the extents of the form
		const DPOINT3D& GetValue (
			) const { return (m_Point); }
			
		//! Get the projection to use for extents selection.
		const SPATREF::COORDREFSYS& GetCoordRefSys (
			) const { return (m_CoordRefSys); }

		//! Determine if user has entered values for both x and y (and z if enabled)
		bool HasValues (
			);

		//! Set delegate to call when value changed.
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeValue = delegate; }

		//! Set the coordinate of the form
		//! Do this after the call to SetProjection() or the call to the method will alter the point
		void SetValue (
			const DPOINT2D& Point,
			bool notify = true
			);

		//! Set the coordinate of the form
		//! Do this after the call to SetProjection() or the call to the method will alter the point
		void SetValue (
			const DPOINT3D& Point,
			bool notify = true
			);

		//! Set the projection to use for coordinate selection.
		//! If a projection and value coordinates exists, the coordinate will be converted to the new projection
		void SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys,
			bool notify = true
			);


	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SPATREF::COORDREFSYS m_CoordRefSys;
		DPOINT3D m_Point;
		bool m_bIs3D;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeCoordRefSys;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeValue;
		
		FORM_BUTTONTEXT m_BtxCoordRefSys;

		LAYOUT_PANE m_XPane;
		CTRL_LABEL m_XLabel;
		CTRL_EDIT_NUMBER m_XPrompt;
		
		LAYOUT_PANE m_YPane;
		CTRL_LABEL m_YLabel;
		CTRL_EDIT_NUMBER m_YPrompt;
		
		LAYOUT_PANE m_ZPane;
		CTRL_LABEL m_ZLabel;
		CTRL_EDIT_NUMBER m_ZPrompt;
		
		void NotifyChangeValue ();
		void OnValueChanged ();
		void OnBtnCoordRefSys ();
		void UpdateProjection ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End of MGUI namespace

#endif		//	INC_MGUI_FORM_COORD_H
