/**
 * \file mgui/imagelst.h
 * \brief MGUI::IMAGELIST class definitions
 *
 * \if NODOC
 * $Id: imagelst.h_v 1.17 2011/12/02 15:21:35 mju Exp $
 *
 * $Log: imagelst.h_v $
 * Revision 1.17  2011/12/02 15:21:35  mju
 * Remove mfc stuff.
 *
 * Revision 1.15  2009/10/05 20:46:09  mju
 * Add ctor using bgcolor.
 *
 * Revision 1.14  2008/01/10 15:54:44  mju
 * Add findIcon.
 *
 * Revision 1.13  2006/10/19 12:43:32  mju
 * Use iconid hdr - Don't know why msvc allows forward-declare enum, is non-std.
 *
 * Revision 1.12  2006/10/18 16:51:49  mju
 * Use miconstruct* instead of micon typedef.
 *
 * Revision 1.8  2003/07/25 22:22:55  dwilliss
 * Added an AddIcon method that takes a background color
 *
 * Revision 1.2  2002/02/06 22:22:09  dwilliss
 * Use ICONIDs which are portable between X and Windows
 *
 * Revision 1.1  2002/02/05 21:08:29  dwilliss
 * Initial revision
 * \endif
**/

#ifndef INC_MGUI_IMAGELST_H
#define INC_MGUI_IMAGELST_H

#ifndef INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef INC_MI32_ICONID_H
	#include <mi32/iconid.h>
#endif

#ifndef INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
struct COLOR;
struct MICONSTRUCT;
#endif

namespace MGUI {

//! Image list for icons, bitmaps, etc.
class IMAGELIST {
	public:

		//! Default constructor.
		IMAGELIST (
			);

		//! Construct with alternative background color.
		explicit IMAGELIST (
			const COLOR& bgcolor
			);

		// Copy constructor.
		IMAGELIST (
			const IMAGELIST& rhs
			);

		//! Destructor.
		~IMAGELIST (); 

		// Assignment.
		IMAGELIST& operator= (
			const IMAGELIST& rhs
			);

		//! Add an icon given an ICONID.
		//! @return Icon index in the image list or < 0 if an error
		int AddIcon (
			ICONID id,								//!< Icon ID
			int size = 16							//!< Icon size
			);

		//! Add an icon given an ICONID.
		//! @return Icon index in the image list or < 0 if an error
		int AddIcon (
			ICONID id,								//!< Icon ID
			const COLOR& bgcolor,				//!< Color to use for background
			int size = 16							//!< Icon size
			);

		//! Find icon in list if exists.
		//! @return Icon pointer or NULL if not in list
		const MICONSTRUCT *FindIcon (
			ICONID iconid
			) const;

		//! Get MICONSTRUCT pointer for image list index.
		//! @return Icon pointer.
		MICONSTRUCT *GetIcon (
			int index
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		SIMPLE_ARRAY<MICONSTRUCT*> m_Icons;
		COLOR m_bgcolor;

		void AddRef ();
		void Free ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

} // End of MGUI namespace


#endif  // INC_MGUI_IMAGELST_H
