/**
 * \file mgui/popupcolor.h
 * \brief MGUI::POPUP_COLOR definitions
 *
 * \if NODOC
 * $Id: popupcolor.h_v 1.5 2011/12/02 22:23:30 mju Exp $
 *
 * $Log: popupcolor.h_v $
 * Revision 1.5  2011/12/02 22:23:30  mju
 * Remove redundant hdrs.
 *
 * Revision 1.4  2011/08/24 21:37:39  mju
 * Add transparency controls.
 *
 * Revision 1.3  2011/08/24 15:48:50  mju
 * Add initialize.
 *
 * Revision 1.2  2011/08/22 20:27:28  mju
 * Add generic toggles instead of special "none" toggle.
 *
 * Revision 1.1  2011/08/19 20:32:05  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_POPUPCOLOR_H
#define	INC_MGUI_POPUPCOLOR_H

#ifndef	INC_MGUI_POPUP_H
#include <mgui/popup.h>
#endif

#ifndef	INC_MGUI_CANVAS_H
#include <mgui/canvas.h>
#endif

#ifndef	INC_MGUI_SLIDER_H
#include <mgui/slider.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class COLORPALETTE;
namespace MGD {
	class CONTEXT;
	}
#endif


namespace MGUI {

//=====================================================================================================================
//! Popup window for choosing color.
class POPUP_COLOR : public MGUI::POPUP {
	public:

		enum STYLE {
			STYLE_Default =				0x00,		//!< Default style, no transparency option
			STYLE_AllowTransparency =	0x01,		//!< Include control for transparency
			};

		//! Construct with optional style.
		explicit POPUP_COLOR (
			STYLE style = STYLE_Default
			);

		//! Destructor.
		~POPUP_COLOR ();

		//! Add toggle with label below palette selection.
		//! If used, must be done before popup is created or first shown.
		void AddToggle (
			MGUI::ID id,				//!< Toggle id
			const MISTRING& label,	//!< Label to show for toggle
			bool dft = false			//!< Default value
			);

		//! Get current color.
		const COLOR& GetColor () const
			{ return (m_color); }

		//! Get value of toggle.
		bool GetToggleValue (
			MGUI::ID id					//!< Toggle id
			) const;

		//! Initialize label, color and style.
		//! If used, must be called before popup is created or first opened.
		void Initialize (
			const MISTRING& label,			//!< Label string
			const COLOR& color,				//!< Initial color
			STYLE style = STYLE_Default	//!< Popup style
			);

		//! Set color.
		void SetColor (
			const COLOR& color		//!< Color to set
			) { m_color = color; }

		//! Set label to show at top of popup.
		//! If used, must be called before popup is created or first opened.
		void SetLabel (
			const MISTRING& label
			) { m_labelstr = label; }

		//! Set value of toggle.
		void SetToggleValue (
			MGUI::ID id,				//!< Toggle id
			bool value					//!< Value to set
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef POPUP_COLOR THISCLASS;

		struct TGLITEM {
			CTRL_TOGGLEBUTTON *m_pTgl;
			MISTRING m_labelstr;
			MGUI::ID m_id;
			bool m_dftvalue;

			TGLITEM ()
				: m_pTgl(0), m_id(-1), m_dftvalue(false) { }

			TGLITEM (MGUI::ID id, const MISTRING& labelstr, bool dftvalue)
				: m_pTgl(0), m_labelstr(labelstr), m_id(id), m_dftvalue(dftvalue) { }

			~TGLITEM ()
				{ delete m_pTgl; }
			};
		typedef MILIST<TGLITEM> TGLLIST;

		STYLE m_style;
		MISTRING m_labelstr;
		COLOR m_color;
		int m_palindex;
		int m_cellsize;
		int m_x0;
		int m_y0;
		CTRL_LABEL m_Label;
		CTRL_SEPARATOR m_Sep;
		CANVAS m_CnvPalette;
		TGLLIST m_TglList;
		CTRL_LABEL m_LblTransparency;
		FORM_SLIDER m_CtlTransparency;
		CTRL_PUSHBUTTON m_BtnMore;
		const COLORPALETTE *m_pColorPalette;

		int ComputePaletteIndex (MGUI::POINT ptCanvas) const;
		void DrawHighlight (int palindex, bool set);
		void DrawPalette ();
		void OnBtnMore ();
		void OnChgTransparency ();
		void OnCnvPalettePaint (MGD::CONTEXT *gc);
		bool OnCnvPalettePointerButtonDown (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate);
		bool OnCnvPalettePointerButtonUp (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate);
		bool OnCnvPalettePointerMove (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate, bool hasmore);
		void OnTgl ();

		// POPUP overrides.
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOpen ();
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(POPUP_COLOR::STYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_POPUPCOLOR_H
