/**
 * \file mgui/toolcircle.h
 * \brief MGUI::TOOL_CIRCLE class definition
 *
 * \if NODOC
 * $Id: toolcircle.h_v 1.11 2011/12/09 20:22:55 mju Exp $
 *
 * $Log: toolcircle.h_v $
 * Revision 1.11  2011/12/09 20:22:55  mju
 * No context in v_draw.
 *
 * Revision 1.10  2011/06/22 13:29:26  mju
 * *** empty log message ***
 *
 * Revision 1.7  2008/02/15 23:37:25  scowan
 * Added manual entry stubs.
 *
 * Revision 1.5  2008/01/04 18:59:05  mju
 * Generalize pointer button methods.
 *
 * Revision 1.1  2003/01/10 23:08:10  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_TOOLCIRCLE_H
#define	INC_MGUI_TOOLCIRCLE_H

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Circle tool.
class TOOL_CIRCLE : public TOOL_BASE {
	public:

		//! Constructor.
		explicit TOOL_CIRCLE (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_CIRCLE ();

		//! Get current rotation angle.
		double GetAngle (
			) const { return (m_angle); }

		//! Get center position.
		const DPOINT2D& GetCenter (
			) const { return (m_center); }

		//! Get radius.
		double GetRadius (
			) const { return (m_radius); }

		//! Set rotation angle.
		//! The rotation will only be visible to the user if SetAngleVisible() has been used.
		void SetAngle (
			double angle								//!< New rotation angle in RADIANS
			);

		//! Set whether rotation angle indicator is visible.
		//! The rotation angle is indicated by a line from the center to the boundary.
		void SetAngleVisible (
			bool anglevisible = true
			);

		//! Set circle center coordinates.
		void SetCenter (
			const DPOINT2D& center
			);

		//! Set marker type and size.
		//! The default marker is MARKER_Plus with a size of 9 pixels.
		void SetMarker (
			MARKER marker,								//!< New marker type
			UINT16 size = 0							//!< New marker size, 0 to retain current size
			);

		//! Set radius.
		void SetRadius (
			double radius
			);

		//! Set allowed radius range.
		void SetRadiusRange (
			double minradius,							//!< Minimum radius in tool units
			double maxradius							//!< Maximum radius, <= 0 for no maximum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Enumeration constants
		enum ZONE {
			ZONE_NoFunction = 0,
			ZONE_Outside,
			ZONE_Inside,
			ZONE_Perimeter,
			ZONE_SetCenter,
			ZONE_Rotate,
			};
		//! Tool settings
		MARKER m_marker;
		UINT16 m_markersize;
		double m_minradius;						//!< Minimum radius
		double m_maxradius;						//!< Maximum radius, 0 if no maximum
		bool m_anglevisible;					//!< Visibility of rotation angle indicator
		//! Tool position values
		DPOINT2D m_center;						//!< Center
		double m_radius;
		double m_angle;
		//! Values used during interaction
		ZONE m_CursorZone;						//!< Current cursor zone
		DPOINT2D m_prevpt;						//!< Previous point during button-down

		//! Manual Circle Entry form forward declaration
		class MANUAL_CIRCLE;
		friend class MANUAL_CIRCLE;

		//! Methods
		void AdjustTool (const DPOINT2D&, bool firsttime=false);
		void ComputeCursorZone (const DPOINT2D&);
		void StartTool (const DPOINT2D&);
		void UpdateCursor (const DPOINT2D&) const;

		//! TOOL_BASE overrides.
		virtual MANUAL_BASE* v_CreateManualBase ();
		virtual void v_DoDraw ();
		virtual void v_DoMove (double dx, double dy, ADJUSTFLAGS flags);
		virtual int v_GenerateRegion (REGION2D& region) const;
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLCIRCLE_H
