/**
 * \file mi32/lablstyl.h
 * \brief Definitions of LABELSTYLE
 *
 * \if NODOC
 * $Id: lablstyl.h_v 1.19 2011/12/23 14:50:53 mju Exp $
 *
 * $Log: lablstyl.h_v $
 * Revision 1.19  2011/12/23 14:50:53  mju
 * Move labelframeparms to own hdr.
 *
 * Revision 1.18  2007/05/08 14:13:30  dwilliss
 * took out unnecessary forward decl of sml context
 *
 * Revision 1.17  2006/09/18 17:55:00  scowan
 * Adjusted for new enums and pattern class names.
 *
 * Revision 1.16  2006/09/15 15:25:47  mju
 * Serialiser changes.
 *
 * Revision 1.14  2004/10/14 16:58:26  dwilliss
 * Added UsesTransparency and UsesAntialiasing methods
 *
 * Revision 1.10  2003/10/01 22:37:46  dwilliss
 * Added margin and arrow members
 *
 * Revision 1.9  2003/09/23 13:11:19  mju
 * Call base copy ctor in labelstyle copy ctor.
 *
 * Revision 1.1  2003/06/18 22:45:00  dwilliss
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_LABLSTYL_H
#define	INC_MI32_LABLSTYL_H

#ifndef	INC_MI32_LABELFRAMEPARMS_H
	#include <mi32/labelframeparms.h>
#endif

#ifndef	INC_MI32_SERIALIZERBASE_H
	#include <mi32/serializerbase.h>
#endif

#ifndef INC_RVC_STYLEPATTERNBITMAP_H
	#include <rvc/stylepatternbitmap.h>
#endif

#ifndef INC_RVC_STYLEPATTERNHATCH_H
	#include <rvc/stylepatternhatch.h>
#endif

#ifndef INC_RVC_STYLEPATTERNLINE_H
	#include <rvc/stylepatternline.h>
#endif

#ifndef INC_MI32_MG2ENUMS_H
	#include <mi32/mg2enums.h>
#endif

//! In-memory structure for labels with frames and leaders
class LABELSTYLE : public LABELFRAMEPARMS {
	public:

		//! Default constructor
		LABELSTYLE (
			) :
			LABELFRAMEPARMS(),
			m_DesignScale(0.0),
			m_bClipUnderPass(false),
			m_BaselineFlag(MGD::BASELINE_Straight)
			{ }
		
		//! Construct from base class
		LABELSTYLE (
			const LABELFRAMEPARMS& rhs
			) :
			LABELFRAMEPARMS(rhs),
			m_DesignScale(0.0),
			m_bClipUnderPass(false),
			m_BaselineFlag(MGD::BASELINE_Straight)
			{ }
		
		//! Copy constructor.
		LABELSTYLE (									
			const LABELSTYLE& rhs
			): LABELFRAMEPARMS(rhs) { MyCopy(rhs); }

		//! Destructor.
		~LABELSTYLE() {}
		
		//! Assignment 
		LABELSTYLE& operator= (					
			const LABELSTYLE& rhs
			) { if (this != &rhs) { MyCopy(rhs); } return (*this); }

		MGD::BASELINE GetBaselineFlag (
			) const { return m_BaselineFlag; }

		const RVC::STYLEPATTERN_LINE& GetBorderPattern (
			) const { return m_BorderPattern; }

		bool GetClipUnderPass (
			) const { return m_bClipUnderPass; }

		double GetDesignScale (
			) const { return m_DesignScale; }

		const RVC::STYLEPATTERN_BASE& GetFramePattern (
			) const {
			if (GetFrameStyle().StyleFlags & STYLEFLAG_UseHatchPatt) return m_FramePatternH;
			return m_FramePatternB;
			}

		const RVC::STYLEPATTERN_LINE& GetLeaderPattern (
			) const { return m_LeaderPattern; }

		const TEXTSTYLE& GetTextStyle (
			) const { return m_TextStyle; }

		void SetBaselineFlag (
			MGD::BASELINE flags
			) { m_BaselineFlag = flags; }

		void SetBorderPattern (
			const RVC::STYLEPATTERN_LINE& pattern
			) { m_BorderPattern = pattern; }

		void SetClipUnderPass (
			bool state
			) { m_bClipUnderPass = state; }

		void SetDesignScale (
			double scale
			) { m_DesignScale = scale; }

		void SetFramePattern (
			const RVC::STYLEPATTERN_BITMAP& pattern
			) { m_FramePatternB = pattern; }

		void SetFramePattern (
			const RVC::STYLEPATTERN_HATCH& pattern
			) { m_FramePatternH = pattern; }

		void SetLeaderPattern (
			const RVC::STYLEPATTERN_LINE& pattern
			) { m_LeaderPattern = pattern; }

		void SetTextStyle (
			const TEXTSTYLE& style
			) { m_TextStyle = style; }

		//! Returns true if the text style is set to do antialiasing
		bool UsesAntialiasing (
			) const { return (m_TextStyle.UsesAntialiasing()); }

		//! Returns true any part of the style (text, frame or leader) uses
		//! transparency.
		//! Does not account for antialiasing text.
		bool UsesTransparency (
			) const { return (m_TextStyle.UsesTransparency() || LABELFRAMEPARMS::UsesTransparency()); }

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		TEXTSTYLE m_TextStyle;
		double m_DesignScale;
		RVC::STYLEPATTERN_BITMAP m_FramePatternB;
		RVC::STYLEPATTERN_HATCH m_FramePatternH;
		RVC::STYLEPATTERN_LINE m_BorderPattern;
		RVC::STYLEPATTERN_LINE m_LeaderPattern;
		bool m_bClipUnderPass;
		MGD::BASELINE m_BaselineFlag;
		friend class SML_LABELSTYLE;	// SML Implementation 
		
		//! Helper method for copying.
		void MyCopy (									
			const LABELSTYLE& rhs
			);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

PREVENT_MEMFUNC(LABELSTYLE)

#endif	// INC_MI32_ELEMSTYL_H
