/**
 * \file mxdcogo.h <mi32/mxdcogo.h>
 * \brief Definitions for COGO dialog
 *
 * \if NODOC
 * $Id: mxdcogo.h_v 1.3 2003/09/15 13:49:56 fileserver!dwilliss Exp $
 *
 * $Log: mxdcogo.h_v $
 * Revision 1.3  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2000/06/29 18:05:38  sparsons
 * Genitor documentation.
 *
 * Revision 1.1  1999/05/06 22:17:20  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MXDCOGO_H
#define	INC_MI32_MXDCOGO_H

#ifndef	INC_MI32_MXDIALOG_H
	#include <mi32/mxdialog.h>
#endif

/*
**
**	COGO Callback reasons
**
*/


#define MxdCOGO_NewInstrument	0x00000001	//!<  when new instrument setup specified 
#define MxdCOGO_Rotate			0x00000002	//!<  when data rotation specified 
#define MxdCOGO_Shift			0x00000004	//!<  when shift specified 
#define MxdCOGO_AddLine			0x00000008	//!<  when adding line 
#define MxdCOGO_AddPoint		0x00000010	//!<  when new point added 
#define MxdCOGO_AddCircle		0x00000020	//!<  when adding circle 
#define MxdCOGO_AddArc			0x00000040	//!<  when adding arc 
#define MxdCOGO_AddPolygon		0x00000080	//!<  when adding polygons 
#define MxdCOGO_AddElement		0x000000F8	//!<  when adding any element 


struct MxdCOGOCallbackStruct {
	UINT32 reason;
	MXDHANDLE dhandle;
	DOUBLE param;								//!<  parameter for rotation or shifting 
	DOUBLE radius;								//!<  for addiding circles 
	DPOINT2D	center;							//!<  center point of the circle 
	DOUBLE startangle; 						//!<  arc definition 
	DOUBLE endangle;
	DPOINT3D *Line;
	INT32 NumPoints;
	int ElemType;
	};

#if defined(__cplusplus)
extern "C" {
#endif

//! Add callback function to the COGO dialog.
int MxdCOGOAddCallback (
	MXDHANDLE handle,											//!< Dialog handle
	void (*func)(MxdCOGOCallbackStruct*, void*),		//! Callback function
	void *udata,												//!< User data to pass into callback
	UINT32 reason												//!< Reason for callback
	);

//! Close COGO dialog.
int MxdCOGOClose (
	MXDHANDLE dhandle							//!< Dialog handle
	);

//! Create COGO dialog.
int MxdCOGOCreate (
	Widget pwidget,							//!< Parent widget
	MXDHANDLE *dhandle,						//!< Handle returned
	char *dialogname,							//!< Dialog name override 
	int relposn,								//!< Position relative to parent's shell (0=none, -1=use previous)
	int dialogflags							//!< Dialog flags
	);

//! Destroy COGO dialog.
int MxdCOGODestroy (
	MXDHANDLE dhandle							//!< Dialog handle
	);

//! Check if dialog is open.
int MxdCOGOIsOpen (
	MXDHANDLE dhandle							//!< Dialog handle
	);

//! Open COGO dialog.
int MxdCOGOOpen (
	MXDHANDLE dhandle							//!< Dialog handle
	);

//! Remove callback.
void MxdCOGORemoveCallback (
	MXDHANDLE handle,											//!< Dialog handle
	void (*func)(MxdCOGOCallbackStruct*, void*),		//! Callback function
	void *udata,												//!< User data to pass into callback
	UINT32 reason												//!< Reason for callback
	);

#if defined(__cplusplus)
}
#endif

#endif	//!< INC_MI32_MXDCOGO_H

