/**
*	mi32/saverest.h - Save / Restore value class
*
*	\if NODOC
*	$Id: saverest.h_v 1.3 2003/09/15 13:49:56 fileserver!dwilliss Exp $
*
*	$Log: saverest.h_v $
*	Revision 1.3  2003/09/15 13:49:56  fileserver!dwilliss
*	Doxygen
*
*	Revision 1.2  2003/04/10 16:21:50  dwilliss
*	Don't use _T in templates.  Mac's ctype.h defines a global _T
*
*	Revision 1.1  2001/10/08 19:26:27  scowan
*	Initial revision
*
*	\endif
**/

#ifndef INC_MI32_SAVEREST_H
#define INC_MI32_SAVEREST_H

//! Class to store the original value of the object, possibly set the object to a new value,
//! and restore the object to the original value when instance goes out of scope

template <class _CT> 
class SAVERESTORE {
public:

	// CONSTRUCTION / DESTRUCTION
	
	//! Constructor to save original values
	explicit SAVERESTORE (
		_CT& item
		) : 
		m_item(item), 
		m_origval(item) 
		{}
		
	//! Constructor to save original value and change to a new value
	SAVERESTORE (
		_CT& item,
		_CT tempval
		) : 
		m_item(item), 
		m_origval(item) 
		{ 
		item = tempval;
		}
		
	//! Destructor to revert back to the original value
	~SAVERESTORE (
		) { 
		m_item = m_origval;
		}
		
private:
	#ifndef GENERATING_DOXYGEN_OUTPUT
	_CT& m_item;
	_CT m_origval;
	#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif		//!< INC_MI32_SAVEREST_H
