/**
 * \file scrnunit.h <mi32/scrnunit.h>
 * \brief Definitions for SCREENUNIT constants and functions
 *
 * \if NODOC
 * $Id: scrnunit.h_v 1.13 2003/10/17 15:48:45 mju Exp $
 *
 * $Log: scrnunit.h_v $
 * Revision 1.13  2003/10/17 15:48:45  mju
 * Fix comment.
 *
 * Revision 1.12  2003/09/25 21:29:07  dwilliss
 * Don't capitalize enum any more. Genitor needed it, doxygen doesn't like it.
 *
 * Revision 1.11  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.10  2003/02/07 14:35:44  mju
 * Fix Mac warning.
 *
 * Revision 1.9  2003/01/23 22:37:01  mju
 * Add SCREENUNIT_Millimeters.
 *
 * Revision 1.8  2002/10/10 17:33:52  mju
 * Add GetScreenPixelSizeMM.
 *
 * Revision 1.7  2001/12/21 15:34:59  dwilliss
 * Have to include stddefns.h or ENUM not defined
 *
 * Revision 1.6  2001/12/20 21:47:19  mju
 * Enum docs.
 *
 * Revision 1.5  2001/09/04 20:36:30  mju
 * Use void* for DC so don't need defn.
 *
 * Revision 1.4  2001/09/04 20:22:48  mju
 * Add MwinGetPixelSizeMM().
 *
 * Revision 1.3  2001/09/04 15:11:37  mju
 * Change SCREENUNIT to enum.
 * Add ConvertScreenUnits() which doesn't require pixelsize.
 *
 * Revision 1.2  2000/10/12 21:49:36  msmith
 * Genitor documentation.
 *
 * Revision 1.1  1999/04/28 14:22:25  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_SCRNUNIT_H
#define	INC_MI32_SCRNUNIT_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

//! Screen unit enumeration values.
//! These values are stored in files and thus must not change.
enum SCREENUNIT {
	SCREENUNIT_Inches =						0,
	SCREENUNIT_Millimeters =				1,
	SCREENUNIT_mm =							SCREENUNIT_Millimeters,
	SCREENUNIT_Points =						2,
	SCREENUNIT_Pixels =						3,
	SCREENUNIT_1OverInches  =				4,
	SCREENUNIT_MetersAtCurrentScale =	5,
	SCREENUNIT_Arbitrary =					6,
	SCREENUNIT_Object =						7,
	SCREENUNIT_COUNT =						8
	};


//! Convert SCREENUNIT value using given "pixel size".
//! @return Converted value
double ConvertScreenUnits (
	double val,									//!< Input value
	double mmPerPixel,						//!< Millimeters per pixel
	double MapScale,							//!< Map scale
	int iunit,									//!< Input units (SCREENUNIT_...)
	int ounit									//!< Output units (SCREENUNIT_...)
	);

//! Convert SCREENUNIT value using pixel size determined from screen.
//! @return Converted value
double ConvertScreenUnits (
	double val,									//!< Input value
	int iunit,									//!< Input units (SCREENUNIT_...)
	int ounit,									//!< Output units (SCREENUNIT_...)
	double MapScale = 1.0					//!< Map scale
	);

//! Get screen pixel size in millimeters.
double GetScreenPixelSizeMM (
	);

#ifdef WIN32_NATIVE
//! Get screen pixel size in millimeters using device context (WIN32_NATIVE only).
double MwinGetPixelSizeMM (
	void *hdc										//!< Windows device context, 0 for default
	);
#endif

#endif	// INC_MI32_SCRNUNIT_H
