#ifdef WIN32
#pragma message(__FILE__ "(2) : warning C9999: Use of sml.h is deprecated, use forward declarations or individual sml/... headers instead")
#endif

/******************************************************************************
 *
 * \file mi32/sml.h
 * \brief SML -- Spatial Manipulation Language
 *
 * SML is used in TNTmips for both a scripting language and for database
 * query operations.
 *
 * \if NODOC
 * $Log: sml.h_v $
 * Revision 1.688  2011/10/12 14:23:47  mju
 * Change mxQueryOpenScript to use int for usage so don't need rvc script hdr.
 *
 * Revision 1.687  2010/09/28 15:59:30  bcolombe
 * SML_ExecuteMain needs productcode
 *
 * Revision 1.686  2009/11/25 20:57:56  mju
 * Fix typo.
 *
 * Revision 1.685  2009/11/25 18:43:10  mju
 * Doxygen fixes.
 *
 * Revision 1.684  2009/02/24 23:39:49  bcolombe
 * Added QFLAG_SuppressToolbar
 *
 * Revision 1.683  2008/11/10 23:33:51  scowan
 * Moved qtype flags to private include file.
 *
 * Revision 1.682  2008/11/10 20:40:37  scowan
 * Removed duplicated defines and sml flag include file.
 *
 * Revision 1.681  2008/11/07 23:23:46  scowan
 * Removed context include file.
 *
 * Revision 1.680  2008/11/07 17:18:49  scowan
 * Removed more unnecessary include files.
 *
 * Revision 1.679  2008/11/06 20:29:17  scowan
 * Removed another include file.
 *
 * Revision 1.678  2008/11/05 18:09:42  scowan
 * Reduced the number of files included with this file.
 *
 * Revision 1.677  2008/11/04 00:16:44  scowan
 * Removed about half of the files this file was including.
 *
 * Revision 1.676  2008/11/03 16:59:32  scowan
 * Moved getParent to context, removed unused setParent methods.
 *
 * Revision 1.675  2008/11/03 16:50:53  mju
 * Move sml_AppInit to context hdr.
 *
 * Revision 1.674  2008/11/03 16:21:30  scowan
 * Removed sml specific infinity macros.
 *
 * Revision 1.673  2008/11/03 16:01:37  scowan
 * Removed calls to init carto script because the function is not needed anymore
 *
 * Revision 1.672  2008/10/31 15:44:19  scowan
 * *** empty log message ***
 *
 * Revision 1.671  2008/10/30 18:03:39  scowan
 * Removed readScript macro.
 *
 * Revision 1.670  2008/10/30 17:55:15  scowan
 * MOved exported function to context include file and removed unused qflag.
 *
 * Revision 1.669  2008/10/23 20:37:31  scowan
 * Some parts moved to sml namespace include files.
 *
 * Revision 1.668  2008/10/16 17:14:29  scowan
 * Moved most context creation code to context specific include file.
 *
 * Revision 1.667  2008/10/15 21:31:23  mju
 * Remove sml/errors hdr.
 *
 * Revision 1.666  2008/10/15 13:50:40  mju
 * Display deprecation warning when include.
 *
 * Revision 1.664  2007/05/08 15:44:55  dwilliss
 * SML namespace
 *
 * Revision 1.663  2007/05/07 15:46:29  dwilliss
 * breaking up into multiple include files
 *
 * Revision 1.662  2007/05/04 23:03:12  dwilliss
 * moved a bunch of stuff to sml include subdirectory
 *
 * Revision 1.661  2007/05/03 15:48:45  dwilliss
 * took out unused method
 *
 * Revision 1.660  2007/05/03 14:55:04  dwilliss
 * fixed typo
 *
 * Revision 1.659  2007/04/20 17:02:39  drb
 * Added some classnames
 *
 * Revision 1.658  2007/04/18 22:59:29  dwilliss
 * added method to get script objitem
 *
 * Revision 1.657  2007/04/18 20:50:57  dwilliss
 * removed stuff that's been deprecated for a while
 * deprecated more old stuff
 *
 * Revision 1.656  2007/04/17 22:48:14  bcolombe
 * Added 2 more pipeline classnames
 *
 * Revision 1.655  2007/04/16 20:18:57  bcolombe
 * Added more pipeline class names
 *
 * Revision 1.654  2007/04/13 15:28:53  bcolombe
 * Added SIG_i_pv_pv_b
 *
 * Revision 1.653  2007/04/13 15:16:41  mju
 * Don't need errcodes hdr at all as included by fixedstr hdr.
 *
 * Revision 1.652  2007/04/13 15:06:14  mju
 * Exclude errcodes for no_deprecated as not needed by this header.
 *
 * Revision 1.651  2007/04/13 14:57:54  bcolombe
 * Added more pipeline classnames
 *
 * Revision 1.650  2007/04/12 20:14:38  bcolombe
 * Added some new classes and SIGs
 *
 * Revision 1.649  2007/04/12 15:59:13  bcolombe
 * Added CBPARMS classname
 *
 * Revision 1.648  2007/04/11 22:19:13  bcolombe
 * Added new sig SIG_i_pv_d_d
 * Added new classnames
 *
 * Revision 1.647  2007/04/10 22:45:58  bcolombe
 * Added new SIG_i_i_i_d
 *
 * Revision 1.646  2007/04/09 16:55:36  drb
 * Removed and deprecated some old info stuff.
 *
 * Revision 1.645  2007/04/09 16:36:18  scowan
 * Removed getGenInfo.
 *
 * Revision 1.644  2007/04/05 21:32:21  dwilliss
 * changed export defines
 * get rid of unnamed include file
 *
 * Revision 1.643  2007/04/05 20:32:47  dwilliss
 * changed export defines
 *
 * Revision 1.642  2007/04/05 14:56:57  bcolombe
 * Added new sig: SIG_i_i_d_d
 *
 * Revision 1.641  2007/04/05 13:22:54  mju
 * Remove some unused deprecated stuff.
 *
 * Revision 1.640  2007/04/05 13:14:39  mju
 * Exclude setInode unless building rvcsys.
 *
 * Revision 1.639  2007/04/05 13:09:08  mju
 * Only define Get/SetGenInfo if building rvcsys as not used outside DLL.
 *
 * Revision 1.638  2007/03/29 14:38:48  drb
 * Removed b----Running as it is now replaced within mstatus
 *
 * Revision 1.637  2007/03/26 19:58:06  scowan
 * Added include file that rvc defns was getting for us.
 *
 * Revision 1.636  2007/03/20 13:57:52  scowan
 * Marked items as can be removed.
 *
 * Revision 1.635  2007/03/19 13:28:56  mju
 * Remove unused sml_readScript variants.
 *
 * Revision 1.634  2007/03/14 21:53:53  mju
 * Remove dbquery_EditUC.
 *
 * Revision 1.633  2007/03/13 20:48:29  dwilliss
 * On Exit Data no longer stored in old linked list structure
 *
 * Revision 1.632  2007/03/13 17:59:14  dwilliss
 * Got rid of an obsolete structure
 *
 * Revision 1.631  2007/03/09 16:56:56  dwilliss
 * Moved some private stuff to mi32p/dbqueryp.h
 *
 * Revision 1.630  2007/03/05 20:46:16  scowan
 * Moved database header to private include file.
 *
 * Revision 1.629  2007/03/05 17:10:49  scowan
 * Set database callback struct to rvc namespace.
 *
 * Revision 1.628  2007/02/12 18:16:34  drb
 * Fixed typo
 *
 * Revision 1.627  2007/02/09 18:39:47  dwilliss
 * removed prototype for an obsolete function
 *
 * Revision 1.626  2007/02/09 17:02:39  scowan
 * Removed commented out references to legacy raster info structure.
 *
 * Revision 1.625  2007/02/08 23:57:07  drb
 * Removing references to RVC-RAST-INFO
 *
 * Revision 1.624  2007/02/05 18:12:19  drb
 * Added DBASE_TINxx defines
 *
 * Revision 1.623  2007/02/02 23:13:19  drb
 * Added some class names
 *
 * Revision 1.622  2007/01/31 18:10:36  dwilliss
 * Moved SMLUSAGE enum to smlbase.h
 *
 * Revision 1.621  2007/01/29 15:49:51  drb
 * DBASE_RASTER
 *
 * Revision 1.620  2007/01/26 23:48:22  drb
 * Added methods to get a the string(s) from a given value from an SMLFLAGS array
 *
 * Revision 1.619  2007/01/26 22:13:57  drb
 * Added some classnames
 *
 * Revision 1.618  2007/01/26 16:42:06  drb
 * Added several class names.
 * Fixed Deletable autoclass constructor to return null since no default
 * construtor is available.
 *
 * Revision 1.617  2007/01/24 16:12:10  drb
 * Add RVC_ELEMENT
 *
 * Revision 1.616  2007/01/22 14:20:27  mju
 * Remove colormap hdr as not needed by this hdr.
 *
 * Revision 1.615  2007/01/17 21:40:00  dwilliss
 * Added some documentation functions
 *
 * Revision 1.614  2007/01/09 16:12:49  dwilliss
 * defined SML_AppInit
 *
 * Revision 1.613  2007/01/09 15:28:51  dwilliss
 * Something was still using the method I renamed, so I had to create a stub
 * to call it.
 *
 * Revision 1.612  2007/01/09 14:42:19  dwilliss
 * Renamed a static method -- allowing more than one funclist to be pre-
 * registered now.
 *
 * Revision 1.611  2007/01/03 15:55:11  bcolombe
 * GPSDBASE
 *
 * Revision 1.610  2007/01/03 15:24:31  bcolombe
 * Added GPSDBASE
 *
 * Revision 1.609  2006/12/29 18:46:20  bcolombe
 * Added new SIG
 *
 * Revision 1.608  2006/12/28 22:11:12  dwilliss
 * Some work on console methods
 *
 * Revision 1.607  2006/12/26 18:54:13  dwilliss
 * Got rid of deprecated methods no longer used
 *
 * Revision 1.606  2006/12/20 17:28:41  dwilliss
 * Got rid of deprecated method
 *
 * Revision 1.605  2006/12/19 15:58:49  dwilliss
 * got rid of a deprecated template no longer used
 *
 * Revision 1.604  2006/12/18 17:04:54  dwilliss
 * Removed some deprecated SMLSTRING methods.  Deprecated a few more.
 *
 * Revision 1.603  2006/12/13 22:57:16  scowan
 * Removed unused dbquery legacy functions.
 *
 * Revision 1.602  2006/12/07 19:03:09  mju
 * Fix comments so not on deprecation list.
 *
 * Revision 1.601  2006/12/07 16:14:01  dwilliss
 * Working toward changing how functions are registered
 *
 * Revision 1.600  2006/11/22 23:51:54  dwilliss
 * Started adding Scope rule support
 *
 * Revision 1.599  2006/11/17 22:46:36  dwilliss
 * deprecated some old functions
 *
 * Revision 1.598  2006/11/17 22:15:12  dwilliss
 * added a method
 *
 * Revision 1.597  2006/11/17 20:35:34  dwilliss
 * Added method to register minimal function list.
 *
 * Revision 1.596  2006/11/17 17:44:51  dwilliss
 * added new private method
 *
 * Revision 1.595  2006/11/16 22:26:33  dwilliss
 * Fixed problem in autoclass deletable template
 *
 * Revision 1.594  2006/11/16 20:28:59  dwilliss
 * Changed how class arrays are stored
 *
 * Revision 1.593  2006/10/26 16:05:11  drb
 * Added deprecation comments to QTYPE
 *
 * Revision 1.592  2006/10/26 13:54:35  dwilliss
 * added SMLFUNC_Deprecated
 *
 * Revision 1.591  2006/10/24 13:41:02  dwilliss
 * added some prototypes
 *
 * Revision 1.590  2006/10/19 22:29:12  dwilliss
 * some class names
 *
 * Revision 1.589  2006/10/16 17:24:19  dwilliss
 * Added SMLCONTEXT::IsRunning()
 *
 * Revision 1.588  2006/10/16 16:30:17  dwilliss
 * added SMLSYMBOL::IsPermanent()
 *
 * Revision 1.587  2006/10/13 20:56:46  drb
 * Added NULLMASK class name
 * Dave W's previous change was unprotecting SetTextColor****(), StartTitle()
 * and EndTitle().
 *
 * Revision 1.586  2006/10/13 20:52:29  dwilliss
 * I don't know :-)
 *
 * Revision 1.585  2006/10/05 21:11:45  mju
 * Remove textid hdr, not needed by this hdr.
 *
 * Revision 1.584  2006/09/18 16:09:28  dwilliss
 * put back an include file that I still needed
 *
 * Revision 1.583  2006/09/18 15:30:05  dwilliss
 * Eliminated some old deprecated methods
 *
 * Revision 1.582  2006/09/14 21:50:54  mju
 * Change message to not use word "error" so easier to search for.
 *
 * Revision 1.581  2006/09/14 17:04:55  mju
 * Include smlbase hdr.
 *
 * Revision 1.580  2006/09/14 14:35:31  dwilliss
 * Added SMLRASTEROBJECT::GetDataType()
 *
 * Revision 1.579  2006/09/13 20:42:11  dwilliss
 * some deprecated stuff
 *
 * Revision 1.578  2006/09/13 15:58:27  dwilliss
 * Added a DeleteExisting to SMLOBJECT
 *
 * Revision 1.577  2006/09/12 22:22:14  dwilliss
 * changed some methods to only be defined if building the dll so that they'll still
 * be exported, but nothing that tries to use them will compile.
 *
 * Revision 1.576  2006/09/07 17:55:55  scowan
 * Removed unused mxqueryeditscript functions.
 *
 * Revision 1.575  2006/09/07 17:41:38  dwilliss
 * Changed SMLCALLBACKSTRUCT to have an RVC::OBJITEM
 *
 * Revision 1.574  2006/09/07 17:07:20  dwilliss
 * prevent memset of SMLCALLBACKSTRUCT
 *
 * Revision 1.573  2006/09/01 17:43:37  dfriberg
 * Add EXIF
 *
 * Revision 1.572  2006/09/01 15:52:54  scowan
 * Removed unnecessary header, added forward decl.
 *
 * Revision 1.571  2006/08/17 16:44:14  dwilliss
 * some deprecation work
 *
 * Revision 1.570  2006/08/16 23:02:39  dwilliss
 * *** empty log message ***
 *
 * Revision 1.569  2006/08/02 17:47:49  dwilliss
 * added a context creation method that takes an RVC::OBJITEM instead of
 * filehandle/inode
 *
 * Revision 1.568  2006/07/27 19:39:21  dwilliss
 * fixed a few deprecated things to still be defined if building rvcsys so that
 * the implementation will compile and be exported
 *
 * Revision 1.567  2006/07/25 21:45:06  dwilliss
 * Implemented AddRef/Release on SMLCONTEXT.  Deprecated Destroy()
 *
 * Revision 1.566  2006/07/20 22:19:54  dwilliss
 * Added SetParmError, depredated <DELETED>
 *
 * Revision 1.565  2006/07/20 17:46:08  dwilliss
 * eliminated the temp mem pool
 *
 * Revision 1.564  2006/07/18 18:02:01  dwilliss
 * support for function overloads
 *
 * Revision 1.563  2006/07/14 14:30:30  dwilliss
 * smlgeostruct no longer has private ctor/dtor.  Go ahead and use new/delete
 * outside the dll.
 *
 * Revision 1.562  2006/07/13 19:52:56  dwilliss
 * Some major changes coming soon
 *
 * Revision 1.561  2006/07/07 17:52:17  dwilliss
 * Really deprecated something old
 *
 * Revision 1.560  2006/07/06 22:27:28  dwilliss
 * deprecated some stuff
 *
 * Revision 1.559  2006/07/06 20:42:38  drb
 * Removed CADINFO stuff
 *
 * Revision 1.558  2006/06/30 22:24:09  dwilliss
 * got rid of a member we don't need any more
 *
 * Revision 1.557  2006/06/16 19:35:50  dwilliss
 * Addes SMLSTRINGHASH
 *
 * Revision 1.556  2006/06/15 17:08:19  drb
 * Added BITSET_OPERARTOR
 *
 * Revision 1.555  2006/06/14 21:24:14  dwilliss
 * Added a method to SMLSYMBOL to get the "alternate" name
 *
 * Revision 1.554  2006/06/12 19:39:07  dwilliss
 * Added a private method to SMLSYMBOL
 *
 * Revision 1.553  2006/06/09 23:28:51  drb
 * Added some SMLCLASSNAMES
 *
 * Revision 1.552  2006/06/09 19:10:39  drb
 * Add constructor and destructor to SMLFILEINFO struct
 *
 * Revision 1.551  2006/05/08 15:59:44  drb
 * Added OBJECTINFO class name
 *
 * Revision 1.550  2006/04/07 21:29:27  dwilliss
 * get rid of prototype of a function that was no longer used
 *
 * Revision 1.549  2006/04/07 19:08:21  dwilliss
 * changed callback struct to be derived from MICALLBACKLIST::ANYCALLBACKSTRUCT
 *
 * Revision 1.548  2006/04/06 15:34:52  dwilliss
 * fixed a prototype that didn't specify return type.  Should have been void.
 *
 * Revision 1.547  2006/04/05 20:06:26  mju
 * Use win32 instead of winnt.
 *
 * Revision 1.546  2006/03/31 18:54:53  drb
 * Added Clear method to SML*HASH classes
 *
 * Revision 1.545  2006/03/07 18:14:06  dwilliss
 * *** empty log message ***
 *
 * Revision 1.544  2006/02/08 21:05:26  dwilliss
 * fixed to compile with g++ 4.0
 *
 * Revision 1.543  2006/01/06 21:41:43  dfriberg
 * Remove deprecated SML ImpExp functions
 *
 * Revision 1.542  2006/01/06 19:08:31  dwilliss
 * un-ifdef-d out something
 *
 * Revision 1.541  2006/01/05 21:45:27  mju
 * Exclude deprecated stuff when compiling for no_deprecated.
 *
 * Revision 1.540  2006/01/05 17:05:12  dwilliss
 * modified autoclass new's copy ctor to cope with in-place copy
 *
 * Revision 1.539  2006/01/03 17:46:18  dwilliss
 * added a method for internal use
 *
*
*******************************************************************************
*		Defining built-in SML functions:
*
* I	SML built-in functions come in 3 flavors: void, double, and string.
*		Functions should take 3 parameters:
*
*		static void function(numargs, arg, argtype)
*			int numargs;		 # of arguments
*			SMLARG *arg;		 Array of arguments
*			UINT8 *argtype;	 STYPE_Xxxx -- See <mi32/dbquery.h>
*
*		All built-in functions should be declared as static.
*
* II	To install a group of built-in functions, do the following:
*
*		A. For each function:
*			1. Prototype the function.
*			2. Define a static array of SMLPARM.
*				a. Give one entry for each parameter.
*				b. Signal the end of the array by defining a paramater
*					with NULL for a name.
*				c. If you request the type STYPE_AnyVal, you may be passed
*					either an STYPE_Constant or STYPE_String.  Check the
*					argtype array in your function and use the parm accordingly.
*				d. If you don't care how many parameters your function gets,
*					name the last parameter (not counting the NULL) "..."
*					This will match 0 or more parameters of the type requested.
*				e. Some parameters can be marked as optional, but then it's
*					up to you to figure out which ones you got.  Put optional
*					parameters at the end.
*
*		B.	Define a static array of SMLFUNC, one entry per function.
*			the type must be either STYPE_BProcedure (void function),
*			STYPE_Function (function returning double), or STYPE_BSFunction
*			(Function returning SMLSTRING).  You will have to cast the
*			function pointer to (VOIDFUNC), or most compilers will hate you.
*			Signal the end of the array by defining a function with NULL as
*			a name.
*
* III	In the function:
*
*		A. Don't free anything passed to you -- ever!
*		B. The arg array passed is an array of unions.  Use the field that
*			matches the type of data you requested.  Rasters are passed as
*			symbols.
*		C. If you encounter an error in your function and want to halt the
*			SML script, call one of the following, then return:
*			1. context->SetErrorCode(int error)
*				Works for any error code listed in <mi32/errcodes.ref>
*			2. context->SetParmError(argnum, int error)
*				Works for SML_EXxxxx errors listed below.  Pass the
*				0-based argument number of the offending argument.
*				The error parameter has a default value of EBadFuncParm
*
* IV.	Actual install.
*		A.	Get the SML context (returned from SML_InitExt)
*		B. Call the following:
*				SML_InstallFunctions(context, class, func_list)
*					void *context;		(returned from SML_InitExt)
*					char *class;		(example: "test")
*					SMLFUNC *func_list; (see above)
*
*			Function returns 0 or an error code
*		C. In the SML editor, the functions will be shown on the
*			"Symbols..." dialog in a group called (in this example)
*			"function_test".  These names are looked up from textres
*
 * \endif
*/
/*****************************************************************************/

#ifndef INC_MI32_SML_H
#define INC_MI32_SML_H

/*#undef SML_API_CONFORMANCE		Can't have until I can check this include file in */

#ifndef SML_API_CONFORMANCE
#define SML_API_CONFORMANCE 0
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_MCB_H
	#include <mi32/mcb.h>
#endif

#ifndef	INC_MI32_CRYPTO_H
	#include <mi32/crypto.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
#endif

// Forward declarations
#ifndef GENERATING_DOXYGEN_OUTPUT
class TRANS2D_MAPGEN;
class MISTRING;
class FILEPATH;
struct MENUITEM;
namespace MGD {
	class CONTEXT;
	}
namespace RVC {
	class OBJITEM;
	}
namespace SML {
	class EDITOR;
	}
#endif
/***************************************************************************/

/* Flags for MxQueryCreate() */
#define QFLAG_SML					0x0001
#define QFLAG_NoShell			0x0002
#define QFLAG_AutoRun			0x0004
#define QFLAG_NoWait				0x0008
#define QFLAG_SingleTable		0x0010
#define QFLAG_PinMap				0x0020
#define QFLAG_DispLayer			0x0040	//!<  Used to edit script of and Mdisp layer -- same as SML, but no Run
#define QFLAG_Formula			0x0080
#define QFLAG_NonModal			0x0100
#define QFLAG_NoEdit				0x0400
#define QFLAG_NoConsole			0x0800
#define QFLAG_SetExitOnQuit	0x1000	//!<  So MxMainLoopUntilExitSet() will drop out
#define QFLAG_NoViewIcon		0x2000	//!<  For SML Toolbar in tntdisp
#define QFLAG_ScriptNoFile		0x4000	//!< Editor has no control over the file. Gets "Script" menu with open/save as/close only
#define QFLAG_FileExtnSML		0x8000	//!< Use ".sml" for file extn instead of ".qry"
#define QFLAG_Filename			0x10000	//!< string passed to SML_EditScript is really a filename (passed on command line)
#define QFLAG_SupressToolbar	0x20000	//!< don't create toolbar for "applidat"

/*-------------------------------------------------------------------------*/
/*		Function Prototypes																	*/
/*-------------------------------------------------------------------------*/

/*:> querylow.c ----------------------------------------------*/

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SML {
//! Read a script into a unicode buffer.
//!
//! This function should be called through the SML_ReadScript() macro
//! which provides the values for the last two parameters

RVCSYSLIBEXPORT int ReadScript (
	RVC::OBJITEM& ObjItem,
	char *encoding,			//!< This must be a buffer I can fill in
	MIUNICODE *upassword,					//!< Default password to try before prompting. Can be NULL
	int (*CryptoPromptPassword) (		//! Can be NULL if we don't want to prompt
		void *data,
		MIUNICODE *password
		),
	void *PromptPasswordData,
	MCRYPTOHDR *crypto,		//!< Will be filled in if the file was encrypted
	MISTRING& ScriptStr,		//!< Script RETURNED
	int chdrsize				//!< sizeof(MCRYPTOHDR)
	);
} // end of SML namespace

#endif // GENERATING_DOXYGEN_OUTPUT

/******************************************************************************/
/* The rest of these are NOT in rvcsys.dll */
/******************************************************************************/

/*:> smlinit.c ----------------------------------------------*/

//! Convert YYYYMMDD to a string
void SML_LongToDate (
	INT32 date,
	char *dateStr
	);

/*:> qqstyle.c ----------------------------------------------*/
//! Cartoscript label drawing function
int DBQuery_DrawOptimizedLabels (
	SML::CONTEXT *context,
	ELEMTYPE ElemType,
	UINT8 *bitarray,
	INT32 NumElem
	);

//! Cartoscript setup function
int DBQuery_SetupStyleDraw (
	SML::CONTEXT *context,
	MGD::CONTEXT *ggc,
	const TRANS2D_MAPGEN& tmg,				//!< Coordinate and clipping transformation
	bool Is3D,
	SML::SYMBOL *DrawDoneSym			//!< SML symbol to indicate that drawing was done, NULL if none
	);

//! Cartoscript line setup function
int DBQuery_LineStyleSetPoints (
	SML::CONTEXT *context,
	const DPOINT2D *pts,
	INT32 NumPts,
	int CloseMode
	);

//! Cartoscript line setup function
int DBQuery_LineStyleSetGC (
	SML::CONTEXT *context,
	MGD::CONTEXT* gc,
	const TRANS2D_MAPGEN& tmg
	);

/*:> qqmgd.c ----------------------------------------------*/

//! Set the MgdGC to be used by the "Drawing" function group
//!
//! If the MgdGC is NULL, the drawing functions will just update
//! the extents, which can be retrieved by calling DBQuery_GetMgdExtents()
//!
//! @see DBQuery_GetMgdExtents
//!
void DBQuery_SetMgdDisplay (
	SML::CONTEXT *context,
	MGD::CONTEXT* ggc,
	const TRANS2D_MAPGEN& tmg,	//!< Conversion from script provided coordinates to "display" coordinates
	const RVC::OBJITEM* StyleObjItem = 0	//!< Style Object (for drawing functions that set patterns and symbols
	);

//! Get the extents of the drawing.
//!
//! Used by SML Layers in GRE.  To find the extents of a script layer,
//! call DBQuery_SetMgdDisplay() and give it NULL for a gc.  Execute
//! the script, then call DBQuery_GetMgdExtents()

int DBQuery_GetMgdExtents (
	SML::CONTEXT *context,
	DRECT2D *extents
	);

//! Return the MGD::CONTEXT being used by the "Drawing" function group
MGD::CONTEXT* DBQuery_GetCurrentMgdGC (
	SML::CONTEXT *context
	);

/*:> qqmdisp.c ----------------------------------------------*/

//! Set the GRE::LAYER for the context (for SML Layers)
//!
//! XXX Layer and View should use actual classes, not (void*)
void DBQuery_SetMdispLayer (
	SML::CONTEXT *context,
	void *layer,
	void *view,
	int ExtentsOnly
	);


/**********************************************************************/

//! Main SML Function.
//!
//! This is another main entry point for SML.
//! This variation is used by TNTDISP when an SML script is picked from
//! a toolbar.
//!
//! The InstallDisplayFunctions parameter was meant to allow you to pass
//! in a function to initialize either Mdisp or Mxdw functions from back
//! when we had both.  Now, it is ignored.
//!
//! @see SML_Main()
//! @see SML_NoEditMain()
//!
int SML_ExecuteMain (
	MDLGPARENT toplevel, 			//!< MxToplevel()
	int (*InstallDisplayFunctions)(SML::CONTEXT *), //! See comments above
	const MIUNICODE *fname, 			//!< Filename to run
	const MIUNICODE *objname,			//!< Object name of script to run (NULL if plain file)
	UINT32 flags,						//!< QFLAG_ flags
	void *encdata = NULL,				//!< Partial SML::EDITOR data for passing encoding data
	MIPRODUCTCODE productcode = MIPRODUCTCODE_Default	//!< Product code to use
	);

//! Another main entry point for SML
//!
//! @see SML()
//! @see SML_ExecuteMain()
int SML_Main (
	MDLGPARENT toplevel,
	SML::CONTEXT *context,
	const char **argv,
	int argc,
	UINT32 flags = 0
	);


//! Run an SML script without the editor window
int SML_NoEditMain (
	MDLGPARENT toplevel, 				//!< parent widget if we need to popup anything
	SML::CONTEXT *context, 				//!< SMLCONTEXT to use
	const MIUNICODE *fname, 				//!< Filename of script to run
	const MIUNICODE *objname, 			//!< RVC Object name of script (NULL if plain file)
	UINT32 flags,
	void *encdata = NULL				//!< Partial SMLEDITOR data for passing encoding data
	);

//! Class to store incoding information when running a script directly from the toolbar
namespace SML {
class ENCODINGDATA {
	public:
		ENCODINGDATA();
		~ENCODINGDATA();

		char *encoding;
		int EncodingNum;
	};
} // end of SML namespace

typedef SML::ENCODINGDATA SMLENCODINGDATA;

//! Called by TNTview when use clicks "Exit" button
//!
//! If there are SML sessions open, the user will be closed.
//! If they have changes that need to be saved, the user will be asked
//!    if they would like to save changes.  If they click "cancel"
//!    this function will return EUserCancel
//! Will return 0 if everything is closed.

int SML_CheckExit (
	MDLGPARENT parent
	);

#if !defined(WIN32_NATIVE) || defined(GENERATING_DOXYGEN_OUTPUT)
#include <X11/intrinsic.h>		// For XtCallbackProc

//! Register an XmNdestroyCallback for a Widget.
//!
//! If you want to pass an SML::CONTEXT as the callback data of a widget's
//! XmNdestroyCallback, you MUS use this function to do it.  Due to the
//! event-driven nature of X11, if you destroy the widget and the context
//! the widget's destroy callback won't get called until after the context
//! had been free'd.  By registering the callback through this wrapper, the
//! context will keep track of it and not free the context until the last
//! registered destroy callback has been called.
//!
//! If you're passing an SML::CONTEXT as the callback data of any other kind
//! of callback, it's a good idea to still register a destroy callback
//! through this function even if it does nothing.
void SML_AddWidgetDestroyCallback (
	SML::CONTEXT *context,
	Widget widget,
	XtCallbackProc func, 		//!< Function to call on XmNdestroy
	void *cbdata					//!< callback data for func (usually the context)
	);
#endif


/**********************************************************************/

//! SML / Query editor
//!
//! Note, the consturctor/destructor of this class are private.
//! You have to call MxQueryCreate() to create one.
//!
//! This class is just an opaque handle for now.
typedef SML::EDITOR SMLEDITOR;

//! Create an SML::EDITOR instance.
//!
//! Note that this function does not actually create an SML::CONTEXT or
//! any widgets.  You should not free the handle returned.
//!
//! Flags:
//! 	\li\b QFLAG_SML					Plain SML
//! 	\li\b QFLAG_DISPLAYER			GRE::LAYER_SCRIPT
//! 	\li\b QFLAG_PINAMP				GRE::LAYER_PINMAP	XXX Check class name here
//! 	\li (none of the above)		Database Query

int MxQueryCreate (
	SML::EDITOR **handle, 	//!< The handle returned.
	UINT32 flags			//!< Just stored for later use XXX What's valid?
	);

//! Close the query windows children
int MxQueryCloseChildren (
	SML::EDITOR *handle 	//!< The handle.
	);

//! Set the SML::CONTEXT and segment for an SML::EDITOR
int MxQuerySetContext (
	SML::EDITOR *handle,
	SML::CONTEXT *context,
	int segment
	);

//! Tells the editor where the script came from
//! This does NOT load the script, it just records the filename so that
//! "Save" knows where to save to.
void MxQuerySetScriptFilepath (
	SML::EDITOR *handle,
	const FILEPATH& filepath
	);
	
//! Tells the editor where the script came from
//! This does NOT load the script, it just records the objitem so that
//! "Save" knows where to save to.
void MxQuerySetScriptObject (
	SML::EDITOR *handle,
	const RVC::OBJITEM& objitem
	);
	

#ifndef WIN32_NATIVE
//! Set the widgets to use in the editor.
//!
//! Note that you have to create the widgets yourself.
//! This also adds a destroy callback to the parent widget which will
//! free the SML::EDITOR when the parent widget is destoied
int MxQuerySetWidgets (
	SML::EDITOR *handle,
	Widget parent, 		//!< Parent widget
	Widget text, 			//!< An XeText widget to edit the script in
	Widget message 		//!< Optional MxMsgLine (Pass NULL if not used)
	);
#endif


//! Add a callback to the query editor
//!
//! <b>Possible reasons:</b>
//! 	MsmlCR_Modified
//!		MsmlCR_Filename
//!
//! @see MxQueryRemoveCallback()
//!
int MxQueryAddCallback (
	SML::EDITOR *handle,
	McbCallbackFunc func,
	void *cbdata,
	UINT32 reason
	);


//! Remove a callback from the query editor
//!
//! @see MxQueryAddCallback()
//!
void MxQueryRemoveCallback (
	SML::EDITOR *handle,
	McbCallbackFunc func,
	void *cbdata,
	UINT32 reason
	);

//! Check the syntax of the script in the editor
//!
//! Returns 1 if syntax OK, 0 if not

int MxQueryCheckSyntax (
	MDLGPARENT parent,
	SML::EDITOR *handle
	);

#ifndef WIN32_NATIVE
//! Create the menu bar for an SML or database query editor.
//!
//! The type of menu bar created will depend on the flags passed to
//! MxQueryCreate()
DEPRECATED_MSG("Use MGUI::FORM_EDIT_SML") Widget MxCreateQueryMenuBar (
	Widget pwidget,
	SML::EDITOR *handle
	);
#endif


#ifndef WIN32_NATIVE
//! Return the MENUITEM array of the Insert menu.
//! I don't know why you would want this, but mdfrmula.c uses it
MENUITEM * MxQueryGetMenuInsert (
	SML::EDITOR *handle
	);
#endif

#ifndef WIN32_NATIVE
//! Return the MENUITEM array of the Syntax menu.
//! I don't know why you would want this, but mdfrmula.c uses it
MENUITEM * MxQueryGetMenuSyntax (
	SML::EDITOR *handle
	);
#endif


//! Load an SML script into the editor.
//! This version will work with scripts in RVC objects.
ERRVALUE MxQueryOpenScript (
	SML::EDITOR* editor,
	const RVC::OBJITEM& ObjItem,
   int usage								//!< RVC::SCRIPT::USAGE
	);


//! Popup dialog for editing SML/Query script.
//!
//!	FLAGS:
//!		QFLAG_SML	Editor has menus and buttons of SML instead of
//!						a database query editor.
//!		QFLAG_NonModal		The dialog created will be non modal
//!		QFLAG_ScriptNoFile  The editor has no control over the "file" containing
//!			the script.  Instead of a "File" menu, the editor will get a "Script"
//!			menu with "Open...", "Save As...", and "Close" entries only.
//!		QFLAG_Pinmap   		Special menu bar for Pin mapping
//!		QFLAG_DispLayer   	Special menu bar for SML layers in display
ERRVALUE SML_EditScript (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	MISTRING& scriptstr,				//!< Script string passed/returned
	SML::CONTEXT* smlcontext,			//!< SML context to use
	int smlsegment = 0,				//!< SML segment number
	UINT32 flags = 0					//!< Flags, see above
	);

//! Read a script into a unicode buffer.
//!
//! @return TRUE if script is encrypted, FALSE if not, < 0 if error.
//!
//! If the script is in an RVC object, it may be encrypted, if so, one of 3 things can happen (in this order):
//! \li If the "password" parameter is filled in and correct, it will be used.
//! \li If CryptoPromptPassword is not NULL, it will be called.  This function should prompt
//!			the user for a password and return it in its "password" parameter.  The CryptoPromptPassword
//!			function can return EUserCancel or 0.
//! \li The user is just out of luck.
//!
//! If the script is in a plain file, the encoding parameter should be set to an encoding string telling
//!	what kind of encoding the file is in (default of "ISO-2022" will be used if encoding is NULL).
//! There is a special case.  If encoding is "Japanese", the text will be scanned to determine if it's
//!	"JIS", "Shift-JIS", "EUC".  The encoding string passed in will be modified to hold the new encoding
//!	name.
//!
//! It is the caller's job to free the buffer returned
inline int SML_ReadScript (
	RVC::OBJITEM& ObjItem,
	MISTRING& ScriptStr,					//!< Script RETURNED
	char *encoding = 0,					//!< Encoding, may be altered
	MIUNICODE *password = 0,			//!< Default password to try before prompting, can be NULL
	int (*CryptoPromptPassword) (		//! Function to prompt user for password, can be NULL if don't want to prompt
		void *data,
		MIUNICODE *password
		) = 0,
	void *PromptPasswordData = 0,		//!< Data to pass to password prompt function
	MCRYPTOHDR *crypto = 0				//!< Will be filled in if the file was encrypted
	) {
	return (SML::ReadScript(ObjItem,encoding,password,CryptoPromptPassword,PromptPasswordData,crypto,ScriptStr,sizeof(MCRYPTOHDR)));
	}


#ifdef WIN32_MFC

ERRVALUE SMLEdit (
	MDLGPARENT parent,
	SML::CONTEXT* context,
	int segment,
	MISTRING& query,
	UINT32 flags
	);

#endif


#ifdef WIN32
#pragma warning (default:4996)	// Restore warning about deprecated declaration
#endif

#undef RVCSYSLIBEXPORT

#endif		//  INC_MI32_SML_H


