/*****************************************************************************
 *
 *	\file mi32/textmetrics.h
 *	\brief Structure for holding text metrics
 *
 *	\if NODOC
 * $Log: textmetrics.h_v $
 * Revision 1.2  2005/07/14 17:39:29  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.1  2005/03/07 16:39:39  dwilliss
 * Initial revision
 *
 * \endif
 */

#ifndef INC_MI32_TEXTMETRICS_H
#define INC_MI32_TEXTMETRICS_H

#ifndef INC_MEMORY_H
#include <mi32/memory.h>		// for memset
#define INC_MEMORY_H
#endif

//! A structure to hold text metrics
//! Ascent is the distance from the base line of the first line of text to the
//!  top of the "ink" of the fist line.  
//! Descent is distance from the last base line (assuming multiple lines) to 
//!  the bottom of the descenders of the last line.
//! Height is the distance between the top ascent and bottom descent.
//! If you're interested in "tight" metrics, that is, only including the 
//!   size of characters in the given string, use the FirstLineAscent,
//!   LastLineDescent and Height
//! If you're interested height that would include room for ascenders and 
//!   descenders even if the given string doesn't have any, use MaxFontAscent,
//!   MaxFontDescent, UniformHeight
struct TEXTMETRICS {
	int Width;					//!< Width of text
	int Height;					//!< Height of text
	int FirstLineAscent;		//!< Ascent of first line from base line
	int LastLineDescent;		//!< Descent of last line from base line
	int MaxFontAscent;		//!< Max font ascent (designed)
	int MaxFontDescent;		//!< Max font descent (designed)
	int UniformHeight;		//!< Height based on MaxFontAscent/MaxFontDescent

	//! Default constructor memsets the whole structure to 0 for you.
	TEXTMETRICS (
		) {
		Clear();
		}

	void Clear (
		) {
		memset(this, 0, sizeof(TEXTMETRICS));
		}
	};



#endif // INC_MI32_TEXTMETRICS_H

