/**
 * \file rvc/object.h 
 *	\brief Interface for the RVCOBJECT class.
 *
 * \if NODOC
 * $Id: object.h_v 1.163 2012/11/19 20:20:29 bcolombe Exp $
 *
 * $Log: object.h_v $
 * Revision 1.163  2012/11/19 20:20:29  bcolombe
 * Add title to openconnection
 *
 * Revision 1.162  2012/11/19 15:54:55  bcolombe
 * Add OpenExternalConnection
 *
 * Revision 1.161  2011/10/10 13:54:01  mju
 * Remove unneeded hdr.
 *
 * Revision 1.160  2011/09/02 12:59:58  mju
 * Exclude macro from dox.
 *
 * Revision 1.159  2011/01/18 17:18:18  mju
 * Add autofilecache nested class and private cache support methods.
 *
 * Revision 1.158  2009/11/25 16:01:15  mju
 * Doxygen fix.
 *
 * Revision 1.157  2009/04/21 13:42:18  mju
 * Add copyflag_NoUpdateEmbeddedNames.
 *
 * Revision 1.156  2009/03/18 15:33:50  mju
 * Add testUsableForLimit and remove lite limit enum.
 *
 * Revision 1.155  2008/11/19 16:12:48  scowan
 * Removed legacy getLinkedFiles.
 *
 * Revision 1.154  2008/11/18 18:45:29  scowan
 * Removed unused changeObjectLInk method.
 *
 * Revision 1.153  2008/11/18 18:29:06  scowan
 * Added different versions of getLinkedFiles and changeObjectLink.
 * Deprecated the old ones.
 *
 * Revision 1.152  2008/11/18 18:01:12  scowan
 * Changed names to getObjItem from getObjectItem.
 *
 * Revision 1.151  2008/11/18 17:03:07  scowan
 *  Added protected methods to get inode numbers from objitem's
 *  Added protected methods to setup objitems' from filepaths and inode numbers..
 *
 * Revision 1.150  2008/10/06 14:48:08  scowan
 * Added setOpenParms to resolve deprecated access.
 *
 * Revision 1.149  2008/07/31 19:29:11  scowan
 * Export autoLinkCachePath and autoLinkLocalPath.
 *
 * Revision 1.148  2008/05/06 16:29:46  scowan
 * Added validateFileMapping methgod.
 *
 * Revision 1.147  2008/03/06 21:18:40  mju
 * Add dumpOpenFileList.
 *
 * Revision 1.146  2008/02/18 14:38:44  mju
 * Remove unused legacy validateName fn.
 *
 * Revision 1.145  2008/01/22 23:30:08  scowan
 * Added virtual method to get helper filter if necessary.
 *
 * Revision 1.144  2007/11/27 18:28:33  scowan
 * Added saveObjectTo method.
 *
 * Revision 1.143  2007/10/26 22:44:02  scowan
 * Updated flags and structs to be enums and rvc namespaced.
 *
 * Revision 1.142  2007/10/26 17:20:34  scowan
 * Switched to new method to get the file locking paths.
 *
 * Revision 1.141  2007/10/26 16:55:25  scowan
 * Added a getAutoLinkPatternList method to the class.
 *
 * Revision 1.140  2007/09/27 17:35:37  scowan
 * Added parameter to searchForInvalidChildren method.
 *
 * Revision 1.139  2007/09/27 17:00:40  scowan
 * Added needNewerThanParent method.
 *
 * Revision 1.138  2007/09/12 17:10:21  scowan
 * Moved gen and basic info into private include files.
 *
 * Revision 1.137  2007/09/07 15:58:42  scowan
 * Replaced gen info header with date time header.
 *
 * Revision 1.136  2007/08/30 15:21:57  scowan
 * Added multi-thread enable static method.
 *
 * Revision 1.135  2007/08/02 16:14:38  dwilliss
 * added protected method
 *
 * Revision 1.134  2007/07/19 17:54:38  scowan
 * Removed unused getFileHandlePro method.
 *
 * Revision 1.133  2007/05/11 13:34:38  mju
 * Remove datetime hdr.
 * Deprecate inclusion of objitem hdr as not needed by this hdr.
 *
 * Revision 1.132  2007/04/20 20:35:42  scowan
 * Added static method to get the icon name from an object type.
 *
 * Revision 1.131  2007/04/16 16:55:51  scowan
 * Moved lite test routines to static rvc object class methods.
 *
 * Revision 1.130  2007/04/16 16:23:34  scowan
 * Added getObjectTag from Object Type static method.
 *
 * Revision 1.129  2007/04/06 21:09:18  scowan
 * Added method to get project file lock state.
 *
 * Revision 1.128  2007/03/23 17:51:17  scowan
 * Removed unused methods.
 *
 * Revision 1.127  2007/03/21 21:27:32  scowan
 * Added method to effciently get the descriptor for the project file.
 *
 * Revision 1.126  2007/01/04 19:02:35  scowan
 * Added credentials to attach.
 *
 * Revision 1.125  2007/01/03 23:47:01  scowan
 * Added v_getParent.
 *
 * Revision 1.124  2006/12/08 18:46:03  mju
 * Don't allow deprecated open methods if credentials are mandatory.
 *
 * Revision 1.123  2006/10/16 16:00:08  scowan
 * Added static methods to get objtype and child separators.
 *
 * Revision 1.122  2006/10/13 21:21:38  mju
 * Use rvc linkitem hdr, remove rvcdefns hdr.
 *
 * Revision 1.121  2006/10/02 23:23:54  scowan
 * Added docs.
 *
 * Revision 1.120  2006/10/02 23:14:46  scowan
 * Added two new obserable overrides.
 *
 * Revision 1.119  2006/09/28 15:43:11  scowan
 * Added objitem include file.
 *
 * Revision 1.118  2006/04/19 00:04:22  dfriberg
 * Add comments for MoveTo method
 *
 * Revision 1.117  2006/04/18 21:55:28  scowan
 * Added docs to findChild methods.
 *
 * Revision 1.116  2006/04/03 21:30:30  scowan
 * Fixed win64 warnings.
 *
 * Revision 1.115  2006/03/31 19:07:53  mju
 * Add openmode_ReadNoScanAutoLink.
 *
 * Revision 1.114  2006/02/01 18:08:23  scowan
 * DOne with deprecated changes Mike wanted.
 *
 * Revision 1.113  2006/01/04 21:58:14  scowan
 * Added another method.
 *
 * Revision 1.112  2005/12/19 17:42:23  dwilliss
 * method of making credentials manditory adjusted for methods that already
 * had default parameters before the credentials
 *
 * Revision 1.111  2005/12/19 16:53:24  dwilliss
 * Fixed spelling error on ifdef.
 *
 * Revision 1.110  2005/12/19 16:45:06  dwilliss
 * credentials now manditory if you #define CREDEINTIALS_MANDITORY first
 *
 * Revision 1.109  2005/12/09 21:20:44  scowan
 * Moved method from protected to public.
 *
 * Revision 1.108  2005/12/09 20:41:45  scowan
 * Added method to disable credential query prompts.
 *
 * Revision 1.107  2005/12/09 17:29:43  scowan
 * Updated to use credentials.
 *
 * Revision 1.106  2005/10/25 16:53:31  mju
 * Add getExternalFileDescription.
 *
 * Revision 1.105  2005/09/15 15:22:15  mju
 * Don't name unreferenced parms.
 *
 * Revision 1.104  2005/09/01 22:50:28  scowan
 * Added virtual methods Dave wanted.
 *
 * Revision 1.103  2005/08/19 14:57:46  scowan
 * Implemented 7.1 changes here.
 *
 * Revision 1.102  2005/08/18 22:45:39  scowan
 * Added virtual method.
 *
 * Revision 1.101  2005/08/17 17:48:25  dwilliss
 * Added a couple methods to OBSERVER
 *
 * Revision 1.100  2005/08/12 15:47:26  scowan
 * Added getParentstack method.
 *
 * Revision 1.99  2005/08/09 21:25:53  scowan
 * Added methods to get/set metadata.
 *
 * Revision 1.98  2005/08/02 22:11:18  scowan
 * Descriptor support.
 *
 * Revision 1.97  2005/07/26 16:39:08  scowan
 * Added another static method.
 *
 * Revision 1.96  2005/07/26 16:09:27  scowan
 * Added statis method.
 *
 * Revision 1.95  2005/07/08 14:51:28  scowan
 * Added setMakeParms method.
 *
 * Revision 1.94  2005/06/23 23:03:26  scowan
 * Made last method const.
 *
 * Revision 1.93  2005/06/23 22:49:03  scowan
 * Added protected method.
 *
 * Revision 1.92  2005/06/13 15:57:29  scowan
 * Added static method to determine if the RVC project file system is initialized.
 *
 * Revision 1.91  2005/06/03 20:03:00  scowan
 * Added move and is valid parent methods.
 *
 * Revision 1.90  2005/03/31 16:57:27  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.89  2005/03/09 21:19:01  scowan
 * DOcs updated.
 *
 * Revision 1.88  2005/02/03 22:31:58  scowan
 * Added conversion operator.
 *
 * Revision 1.87  2005/01/19 16:05:08  mju
 * Include descriptor hdr so can use implicit ctors.
 *
 * Revision 1.86  2005/01/18 23:43:37  scowan
 * Updated to use descriptors.
 *
 * Revision 1.85  2004/12/16 17:05:21  mju
 * Add filter_objtypeset::setTypes method.
 *
 * Revision 1.84  2004/09/01 22:16:07  scowan
 * Added copy to edit format flag.
 *
 * Revision 1.83  2004/05/14 17:15:39  scowan
 * Added method to get invalid children.
 *
 * Revision 1.82  2004/05/03 19:26:59  scowan
 * Added class to inform copy of name change.
 *
 * Revision 1.81  2004/03/23 23:03:24  scowan
 * Added child observer support.
 *
 * Revision 1.80  2004/03/19 18:48:06  scowan
 * Added more things to observe.
 *
 * Revision 1.79  2003/12/18 14:24:14  mju
 * Remove deprecated filter typedefs.
 *
 * Revision 1.78  2003/12/03 23:31:02  scowan
 * Added new set make object method.
 *
 * Revision 1.77  2003/11/25 18:42:41  scowan
 * Added get child list method.
 *
 * Revision 1.76  2003/11/21 23:10:03  scowan
 * Added new copy flag.
 *
 * Revision 1.75  2003/11/19 22:35:13  scowan
 * Added more open flags.
 *
 * Revision 1.74  2003/11/19 20:14:37  scowan
 * Added new copy flag.
 *
 * Revision 1.73  2003/11/04 22:29:27  scowan
 * Added copy flag and make link method.
 *
 * Revision 1.72  2003/10/30 18:27:47  scowan
 * Added is older than method.
 *
 * Revision 1.71  2003/10/08 22:30:22  scowan
 * Added method for object info dump.
 *
 * Revision 1.70  2003/10/07 17:32:18  scowan
 * Object virtual method changes.
 *
 * Revision 1.69  2003/10/02 19:02:19  scowan
 * Added another statis method.
 *
 * Revision 1.68  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.67  2003/09/25 21:56:58  dwilliss
 * doxygen
 *
 * Revision 1.66  2003/09/22 19:55:29  scowan
 * Added recover project file call.
 *
 * Revision 1.65  2003/09/19 22:27:03  scowan
 * Moved get name and get desc to virtual methods.
 *
 * Revision 1.64  2003/09/15 23:13:31  scowan
 * Added yet another fine method.
 *
 * Revision 1.63  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.62  2003/09/08 15:30:59  scowan
 * Added multiple is... methods.
 *
 * Revision 1.61  2003/08/28 15:25:24  scowan
 * Added methods to determine and get linked file paths.
 *
 * Revision 1.60  2003/08/27 21:26:54  scowan
 * Support new copy object methods.
 *
 * Revision 1.59  2003/06/05 15:19:44  scowan
 * Changed filter name for newer than parent.
 *
 * Revision 1.58  2003/05/06 23:07:44  scowan
 * Added no_ deprecated around old typedef.
 *
 * Revision 1.57  2003/05/06 23:05:04  scowan
 * Added Mike's suggested changes.
 *
 * Revision 1.56  2003/04/09 22:03:48  scowan
 * Added protected method to get the file handle.
 *
 * Revision 1.55  2003/03/19 23:07:00  scowan
 * Forgot to inherit.
 *
 * Revision 1.54  2003/03/19 23:00:05  scowan
 * Added unique child name filter.
 *
 * Revision 1.53  2003/03/14 21:30:17  scowan
 * Added compute free file space method.
 *
 * Revision 1.52  2003/03/06 23:12:47  scowan
 * Added methods to flush and change temp file modes.
 *
 * Revision 1.51  2002/11/25 15:18:39  scowan
 * Added new change source method.
 *
 * Revision 1.50  2002/11/19 22:13:46  scowan
 * In newer than parent, assume equal is newer.
 *
 * Revision 1.1  2000/09/28 22:24:17  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::OBJECT rvc/object.h
//!	\brief The base class for all RVC objects
//!
//!	<b> Defined contract of the RVC Object API system.</b>
//!	
//!	The RVC Object API is designed with multiple goals, the goals are defined below
//!	
//!	-#  Support for thread safe use of RVC objects.  This goal is met using these conditions:
//!			-	RVC Object classes do not allow direct access to information that was previously in rvc object header information.  Since data
//!				can change in a multi-threaded environment, methods are used to gain information.  The removal of access to rvc object headers
//!				causes some desirable design changes including:
//!				-#	A base level open method for rvc objects that support an open paradigm.  All open functions previously took a header pointer 
//!					thereby requiring a different open function for each rvc object.  This has been rendered moot.
//!				-# A base level close method for most rvc objects.  All close functions took an optional header pointer thereby requiring a
//!					different close function for each open rvc object type.
//!			-  Read/Write access must be explicitly defined when opening an rvc object.  This allows a multi-threaded implementation to
//!				generate 'locks' on rvc objects based on the access requirement.
//!			-  Each instance of a rvc object gets a unique handle to the rvc object that the class represents.  Handle management is class based.
//!	-#	Strict hierarchy level of classes to limit errors.  This goal is met using these conditions:
//!		-	RVC Object classes determine what rvc object types they can deal with.  API's cannot be mixed with this check.
//!		-	Class hierarchies with base classes that support common functionality.
//!	-#	Eliminating the use of RVCINODENUM's to manage RVC objects.
//!		-	RVCINODENUM's are not type safe and can be intermixed between open rvc objects, causing errors.
//!		-	The tendancy of programmers to store RVCINODENUM's in reference files is to be RETIRED.  Hiding RVCINODENUM's from 
//!				code will prevent this action.
//!	
//!	<b>Attaching and opening rvc objects.</b>
//!	
//!	The construct of attachment and opening comes from the fact that we do not handle exceptions.  Therefore error handling is done
//!	through return values.  Since constructors do not have return values, the constructor set is kept to a minimum.  In addition, 
//!	the concept of closing an rvc object while maintaining knowledge of where the rvc object is needs to be implemented.  Therefore two
//!	methodologies have been introduced to deal with the requirements:
//!
//!	
//!		-#	The concept of attaching the instance to an rvc object using Attach() methods.  These methods will validate the attachment
//!			by opening the file and locating the rvc object and validate that the derived class is appropriate for the rvc object.  The method
//!			will close the file after it has accomplished its goal.
//!
//!		-#	Opening the rvc object will perform the same checks as the Attach() method as well as keeping the file open and if possible,
//!			opening the derived class rvc object.
//!
//!	
//!	<b>Use of the 'const' keyword</b>
//!	
//!	The 'const' keyword is used to define methods that do not modify the rvc object.  "Const" methods for RVC::OBJECT will open the object using
//! RVC::OBJECT::OPENMODE_Read and perform the operation.  Attempting to use a non-open RVC::OBJECT instance for non-const methods will result
//! in the method returning an R_WriteInvalid error.


#ifndef INC_RVC_OBJECT_H
#define INC_RVC_OBJECT_H

#ifndef INC_RVC_OTYPESET_H
	#include <rvc/otypeset.h>
#endif

#ifndef INC_RVC_OBJECTNAME_H
	#include <rvc/objectname.h>
#endif

#ifndef INC_MI32_RVCDATETIME_H
	#include <mi32/rvcdatetime.h>
#endif

#ifndef	INC_MI32_OBSERVER_H
	#include <mi32/observer.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifndef	INC_RVC_CREDENTIALS_H
	#include <rvc/credentials.h>
#endif

#ifndef INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
enum BYTESWAP {
	BYTESWAP_LoHi,
	BYTESWAP_HiLo
	};
class MISTRING;
class DATETIME;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
struct ANYCALLBACKSTRUCT;
struct GENINFO;
class OBJITEM;
class OBJITEMLIST;
class FILEHANDLE;
class OBJECTOBSERVER;
class DESCRIPTOR;
class OBJMAKEPARMS;
class OBJOPENPARMS;
#endif // GENERATING_DOXYGEN_OUTPUT
	
#ifdef RVC_CREDENTIALS_MANDATORY
#define RVC_DEFAULT_CREDENTIALS 
#else
#define RVC_DEFAULT_CREDENTIALS = 0
#endif

class OBJECT : public SUBJECT<OBJECTOBSERVER> {
	public:

		enum COPYFLAGS {
			COPYFLAG_None = 						0x00000000,	//!< Default, copy subobjects and links
			COPYFLAG_DontCopyChildren = 		0x00000001,	//!< Do not copy child objects
			COPYFLAG_CopySpatialFromLink =	0x00000002,	//!< Copy data from spatial link, not the link itself
			COPYFLAG_CopyDBTableFromLink =	0x00000004,	//!< Copy data from DB table link, not the link itself
			COPYFLAG_CopyRVCObjectFromLink =	0x00000008,	//!< Copy data from RVC object link, not the link itself
			COPYFLAG_CopyTextFromLink =		0x00000010,	//!< Copy data from text link, not the link itself
			COPYFLAG_DontCopyStdAttTables	=	0x00000020,	//!< Do not copy standard attrib tables
			COPYFLAG_ReportInvalidParent =	0x00000040,	//!< Report invalid parent errors
			COPYFLAG_ForceOneObjectCopy =		0x00000080,	//!< Force the copy of the most recently used object if the singleton rule is violated
			COPYFLAG_CopyIfNotModifiable =	0x00000100,	//!< Copy data from the link, not the link itself, if the data is not modifiable through the link
			COPYFLAG_CopyToEditFormat =		0x00000200,	//!< Copy data from the source to an editable format, used for CAD currently
			COPYFLAG_NoUpdateEmbeddedNames =	0x00000400,	//!< Don't update embedded name strings to new location
			};
			
		enum DUMPINFOFLAGS {
			DUMPINFOFLAG_Minimal = 				0x00000000,			// Default, dumps only minimal information
			DUMPINFOFLAG_Subobjects =			0x00000001,			// Dump sub-object information at the end of the log
			DUMPINFOFLAG_FullHeader =			0x00000002,			// Dump full header information into the log
			DUMPINFOFLAG_ObjectData =			0x00000004,			// Dump object data information into the log
			// Combination flags here
			DUMPINFOFLAG_Standard =				(DUMPINFOFLAG_Subobjects | DUMPINFOFLAG_FullHeader),
			DUMPINFOFLAG_Maximal = 				(DUMPINFOFLAG_Subobjects | DUMPINFOFLAG_FullHeader | DUMPINFOFLAG_ObjectData)
			};
			
		enum LINKFILEFLAGS {
			LINKFILEFLAG_None = 					0x00000000,			//!< Get only the link information for the object
			LINKFILEFLAG_SearchChildren = 	0x00000001			//!< Get link information for the object and children
			};
			
		enum LOCKSTATE {
			LOCKSTATE_Unlocked,								//!< Project file does not have any lock upon it, or it is a temp file
			LOCKSTATE_CurrentOpenReadIsWritable,		//!< Current process has the file open for reading and can open for writing
			LOCKSTATE_CurrentOpenReadNotWritable,		//!< Current process has the file open for reading and CANNOT open for writing
			LOCKSTATE_CurrentOpenWrite,					//!< Current process has the file open for writing
			LOCKSTATE_ExternalOpenRead,					//!< Another process has file opened for reading, no lock file
			LOCKSTATE_ExternalOpenWrite,					//!< Another process has file opened for writing, no lock file
			LOCKSTATE_ExternalReadLockFile,				//!< A process has a read lock file for this project file
			LOCKSTATE_ExternalWriteLockFile				//!< A process has a write lock file for this project file
			};
			
		enum OPENMODE {					//! Mode for opening rvc files and objects, mutually exclusive
			OPENMODE_NotOpen = 0,		//!< Returned by GetOpenMode() if the rvc file / object is not open
			OPENMODE_Read,					//!< Open file with 'read' access
			OPENMODE_Write,				//!< Open file with 'write' access
			OPENMODE_WriteAttempt,		//!< Attempt to open the file for writing, if failed, open for reading only
			OPENMODE_ReadNoLock,			//!< Open file generating no lock files, used for system resource files
			OPENMODE_WriteAbsolute,		//!< Override ReadAbsolute system flag for a specific file
			OPENMODE_ReadAbsolute,		//!< File is to be opened read-only, no lock file and no possibility of change
			OPENMODE_WriteNoLock,		//!< Open file generating no lock files, used for system resource files
			OPENMODE_ReadNoDialog,		//!< Open file for reading without popping up the lock dialog
			OPENMODE_ReadNoScanAutoLink,	//!< Open for read without doing autolink scanning
			OPENMODE_WriteNoDialog		//!< Open file for writing without popping up the lock dialog
			};

		enum TEMPFILE {
			TEMPFILE_DeleteOnClose = 0x00000100,
			TEMPFILE_DeleteOnExit =  0x00000200
			};
		
		// NESTED CLASSES
		// FORWARD Declarations
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class CONST_ITERATOR;
		friend class CONST_ITERATOR;
		class ITERATOR;
		friend class ITERATOR;
		class FILTER;
		class FILTER_OBJTYPESET;
		class FILTER_NEWERTHANPARENT;
		class TOPLEVELNAMECHANGE;
		friend class TOPLEVELNAMECHANGE;
		class UNIQUENAMEFILTER;
		class UNIQUECHILDNAMEFILTER;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Sentry class to automatically enable/disable open file caching.
		//! Use of open file caching can dramatically improve performance when multiple open/close operations might be performed on the same file(s).
		//! Should only be created on the stack.
		class AUTOFILECACHE {
			public:
				//! Constructor, turns open file caching on.
				AUTOFILECACHE ()
					{ OBJECT::CacheEnable(); }
				//! Destructor, turns open file caching off.
				~AUTOFILECACHE ()
					{ OBJECT::CacheDisable(); }
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				// Not allowed.
				AUTOFILECACHE (const AUTOFILECACHE&);
				AUTOFILECACHE& operator= (const AUTOFILECACHE&);
				#endif
			};

		//! Item holding the link information to either an external file, another project file object, or a RDBMS link
		//! Used in GetLinkedFiles()
		class LINKITEM {
			public:
				//! Get the FILEPATH for the link.  Is empty if the link does not reference a file.
				const FILEPATH& GetFilename (
					) const { return (m_Filename); }
			
				//! Get the object name for the link.  
				//! Is empty if the link does not reference a project file object.
				//! If Filename is empty, ObjName contains the connection string
				const MISTRING& GetObjName (
					) const { return (m_ObjName); }
			
				//! Get the OBJITEM of the object that holds the link
				const OBJITEM& GetSrcObjItem (
					) const { return (m_SrcObjItem); }
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				FILEPATH m_Filename;
				MISTRING m_ObjName;
				OBJITEM m_SrcObjItem;				//!< The source object that contains the link
				
				friend class OBJECT;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};
		typedef MILIST<LINKITEM> LINKITEMLIST;

		//! Default constructor
		OBJECT (							//! Empty constructor, must attach to an object using Attach()
			);

		//! Copy constructor
		OBJECT (							//! Copy Constructor
			const RVC::OBJECT& obj
			);

		//! Destructor
		virtual ~OBJECT (
			);

		//! Assignment.  Will close if necessary and reopen if 'rhs' is open
		RVC::OBJECT& operator= (					//! Assignment operator
			const RVC::OBJECT& rhs
			);

		//! Conversion operator to pass RVC::OBJECT and derivatives to methods that require RVC::OBJITEM			
		operator RVC::OBJITEM (
			) const;

		//! Attach an OBJECT instance to a rvc file root folder using a FILEPATH
		//!
		//! This will open the file to determine parameter correctness.
		//! If the class is derived, this function will fail and return an error.
		ERRVALUE Attach (
			const FILEPATH& FilePath,			//!< Location of file to attach to
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Attach an OBJECT instance to a file using an OBJITEM
		//!
		//! This will open the file to determine parameter correctness
		//! If the derived class cannot handle the rvc object type, the rvc object is not attached and an error is returned
		ERRVALUE Attach (
			const RVC::OBJITEM& ObjItem,			//!< Item containing file and rvc object location to attach to
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Attach an OBJECT instance to a file using the FilePath and an rvc object path
		//!
		//! This will open the file to determine parameter correctness and locate the correct rvc object.
		//! If the derived class cannot handle the rvc object type, the rvc object is not attached and an error is returned
		ERRVALUE Attach (
			const FILEPATH& FilePath,			//!< Location of project file to open
			const MISTRING& ObjectPath,			//!< Location of rvc object in project file to open
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Attach an OBJECT instance to a file using a parent rvc object, child rvc object type and child name
		//!
		//! This will open the file to determine parameter correctness and locate the correct rvc object.
		//! If the derived class cannot handle the rvc object type, the rvc object is not attached and an error is returned
		ERRVALUE Attach (
			const RVC::OBJECT& parent,				//!< Parent object to look for the correct child
			const RVC::OBJTYPE ObjType,			//!< Child rvc object type to attach to
			const RVC::OBJECTNAME& ShortName,		//!< Name of child rvc object to attach to
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Attach an OBJECT instance to a file using a parent rvc object, child rvc object type and child descriptor
		//!
		//! This will open the file to determine parameter correctness and locate the correct rvc object.
		//! If the derived class cannot handle the rvc object type, the rvc object is not attached and an error is returned
		ERRVALUE Attach (
			const RVC::OBJECT& parent,				//!< Parent object to look for the correct child
			const RVC::OBJTYPE ObjType,			//!< Child rvc object type to attach to
			const RVC::DESCRIPTOR& Descriptor,	//!< Identifier of child rvc object to attach to
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Attach an RVC::OBJECT instance to a file using a source rvc object, possible relative path, and rvc object path
		//!
		//! This will open the file to determine parameter correctness and location of the correct rvc object.
		//! If the derived class cannot handle the rvc object type, the rvc object is not attached and an error is returned
		ERRVALUE Attach (
			const RVC::OBJECT& source,			//!< If 'filename' is relative, this specifies the source path
			const MIUNICODE *filename,			//!< Location of rvc file, can be a relative path
			const MISTRING& ObjectPath,			//!< Location of rvc object in the rvc file
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Attach an RVC::OBJECT instance to a file using its twin.
		//!
		//! This will open the file if the twin's file is open
		//! If the derived class cannot handle the rvc object type, the rvc object is not attached and an error is returned
		ERRVALUE Attach (
			const RVC::OBJECT& twin,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Attach an OBJECT instance to an OBJITEM's parent
		//!
		//! This will open the file to determine parameter correctness
		//! If the derived class cannot handle the rvc object type, the rvc object is not attached and an error is returned
		ERRVALUE AttachToParent (
			const RVC::OBJITEM& ObjItem,			//!< Item containing file and rvc object location to attach to
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Determine if the rvc file can be opened for writing, irregardless of its current open state
		//!
		//! Object must be attached or false is returned
		//! This function does not check if another process or thread has a write lock on the file, it
		//! determines that the file \b CAN be opened for writing.
		//! @return true if file is writable and the RVC system allows writing, false if cannot open for write
		bool CanOpenForWrite (						//! Is file read-only?  (Object cannot be opened for writing)
			) const;

		//! Change the rvc object description
		//! Object must be open with OPENMODE_Write
		ERRVALUE ChangeDescriptor (
			const DESCRIPTOR& NewDescriptor,			//!< New description for the object
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Change the rvc object that this object is linked to
		//! Object must be open with OPENMODE_Write
		ERRVALUE ChangeObjectLink (
			const FILEPATH& Filename,
			const MISTRING& ObjName,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS	//!< User credentials passed in, NULL for default behavior
			);

		//! Change the rvc object source
		//! Object must be open with OPENMODE_Write
		ERRVALUE ChangeSource (
			const RVC::OBJECTSOURCE& NewSource,		//!< New source name for the object
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Change the temp file mode to TEMPFILE_DeleteOnClose or TEMPFILE_DeleteOnExit 
		ERRVALUE ChangeTempFileMode (
			TEMPFILE flags
			);
			
		//! Obtain a const rvc child object iterator given an rvc object type.  This iterator can be used to step through 
		//! all of the children of 'this'.
		//! @return A constant iterator to the first child that meets the specifications
		CONST_ITERATOR ChildListBegin (			//! Get iterator to the beginning of the child list
			RVC::OBJTYPE m_ObjType						//!< Object type of children to search for
			) const;
		
		//! Obtain a rvc child object iterator given an rvc object type.  This iterator can be used to step through 
		//! all of the children of 'this'.
		//! @return An iterator to the first child that meets the specifications
		ITERATOR ChildListBegin (				//! Get iterator to the beginning of the child list
			RVC::OBJTYPE m_ObjType						//!< Object type of children to search for
			);

		//! Obtain a const rvc child object iterator that points to the end of the list.
		//! @return A constant iterator to the end of the child list
		CONST_ITERATOR ChildListEnd (			//! Get iterator to the child list end
			) const;

		//! Obtain a rvc child object iterator that points to the end of the list.
		//! @return An iterator to the end of the child list
		ITERATOR ChildListEnd (					//! Get iterator to the child list end
			);

		//! Close the rvc object
		//!
		//! Will close the file and the rvc object if the derived class supports the paradigm.  The instance is still considered attached.
		ERRVALUE Close (	 					//! Close the open file/object
			);

		//! Compute project file free space
		//! @return Amount of project file free space
		INT64 ComputeProjectFileFreeSpace (
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			) const;
			
		//! Make a copy of the rvc object, and its children, to another place specified by the destination parent object.
		//! Does not copy DBTABLE objects.  Use RVC::DBASE::CopyTableTo() method.
		ERRVALUE CopyTo (
			RVC::OBJECT& DestParentObj,			//!< Parent object to copy object and children to
			RVC::OBJECT& DestObj,					//!< Attached instance to destination object
#ifdef RVC_CREDENTIALS_MANDATORY
			COPYFLAGS CopyFlags,
#else
			COPYFLAGS CopyFlags = COPYFLAG_None,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			) const;

		//! Make a copy of the rvc object, and its children, to another place specified by the destination parent object.
		//! Does not copy DBTABLE objects.  Use RVC::DBASE::CopyTableTo() method.
		ERRVALUE CopyTo (
			RVC::OBJECT& DestParentObj,			//!< Parent object to copy object and children to
			RVC::OBJECT& DestObj,					//!< Attached instance to destination object
			const DESCRIPTOR& Descriptor,			//!< New name of copied object
#ifdef RVC_CREDENTIALS_MANDATORY
			COPYFLAGS CopyFlags,
#else
			COPYFLAGS CopyFlags = COPYFLAG_None,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			) const;

		//! Make a copy of the rvc object, and its children, to another place specified by the destination parent object.
		//! Does not copy DBTABLE objects.  Use RVC::DBASE::CopyTableTo() method.
		ERRVALUE CopyTo (
			RVC::OBJECT& DestParentObj,			//!< Parent object to copy object and children to
			RVC::OBJECT& DestObj,					//!< Attached instance to destination object
			FILTER& filter,							//!< Method called with an rvc object to determine fitness to copy
#ifdef RVC_CREDENTIALS_MANDATORY
			COPYFLAGS CopyFlags,
#else
			COPYFLAGS CopyFlags = COPYFLAG_None,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			) const;

		//! Make a copy of the rvc object, and its children, to another place specified by the destination parent object.
		//! Does not copy DBTABLE objects.  Use RVC::DBASE::CopyTableTo() method.
		ERRVALUE CopyTo (
			RVC::OBJECT& DestParentObj,			//!< Parent object to copy object and children to
			RVC::OBJECT& DestObj,					//!< Attached instance to destination object
			FILTER& filter,							//!< Method called with an rvc object to determine fitness to copy
			const DESCRIPTOR& Descriptor,			//!< New name of copied object
#ifdef RVC_CREDENTIALS_MANDATORY
			COPYFLAGS CopyFlags,
#else
			COPYFLAGS CopyFlags = COPYFLAG_None,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			) const;

		//! Copy rvc object's children to the given destination parent object
		ERRVALUE CopyChildrenTo (
			RVC::OBJECT& DestParentObj,			//!< Parent to copy children to.
#ifdef RVC_CREDENTIALS_MANDATORY
			COPYFLAGS CopyFlags,
#else
			COPYFLAGS CopyFlags = COPYFLAG_None,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			) const;

		//! Copy rvc object's children to the given destination parent object with filter
		ERRVALUE CopyChildrenTo (
			RVC::OBJECT& DestParentObj,			//!< Parent to copy children to.
			FILTER& filter,							//!< Method called with an rvc object to determine fitness to copy
#ifdef RVC_CREDENTIALS_MANDATORY
			COPYFLAGS CopyFlags,
#else
			COPYFLAGS CopyFlags = COPYFLAG_None,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			) const;

		//! Delete the rvc object.  The method will call Close().
		//!
		//! For this function to work correctly, <b>ALL</b> instances that refer to this object must be closed.
		//! This method does not support delete when all closed paradigm.
		ERRVALUE Delete (						//! Delete the RVC::OBJECT
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Detach instance from RVC object
		//! Will close the object and file
		void Detach (							//! Close and unattach the file representing this RVC::OBJECT
			);

		//! Dump object information into a MISTRING reference
		ERRVALUE DumpObjectInfo (
			MISTRING& LogString,
#ifdef RVC_CREDENTIALS_MANDATORY
			DUMPINFOFLAGS DumpInfoFlags,
#else
			DUMPINFOFLAGS DumpInfoFlags = DUMPINFOFLAG_Minimal,	//!< Controls how much information is placed into "LogString'
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			);

		//! Dump list of all currently open files and objects.
		static void DumpOpenFileList (
			MISTRING& result,
			bool IncludeOpenObjects = true
			);
			
		//! Locate rvc child object based on rvc child default object type and given name
		//!
		//! This routine determines the derived class default rvc object type and uses that for its search.
		//! Object must be attached for this method to work.  Returns false if not
		//! @return true if rvc object found and child sucessfully attached, false if not
		bool FindChild (						//! Find child RVC::OBJECT by RVC::OBJECTNAME, if exists
			RVC::OBJECT& Child,						//!<  Child object attached if found
			const RVC::OBJECTNAME& name				//!< Name of rvc child object to find
			) const;

		//! Locate rvc child object based on rvc child default object type, given name, and a filter object
		//!
		//! This routine determines the derived class default rvc object type and uses that for its search.
		//! Object must be attached for this method to work.  Returns false if not
		//! @return true if rvc object found and child sucessfully attached, false if not
		bool FindChild (						//! Find child RVC::OBJECT by RVC::OBJECTNAME and filter, if exists
			RVC::OBJECT& Child,						//!<  Child object attached if found
			const RVC::OBJECTNAME& name,			//!< Name of rvc child object to find
			FILTER& filter								//!< Method called with an rvc object to determine child fitness
			) const;

		//! Locate rvc child object based on object path
		//!
		//! This routine locates the proper rvc object using the child's filepath and the rvc object path.
		//! Object must be attached for this method to work.  Returns false if not
		//! @return true if rvc object found and child sucessfully attached, false if not
		bool FindChild (						//! Find child RVC::OBJECT by RVC Object Path, if exists
			RVC::OBJECT& Child,						//!<  Child object attached if found
			const MISTRING& ObjectPath			//!< RVC object path to look for child with
			) const;

		//! Locate rvc child object based on rvc child default object type and given name
		//! This routine determines the derived class default rvc object type and uses that for its search.
		//! The DESCRIPTOR::GetShortName() is used first, if no matches found, then the DESCRIPTOR::GetFullName()
		//! is searched on.
		//! Object must be attached for this method to work.  Returns false if not
		//! @return true if rvc object found and child sucessfully attached, false if not
		bool FindChild (
			RVC::OBJECT& Child,						//!< Child object attached if found
			const RVC::DESCRIPTOR& Descriptor
			) const;

		//! Locate rvc child object based on rvc child default object type, given name, and a filter object
		//! This routine determines the derived class default rvc object type and uses that for its search.
		//! The DESCRIPTOR::GetShortName() is used first, if no matches found, then the DESCRIPTOR::GetFullName()
		//! is searched on.
		//! Object must be attached for this method to work.  Returns false if not
		//! @return true if rvc object found and child sucessfully attached, false if not
		bool FindChild (
			RVC::OBJECT& Child,						//!< Child object attached if found
			const RVC::DESCRIPTOR& Descriptor,
			FILTER& filter								//!< Method called with an rvc object to determine child fitness
			) const;

		//! Locate last used rvc child object based on child rvc object type
		//! This routine determines the derived class default rvc object type and uses that for its search.
		//! Object must be attached for this method to work.  Returns false if not
		//! @return true if rvc object found and child sucessfully attached, false if not
		bool FindChildLastUsed (				//! Last used RVC::OBJECT if found assigned to "Child"
			RVC::OBJECT& Child						//!< RVC Child object attached if found
			) const;

		//! Locate last used rvc child object based on child rvc object type and a filter object
		//! This routine determines the derived class default rvc object type and uses that for its search.
		//! Object must be attached for this method to work.  Returns false if not
		//! @return true if rvc object found and child sucessfully attached, false if not
		bool FindChildLastUsed (			//! Last used RVC::OBJECT if found assigned to "Child"
			RVC::OBJECT& Child,				//!<  Child object attached if found
			FILTER& filter						//!< Method called with an rvc object to determine fitness
			) const;

		//! Update file information stored in caches to the file itself		
		ERRVALUE FlushFile (
			);

		//! Generate lock file names for given source file.
		//! This checks if the specified 'SourceFile' is actually an RVC file and if not, will
		//! determine if it is a supported auto-link format.  If supported then the lok filepath
		//! will be based on the name and location of the RVC file (.rlk) which links to the
		//! specifed 'SourceFile'.
		static ERRVALUE GenerateLockNames (
			const FILEPATH& SourceFile, 			//!< Source file path, may be RVC or "external" auto-linked file.
			FILEPATH& lokfilepath,					//!< Name of .lok file returned
			FILEPATH* kolfilepath = 0				//!< Optional .kol file returned
			);

		//! Generate a unique object name based on the passed in filter
		static ERRVALUE GenerateUniqueDescriptor (
			RVC::DESCRIPTOR& Descriptor,
			UNIQUENAMEFILTER& filter
			);

		//! Generate a unique object name based on the passed in filter
		static ERRVALUE GenerateUniqueName (
			RVC::OBJECTNAME& name,
			UNIQUENAMEFILTER& filter
			);

		//! Get the link file location for an auto-link file in the auto-link cache.
		static ERRVALUE GetAutoLinkCachePath (
			FILEPATH& rvcfilepath,
			const FILEPATH& extfilepath
			);
			
		//! Get the link file name for an auto-link file.
		static void GetAutoLinkLocalPath (
			FILEPATH& rvcfilepath,
			const FILEPATH& extfilepath
			);

		//! Get file name pattern list for automatic linking for specified object types.
		static void GetAutoLinkPatternList (
			const OBJTYPESET& objtypeset,			//!< Object types to determine patterns for
			MISTRINGLIST& patternlist				//!< Name search pattern list returned
			);
	
		//! Determine the number of children under this rvc object
		//!
		//!  Object must be opened for this method to work
		//! @return number of children found
		INT32 GetChildCount (
			RVC::OBJTYPE otype = RVC::OBJTYPE_All				//!< Type of rvc object to count, RVC::OBJTYPE_All to count all children
			) const;

		//! Determine the number of children under this rvc object
		//!
		//!  Object must be opened for this method to work
		//! @return number of children found
		INT32 GetChildCount (
			RVC::OBJTYPE otype,					//!< Type of rvc object to count, RVC::OBJTYPE_All to count all children
			FILTER& filter							//!< Method called with an rvc object to determine fitness
			) const;

		//! Determine the number of children under this rvc object
		//!
		//!  Object must be opened for this method to work
		//! @return number of children found
		INT32 GetChildList (
			RVC::OBJTYPE ObjType,				//!< Type of rvc object to count, RVC::OBJTYPE_All to count all children
			RVC::OBJITEMLIST& ObjItemList,	//!< List of children RETURNED
			FILTER* filter = 0					//!< Method called with an rvc object to determine fitness
			) const;

		//! Get the separator used between the parent and child in the GetObjectPath()'s 'ObjectPath' string 
		//! @return Character that separates the child identifier from the parent identifier in the object path string
		static char GetChildSeparator (
			) { return (' '); }

		//! Get the credentials used to open this instance
		//! @return The credentials used to open this instance
		const CREDENTIALS& GetCredentials (
			) const { return (m_Credentials); }
		
		//! Obtain the rvc object name
		//! The object must be opened for this method to work
		ERRVALUE GetDescriptor (
			RVC::DESCRIPTOR& Descriptor,			//!< Object identifier RETURNED
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			) const;

		//! Obtain the default object type for the object instance
		//! @return The default object type for 'this'.  The default value is RVC::OBJTYPE_File.
		RVC::OBJTYPE GetDftObjectType (
			) const;

		//! Get the description for external auto-linkable file.
		//! Does not create the RVC link file.
		static ERRVALUE GetExternalFileDescription (
			const FILEPATH& extfilepath, 			//!< Path to external (non-RVC) file
			MISTRING& description					//!< Description returned
			);
		
		//! Get the project link file that cooresponds to the auto-link file
		static ERRVALUE GetExternalProjectFileLink (
			const FILEPATH& extfilepath, 			//!< Path to external (non-RVC) file
			FILEPATH& ProjectFileLink				//!< Empty if no file exists
			);

		//! Fill in the 'ObjItem' based on 'Filename'.  This method will handle checks for valid project file
		//! and if an auto-linkable file, get its description and put it in the ObjItem DESCRIPTOR without doing 
		//! an actual auto-link.
		//! The 'ObjItem' will only be attached if it is an actual project file.
		static ERRVALUE GetFileObjItem (
			const FILEPATH& Filename, 
			RVC::OBJITEM& ObjItem,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			);
			
		//! Get the FILEPATH associated with this RVC::OBJECT instance
		//! @return The filepath associated with this object.  FILEPATH will be empty if the object is not attached 
		const FILEPATH& GetFilePath (
			) const;

		//! Get the FILEPATH linked file associated with this RVC::OBJECT instance
		//! @return The filepath linked file associated with this object.  FILEPATH will be empty if the object link is not attached 
		const FILEPATH& GetFilePathLink (
			) const;

		//! Get the RVCINODENUM associated with this RVC::OBJECT instance
		//! /deprecated Use other RVC wrapper classes
	#ifndef NO_DEPRECATED
		DEPRECATED RVCINODENUM GetInodeNum (				//! Internal inode number returned, for derived classes
			) const { return (GetInodeNumPro()); }
	#endif
	
		//! Obtain link file information for this object and/or children
		ERRVALUE GetLinkedFiles (
			LINKITEMLIST& LinkFileList,
#ifdef RVC_CREDENTIALS_MANDATORY
			LINKFILEFLAGS LinkFileFlags,
#else
			LINKFILEFLAGS LinkFileFlags = LINKFILEFLAG_None,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			) const;

		// Get the metadata for this object
		ERRVALUE GetMetadata (
			SIMPLE_ARRAY<UINT8>& buffer,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			) const;
			
		//! Obtain the icon name that goes with the OBJTYPE
		//! @return The icon name associated wit the object type, DO NOT FREE
		static const char* GetObjectIconName (
			OBJTYPE ObjType
			);

		//! Fill in the RVC::OBJITEM for the location of this object
		//!
		//! The object must be attached for this method to work
		DEPRECATED ERRVALUE GetObjectItem (
			RVC::OBJITEM& ObjectItem				//!< Object Item RETURNED
			) const;

		//! Determine the RVC Object Path from the root to this object
		//!
		//! The object must be attached for this method to work
		//! This rvc object path is defined to be a MIUNICODE string containing the heirarchy from the file root
		//! rvc object to the rvc object referenced by this instance.  It is used in specific Attach() and Open() methods.
		ERRVALUE GetObjectPath (
			MISTRING& ObjectPath				//!< Path to the rvc object from the root(file) rvc object.
			) const;

		//! Obtain object tag from the object type given
		//! @return The tag string used in the object path to denote the object type, DO NOT FREE
		static const char* GetObjectTagFromObjectType (
			RVC::OBJTYPE ObjType
			);

		//! Get the RVC::OBJTYPE_... associated with this RVC::OBJECT instance
		//! @return The RVC::OBJTYPE_...associated with this object.
		RVC::OBJTYPE GetObjectType (
			) const;
		
		//! Obtain object type from defined tag
		//! @return The RVC::OBJTYPE_...associated with this tag
		static RVC::OBJTYPE GetObjectTypeFromTag (
			const char* tag
			);

		//! Obtain object type from defined tag
		//! @return The RVC::OBJTYPE_...associated with this tag
		static RVC::OBJTYPE GetObjectTypeFromTag (
			const MIUNICODE* tag
			);

		//! Fill in the RVC::OBJITEM for the location of this object
		//!
		//! The object must be attached for this method to work
		ERRVALUE GetObjItem (
			RVC::OBJITEM& ObjectItem				//!< Object Item RETURNED
			) const;

		//! Obtain the parent rvc object.  If the rvc object is RVC::OBJTYPE_File, the parent rvc object returned is the same 
		//! as the rvc object
		ERRVALUE GetParent (
			RVC::OBJECT& ObjParent					//!< Parent object attached if sucessful
			) const { return (v_GetParent(ObjParent)); }

		//! Obtain the parent stack list.
		//! The list is generated in order of file, parent, parent, ..., this object.
		ERRVALUE GetParentStack (
			RVC::OBJITEMLIST& ParentStack					//!< Parent object attached if sucessful
			) const;

		//! Obtain the product version number which this rvc object was created
		ERRVALUE GetProductVersion (
			UINT16& ProductVersion				//!< TNT product version in the form of MMmm (M - Major, m - minor) RETURNED
			) const;

		//! Determine the current project file lock state
		//! @return The lock state of the project file at the time this method was called
		static LOCKSTATE GetProjectFileLockState (
			const FILEPATH& ProjectFile,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			);

		//! Obtain the size of this rvc object in bytes
		ERRVALUE GetSize (
			UINT64& size						//!< Size of opened rvc object RETURNED
			) const;

		//! Obtain the creation source string for this rvc object
		ERRVALUE GetSource (
			RVC::OBJECTSOURCE& Source				//!< RVC Object creation source name RETURNED
			) const;
			
		//! Obtain TEXTID from defined object type
		//! @return The TEXTID associated with the RVC::OBJTYPE_...
		static TEXTID GetTextIDFromObjectType (
			OBJTYPE ObjType
			);

		//! Obtain the creation time for this rvc object
		//!
		//! The  Object must be opened for this method to work
		void GetTimeCreate (
			DATETIME& Create					//!< Creation time RETURNED
			) const;

		//! Obtain the Last Modified Time for this rvc object
		//!
		//! The  Object must be opened for this method to work
		ERRVALUE GetTimeLastMod (
			DATETIME& LastMod					//!< Last modified time RETURNED
			) const;

		//! Obtain the Last Used Time for this rvc object
		//!
		//! The  Object must be opened for this method to work
		ERRVALUE GetTimeLastUsed (
			DATETIME& LastUsed					//!< Last used time RETURNED
			) const;
			
		//! Get the separator used between the object name and the object type for each object in the GetObjectPath()'s 'ObjectPath' string 
		//! @return Character that separates the object name from the object type in the object path string
		static char GetTypeSeparator (
			) { return ('.'); }
			
		//! Get the number of available undo levels
		//! @return Number of available undo levels
		INT32 GetUndoNumLevels (
			) const;

		//! Is this RVC::OBJECT attached to a RVC object?
		bool IsAttached (
			) const;
			
		//! Is this RVC::OBJECT a child of the given parent?
		bool IsChildOf (
			const RVC::OBJECT& parent				//!< Parent object to test
			) const;

		//! Is this RVC::OBJTYPE copyable?
		static bool IsCopyable (
			RVC::OBJTYPE ObjType
			);

		//! Do the two rvc object instances refer to the same RVC object?
		//! If either are unattached, this method returns false
	 	bool IsEqual (
			const RVC::OBJECT& rhs				//!< RVC Object to compare to
			) const;
			
		//! Is the file a link file or not
		//! @return True if the file is linked, false if not
		bool IsFileLinked (
			) const;

		//! Is this rvc object a root file rvc object?
		bool IsFileObject (
			) const;

		//! Is the object / file created by TNTlite
		//! @return True if the object / file is created by TNTlite, false if not
		bool IsLiteCreated (
			) const;

		//! Is the object a link object or not
		//! @return True if the object is linked, false if not
		bool IsObjectLinked (
			) const;

		//! Is the object older than the passed in object, using the last modified timestamp
		//! @return True if 'this' is older than 'Object', false if not
		bool IsOlderThan (
			const RVC::OBJECT& Object
			) const;

		//! Determine if rvc object is opened
		//! @return True if rvc object is opened
		bool IsOpen (
			) const;
			
		//! Determine if the RVC project file system is initialized
		//! @return 'True' if initialized, 'false' if not
		static bool IsSystemInitialized (
			);

		//! Determine if the attached rvc file is a temporary file
		//! @return True if file is temporary, False if not
		bool IsTempFile (
			) const;
		
		//! Is this rvc object a child of a folder or the root file rvc object?
		bool IsTopObject (
			) const;					

		//! Determine if 'this' is a valid child object of 'DestParentObj'.
		//! Both 'this' and 'DestParentObj' must exist as actual RVC objects
		//! @return '0' if parent is valid, <0 if not, use GetErrString() to determine why
		ERRVALUE IsValidParent (
			const RVC::OBJECT& DestParentObj
			) const;
		
		//! Determine that the file 'filename' is a valid project file
		//! @return 'True' if a valid project file, 'false' if not
		static bool IsValidProjectFile (
			const FILEPATH& filename
			);
		
		//! Is the rvc object opened for writing
		bool IsWritable (
			) const;

		//! Create a RVC file
		//!
		//! This method will create a project (RVC) file using the given location
		ERRVALUE MakeFile (
			const FILEPATH& FilePath,			//!< Place to create the file
			const MISTRING& desc					//!< Description to give the file
			);

		//! Create a RVC link object under 'this'
		//!
		//! This method will create a linked object (NewLinkObject) to an existing RVC object (LinkToObject)
		ERRVALUE MakeLink (
			const RVC::OBJECT& LinkToObject,
			RVC::OBJECT& NewLinkObject,			//!< Only attached to the new link, not opened
#ifdef RVC_CREDENTIALS_MANDATORY
			bool LinkToSubobjects,
#else
			bool LinkToSubobjects = false,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			);

		//! Create a RVC link object under 'this'
		//!
		//! This method will create a linked object (NewLinkObject) to an existing RVC object (LinkToObject)
		ERRVALUE MakeLink (
			const RVC::OBJECT& LinkToObject,
			RVC::OBJECT& NewLinkObject,			//!< Only attached to the new link, not opened
			const DESCRIPTOR& Descriptor,
#ifdef RVC_CREDENTIALS_MANDATORY
			bool LinkToSubobjects,
#else
			bool LinkToSubobjects = false,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			);

		//! Create a RVC file as a temporary file
		//!
		//! This method will create a temporary RVC file using the temporary file path 
		//! with the largest free space.
		ERRVALUE MakeTempFile (
			TEMPFILE flags = TEMPFILE_DeleteOnClose		//!< Determines if to delete the temp file on closing of the file, or when the process ends
			);

		//! Move object referenced by 'this' from its current parent to its new parent 'DestParentObj'.  The new parent must
		//! be in the same file as the current parent.  Does not perform a copy of the object; just adjusts the parent inode.
		//! Does not move DBTABLE objects.  Use RVC::DBASE::CopyTableTo() method.
		ERRVALUE MoveTo (
			RVC::OBJECT& DestParentObj,			//!< Parent object to move object and children to
#ifdef RVC_CREDENTIALS_MANDATORY
			COPYFLAGS CopyFlags,
#else
			COPYFLAGS CopyFlags = COPYFLAG_None,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			);

		//! Move object referenced by 'this' from its current parent to its new parent 'DestParentObj'.  The new parent must
		//! be in the same file as the current parent.  Does not perform a copy of the object; just adjusts the parent inode.
		//! Move 'this' from its current parent to its new parent 'DestParentObj'.
		//! Does not copy DBTABLE objects.  Use RVC::DBASE::CopyTableTo() method.
		ERRVALUE MoveTo (
			RVC::OBJECT& DestParentObj,			//!< Parent object to copy object and children to
			FILTER& filter,							//!< Method called with an rvc object to determine fitness to move
#ifdef RVC_CREDENTIALS_MANDATORY
			COPYFLAGS CopyFlags,
#else
			COPYFLAGS CopyFlags = COPYFLAG_None,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			);

		//! Determine if the rvc object that this instance is attached to requires
		//! that it be newer than its parent object.
		//! @return 'True' if parent needs to be older than this object, 'false' if not.
		bool NeedNewerThanParent (
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS							//!< User credentials passed in, NULL for default behavior
			);

		//! Open an <b>attached</b> RVC::OBJECT instance
		//!
		//! This method will open the file and derived rvc class object if possible
		//! If the instance is already open and the OPENMODE is not different, this
		//! method will return without doing anything, else it will Close() then Open()
		//! the instance.
		//! The instance must be attached for this to work
		ERRVALUE Open (
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,
			MDLGPARENT widget,
			const CREDENTIALS* Credentials 		//!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT widget = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials = 0	//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an RVC::OBJECT instance using another instance to the same rvc object.
		//!
		//! This method will attach and open the file and derived rvc class object if possible.
		//! Use this method to re-open the rvc object using a different OPENMODE.
		ERRVALUE Open (
			const RVC::OBJECT& twin,				//!< Identical rvc object to open with a different OPENMODE
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT widget,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT widget = 0,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials = 0	//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an RVC::OBJECT instance using the rvc object's parent, type of rvc object, and the rvc object's name
		//!
		//! This method will attach and locate the rvc object and open the file and derived rvc class object if possible
		ERRVALUE Open (
			const RVC::OBJECT& parent,			//!< Parent rvc object to search under
			const RVC::OBJTYPE ObjType,				//!< Type of rvc object to look for
			const RVC::DESCRIPTOR& Descriptor,
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT widget,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT widget = 0,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials = 0	//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an RVC::OBJECT instance to a rvc root folder using the given file location
		//!
		//! This method will attach and open the file
		//! If the class is derived, this function will fail and return an error.
		ERRVALUE Open (
			const FILEPATH& FilePath,			//!< Location of file to attach to
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT widget,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT widget = 0,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials = 0	//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an RVC::OBJECT instance using a RVC::OBJITEM instance
		//!
		//! This method will attach and open the file and derived rvc class object if possible
		ERRVALUE Open (
			const RVC::OBJITEM& ObjItem,			//!< This contains the file and rvc object location information
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT widget,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT widget = 0,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials = 0	//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an RVC::OBJECT instance using the given file location and rvc object path
		//!
		//! This method will attach and locate the rvc object and open the file and derived rvc class object if possible
		ERRVALUE Open (
			const FILEPATH& FilePath,			//!< Location of file to open
			const MISTRING& ObjectPath,			//!< Full name of rvc object obtained from GetObjectPath()
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT widget,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT widget = 0,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials = 0	//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an RVC::OBJECT instance using a source rvc object, possible relative path, and rvc object path
		//!
		//! This method will attach and open the file and derived rvc class object if possible
		ERRVALUE Open (
			const RVC::OBJECT& source,			//!< Source rvc object used if 'filename' is a relative path
			const MIUNICODE *filename,			//!< Full or relative path to the rvc file to open
			const MISTRING& ObjectPath,			//!< Full name of rvc object obtained from GetObjectPath()
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT widget,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT widget = 0,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials = 0	//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open a parent of an RVC::OBJECT instance using a RVC::OBJITEM instance
		//! This method will attach and open the file and derived rvc class object if possible
		ERRVALUE OpenParent (
			const RVC::OBJITEM& ObjItem,			//!< This contains the file and rvc object location information
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT widget,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT widget = 0,				//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials = 0	//!< User credentials passed in, NULL for default behavior
#endif
			);
			
		//! Pack Project File.  
		//! Removes the free space present in the project file.
		static ERRVALUE PackProjectFile (
			const FILEPATH& ProjectFile,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Perform project file recovery to the output object specified
		ERRVALUE RecoverProjectFileTo (
			RVC::OBJECT& OutputObject,
			MISTRING& LogString,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS	//!< User credentials passed in, NULL for default behavior
			);
	
		//! Redo (reinstate) changes made by an undo operation, if enabled
		ERRVALUE RedoChanges (
			);
			
		//! Save an object to a destination.  If the 'DestObjItem' refers to an existing object, then a new object is created and if
		//! that succeeds, the original is deleted, otherwise the original is kept.  If the 'DestObjItem' does not exist, then it is a simple
		//! copy of the source object.
		//! Does not copy DBTABLE objects.  Use RVC::DBASE::CopyTableTo() method.
		ERRVALUE SaveObjectTo (
			RVC::OBJITEM& DestObjItem,							//!< Parent object to copy object and children to, new destination object RETURNED
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			) const;

		//! Search children of 'this' for any children that are invalid due to:
		//! 1.  Violation of parent object rules (child cannot be under this parent object)
		//! 2.  Singleton violation:  There can be only one object of this type under the parent
		ERRVALUE SearchForInvalidChildObjects (
			bool SearchAllSubLevels,							//!< 'false' to return only the first level children, true to do levels of children
			OBJITEMLIST& InvalidChildList,					//!< List of children that violate parent object rules  (child cannot be under this parent object)
			OBJITEMLIST& SingletonChildList,					//!< Singleton violation:  There can be only one object of this type under the parent
			OBJITEMLIST& OutOfDateChildList,					//!< Out of date violation:  The object MUST have a last modified date later than its parent object
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			) const;

		//! Disable credential query dialog from appearing.  The credential query dialog is the username / password prompt dialog.
		static void SetDisableCredentialQuery (
			bool value
			);
			
		// Get the metadata for this object
		ERRVALUE SetMetadata (
			const SIMPLE_ARRAY<UINT8>& buffer,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);
			
		//! Set the minimum version of the TNT products that can view this project file.
		//! The form of the 'Version' parameter is 6900, 7000, 7100, etc...
		ERRVALUE SetMinimumTNTVersion (
#ifdef RVC_CREDENTIALS_MANDATORY
			UINT16 Version,				//!< Default is to the current version
#else
			UINT16 Version = 0,			//!< Default is to the current version
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Enable or disable thread synchronization in rvcsys.dll.
		//! Calling these or derived class methods from multiple threads requires that multi-thread mode be set to true.
		//! RVC::OBJECT and derived class instances cannot be shared between threads, each thread MUST have its own.
		//! Default upon startup is 'false' for multi-thread mode.
		static void SetMultiThreadMode (
			bool value
			);

		//! Modify the rvc object's TimeLastUsed field to the current time
		//!
		//! The object must be opened for writing
		ERRVALUE SetTimeLastUsed (
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Modify the rvc object's TimeLastUsed field to the time given
		//!
		//! The object must be opened for writing
		ERRVALUE SetTimeLastUsed (
			const DATETIME& NewTime,				//!< Date / Time to set the rvc object's "last used" field
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Modify the rvc object's TimeLastModified field to the current time
		//!
		//! The object must be opened for writing
		ERRVALUE SetTimeLastMod (
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Modify the rvc object's TimeLastModified field to the time given
		//!
		//! The rvc object must be opened for writing
		ERRVALUE SetTimeLastMod (
			const DATETIME& NewTime,				//!< Date / Time to set the rvc object's "last modified" field
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Set the maximum number of undo levels
		ERRVALUE SetUndoMaxLevel (
			INT32 NewLevel
			);

		//! Test if usable for specified limit mode.
		//! Does not check subobjects.
		//! @return TRUE if usable, FALSE if not, < 0 if error.
		static int TestUsableForLimit (
			const RVC::OBJITEM& objitem,
			UINT32 limitmode
			);
			
		//! Test if object usabe in current "limited" mode, if any.
		//! Returns error if not usable or 0 if usable.
		//! If not running limited (Free/Basic) mode will always return 0.
		static ERRVALUE TestUsableInLite (
			const RVC::OBJITEM& objitem
			);
	
		//! Method to traverse an object link if 'this' references a RVC object link.
		//! If 'this' references a link, 'TraversedObject' will be attached to the linked to object, if 'this'
		//! is not a link, 'TraversedObject' will be attached to 'this'.
		//! @return '1' if the link existed and was traversed, '0' if not a link, < 0 error
		ERRVALUE TraverseLink (
			RVC::OBJECT& TraversedObject
			);
			
		//! Undo changes made from the last undo mark		
		ERRVALUE UndoChanges (
			bool SetRedo = false					//!< Set redo information to undo the undo
			);

		//! Clear out all undo information		
		ERRVALUE UndoClear (
			);
			
		//! Mark a spot for an undo call to undo to.  This first call to this function
		//! marks the initial position for undo and enables undo for this file
		ERRVALUE UndoMark (
			);
		
		//! Update file information stored in object caches to the file itself		
		ERRVALUE UpdateFile (
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Update "Last Used" time to be later than any like object.  Only updates the object if necessary.
		//! Guarentees this object will be selected be FindChildLastUsed()
		ERRVALUE UpdateTimeLastUsed (
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);
			
		//! Determine the status of the object to file mapping layer.  If the 'LogString' is empty, then the mapping is valid.
		ERRVALUE ValidateFileMapping (
			MISTRING& LogString
			);
			
	protected:

		//! Called by the CopyTo() method to copy the derived class, if possible and necessary
		virtual ERRVALUE v_CopyObject (
			RVC::OBJECT& DestParentObj,			//!< Parent object to copy object and children to
			RVC::OBJECT& DestObj,					//!< Attached instance to destination object
			FILTER* filter,							//!< Method called with an rvc object to determine fitness to copy
			COPYFLAGS CopyFlags,
			const CREDENTIALS* Credentials		//!< User credentials passed in, NULL for default behavior
			) const;

		virtual ERRVALUE v_GetDescriptor (DESCRIPTOR& Descriptor, const CREDENTIALS* Credentials) const;

		void AssignInfo (const GENINFO& info);
		void AssignInode (RVCINODENUM inode);
		ERRVALUE AttachLow (const FILEPATH& FilePath, RVCINODENUM InodeNum, OPENMODE OpenMode, MDLGPARENT parent, const RVC::CREDENTIALS* Credentials);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		void CallOnACLChanged ();
		void CallOnAttach ();
		void CallOnChildClosing (const OBJECT& object);
		void CallOnChildCreate (const OBJECT& object);
		void CallOnChildDestroying (const OBJECT& object);
		void CallOnChildOpen (const OBJECT& object);
		void CallOnClose ();
		void CallOnDecendentDescriptorChanged (const OBJECT& object);
		void CallOnDescriptorChanged ();
		void CallOnDestroy ();
		void CallOnDetach ();
		void CallOnNotifyRaw (UINT32 reason, void *data = 0);
		void CallOnUndoBegin ();
		void CallOnUndoEnd ();
		void CallOnUndoPossible ();
		#endif // GENERATING_DOXYGEN_OUTPUT
		
		ERRVALUE CloseFile (			  		//! Close the open file
			);

		void CopyObject (
			const RVC::OBJECT& rhs
			);
			
		//! Get the RVC file handle associated with this RVCRVC::OBJECT instance
		//! The instance must be open to get a valid handle
		RVC::FILEHANDLE* GetFileInstance (						//! For derived classes, gets the internal file handle
			) const;
		
		//! Get the RVCINODENUM associated with this RVC::OBJECT instance
		//! @return Internal inode number
		static RVCINODENUM GetInodeNum (				
			const RVC::OBJITEM& ObjItem				//!< OBJITEM to get the inode from
			);
		
		//! Get the RVCINODENUM associated with this RVC::OBJECT instance
		//! @return Internal inode number
		RVCINODENUM GetInodeNumPro (
			) const;
		
		//! Fill in the RVC::OBJITEM for the location of 'InodeNum' and the project filepath of 'this'
		//!
		//! The object must be attached for this method to work
		ERRVALUE GetObjItem (
			RVCINODENUM InodeNum,			//!< Inode number of a different object in the same file
			OBJITEM& ObjectItem				//!< Object Item RETURNED
			) const;

		//! Fill in the RVC::OBJITEM for the location of 'FilePath' and 'InodeNum'
		static ERRVALUE GetObjItem (
			const FILEPATH& FilePath,
			RVCINODENUM InodeNum,
			OBJITEM& ObjectItem				//!< Object Item RETURNED
			);

		//! Obtain current open mode for this rvc object instance.
		//! Used in copy constructors and assignment operators of derived classes to open the rvc object using
		//! the same OPENMODE as the 'rhs' rvc object instance
		//!
		//! @return Open mode of rvc object instance
		OPENMODE GetOpenMode (				//! Returns open object read or write, useful for copy and assignment
			) const;
		
		bool IsFileOpen (						//! Is the file opened?
			) const;
			
		bool IsObjectOpen (
			const OBJECT& Object
			) const;

		ERRVALUE OpenFile (				  		//! Open file only
			OPENMODE OpenMode,
			MDLGPARENT parent,
			const CREDENTIALS* Credentials						//!< User credentials passed in, NULL for default behavior
			);

		ERRVALUE OpenRVCFile (const FILEPATH& filepath, OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);
		ERRVALUE OpenConnection (const MISTRING& Connection, const MISTRING& Title, OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);
		ERRVALUE ReadObjectHeader (GENINFO* ginfo, int HeaderSize, int HeaderType, const CREDENTIALS* Credentials) const;

		void SetInodeNum (	
			RVCINODENUM inode
			) { m_InodeNum = inode; }

		void SetMakeObject (const RVC::OBJECT& rhs);

		void SetMakeParms (
			OBJMAKEPARMS& ObjMakeParms,
			const RVC::OBJECT& parent,
			const RVC::DESCRIPTOR& Descriptor, 
			GENINFO* newinfo, 
			int HeaderSize, 
			int HeaderType,
			const CREDENTIALS* Credentials
			); 

		void SetOpenParms (
			OBJOPENPARMS& ObjOpenParms,
			const RVC::OBJECT& object,
			OPENMODE OpenMode,
			const CREDENTIALS* Credentials
			); 

		ERRVALUE SetParent (const OBJITEM& ObjItem);
			
		UINT32 TranslateOpenFlags (				//! Translates OPENMODE to OBJECTACCESS_... flags
			OPENMODE flags
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		FILEPATH m_FilePath;		//!< File to re-open if closed
		FILEHANDLE *m_FileHandle;
		bool m_Attached;
		bool m_IsWritable;
		bool m_CanWrite;
		CREDENTIALS m_Credentials;

		// Members listed below are the members of GENINFO that are inmutable for multi-threading purposes
		RVCDATETIME m_Create;
		OBJTYPE m_ObjectType;
		RVCINODENUM m_InodeNum;
		
		class TEMPOPENOBJECT;
		friend class TEMPOPENOBJECT;
		class FILTERTYPE;
		friend class FILTERTYPE;
		class FILTERFUNC;
		friend class FILTERFUNC;

		OBJECT (							//! Constructor by file handle and inode number, used for internal creation
			const FILEHANDLE* FileHandle,
			RVCINODENUM InodeNum
			);

		bool operator== (const OBJECT& rhs) const;

		ERRVALUE AttachLow (const OBJITEM& ObjItem, OPENMODE OpenMode, MDLGPARENT parent, const RVC::CREDENTIALS* Credentials);
		ERRVALUE AttachLow (const FILEPATH& FilePath, const MISTRING& ObjectPath, OPENMODE OpenMode, MDLGPARENT parent, const RVC::CREDENTIALS* Credentials);
		ERRVALUE AttachLow (const OBJECT& parent, const OBJTYPE ObjType, const DESCRIPTOR& Descriptor, OPENMODE OpenMode, MDLGPARENT widget, const RVC::CREDENTIALS* Credentials);
		ERRVALUE AttachLow (const OBJECT& parent, const OBJTYPE ObjType, const OBJECTNAME& Name, OPENMODE OpenMode, MDLGPARENT widget, const RVC::CREDENTIALS* Credentials);
		ERRVALUE AttachLow (const OBJECT& source, const MIUNICODE *filename, const MISTRING& ObjectPath, OPENMODE OpenMode, MDLGPARENT parent, const RVC::CREDENTIALS* Credentials);

		static void CacheAdd (const FILEPATH& filepath, const CREDENTIALS *pCredentials);
		static void CacheDisable ();
		static void CacheEnable ();

		static void CallbackFunc (ANYCALLBACKSTRUCT* cb, void *data);

		#endif // GENERATING_DOXYGEN_OUTPUT
		
		//! Called by the Close() method to close the derived class open handle, if possible and necessary
		//!
		//! If the derived class retains an open state with the rvc object, then the derived class MUST support this method
		//! The default behaviour is to return (0).
		//!
		//! If this method is supported, then the derived class must support IsObjectOpen() and OpenObject().
		virtual ERRVALUE v_CloseObject (
			);

		//! Called whenever a default object type is needed for comparison or default parameters.
		//!
		//! All derived classes that model a specific object must override this method and provide a correct
		//! RVC::OBJTYPE_... value.  The default value is RVC::OBJTYPE_File.
		virtual RVC::OBJTYPE v_GetDftObjectType (
			) const;

		//! Used in FindChildLastUsed() if the child object passed in needs a filter to get the correct project file object.
		//! @return A pointer to an OBJECT::FILTER instance allocated using new().  The caller will free the instance.  NULL if no filter is needed, the default.
		virtual FILTER* v_GetObjectFilter (
			) const;

		//! Called when parent object is needed.  Default operation is to get the parent object.
		virtual ERRVALUE v_GetParent (
			RVC::OBJECT& ObjParent
			) const;

		//! Can the derived class handle multiple object types.
		//!
		//! Used in caes where the object instance determines the object to look for
		virtual bool v_IsMultiObject (
			) const;

		//! Called by the IsOpen() method to determine if the rvc object is open, if possible and necessary.
		//!
		//! If the derived class retains an open state with the rvc object, then the derived class MUST support this method
		//! The default behaviour is to return (true).
		//!
		//! If this method is supported, then the derived class must support CloseObject() and OpenObject().
		virtual bool v_IsObjectOpen (
			) const;					

		//! Called by the Open() methods to open an rvc object.
		//!
		//! If the derived class retains an open state with the rvc object, then the derived class MUST support this method
		//! The default behaviour is to return (0).
		//!
		//! If this method is supported, then the derived class must support CloseObject() and IsObjectOpen().
		virtual ERRVALUE v_OpenObject (
			OPENMODE OpenMode,
			MDLGPARENT parent,
			const CREDENTIALS* Credentials
			);

		//! Called to determine if the derived class can handle the specified object type
		//!
		//! All derived classes that model a specific object must override this method and test the validity of the 
		//! RVC::OBJTYPE_... value passed in.  The default value is true, since the RVC::OBJECT class can handle any object type
		virtual bool v_ValidateObjectType (
			const RVC::OBJTYPE otype
			) const;

	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(OBJECT::COPYFLAGS)
#endif


//! Constant iterator class
//!
//! This class is used to iterate through a parent rvc object's children.  This construct replaces the MfGetList() routines.
//! See methods ChildListBegin() and ChildListEnd() for obtaining an child iterator
class OBJECT::CONST_ITERATOR {
	public:

		//! Default constructor
		CONST_ITERATOR (
			) : 
			m_Iterator(0),
			m_CurrentObj(0),
			m_ObjType(OBJTYPE_All)
			{}

		//! Idiot constructor for G++
		explicit CONST_ITERATOR (
			int
			) : 
			m_Iterator(0),
			m_CurrentObj(0),
			m_ObjType(OBJTYPE_All)
			{}

		//! Copy constructor
		CONST_ITERATOR (
			const CONST_ITERATOR& rhs
			);

		//! Interanl constructor used by ChildListBegin() methods
		CONST_ITERATOR (
			const RVC::OBJECT* parent,
			RVC::OBJTYPE m_ObjType
			);

		//! Destructor
		virtual ~CONST_ITERATOR (
			);

		//! Assignment
		CONST_ITERATOR& operator= (
			const CONST_ITERATOR& rhs
			);

		//! Dereference
		const RVC::OBJECT& operator* (
			) const {return (*m_CurrentObj);}

		//! Dereference arrow
		const RVC::OBJECT* operator-> (
			) const {return (&**this);}

		//! Pre-increment
		CONST_ITERATOR& operator++ (
			);

		//! Comparison for equality
		bool operator== (
			const CONST_ITERATOR& rhs
			) const {return (m_Iterator == rhs.m_Iterator);}

		//! Comparison for inequality
		bool operator!= (
			const CONST_ITERATOR& rhs
			) const {return (!(*this == rhs));}

	protected:
		void *m_Iterator;
		OBJECT *m_CurrentObj;
		OBJTYPE m_ObjType;
	};


//! Iterator class
//!
//! This class is used to iterate through a parent rvc object's children.  This construct replaces the MfGetList() routines.
//! See methods ChildListBegin() and ChildListEnd() for obtaining an child iterator
class OBJECT::ITERATOR : public OBJECT::CONST_ITERATOR {
	public:
		//! Default constructor
		ITERATOR (
			) : CONST_ITERATOR() {}

		//! Idiot constructor for G++
		explicit ITERATOR (
			int
			) : CONST_ITERATOR() {}

		//! Copy constructor
		ITERATOR (
			CONST_ITERATOR& rhs
			) : CONST_ITERATOR(rhs) {}

		//! Interanl constructor used by ChildListBegin() methods
		ITERATOR (
			const RVC::OBJECT* parent,
			RVC::OBJTYPE ObjType
			) : CONST_ITERATOR(parent, ObjType) {}

		//! Dereference
		RVC::OBJECT& operator* (
			) const {return (*m_CurrentObj);}

		//! Dereference arrow
		RVC::OBJECT* operator-> (
			) const {return (&**this);}

		//! Pre-increment
		ITERATOR& operator++ (
			) {
			CONST_ITERATOR::operator++();
			return (*this);
			}

		//! Comparison for equality
		bool operator== (
			const ITERATOR& rhs
			) const {return (m_Iterator == rhs.m_Iterator);}

		//! Comparison for inequality
		bool operator!= (
			const ITERATOR& obj
			) const {return (!(*this == obj));}

	};

		
//! Base RVC observer class.
//! Explination:  Rather than have specific subjects for specific objects and derived objects, which would cause some confusion
//! as to which subject was being monitored, we placed "the RVC" subject in RVCOBJECT.  Hence we have placed a base RVC::OBJECT::OBSERVER class
//! here.  Any other classes that need subject/observer behavior need to inherit from OBJECT::OBSERVER and support the OnNotifyRaw()
//! method and classify as private.  This allows any derived class to support its own notification plus any parent class notification.  In this
//! case, the OBJECT can report Undo and Close notifications.
class OBJECTOBSERVER : public OBSERVER_BASE {
	public:

		//! Constructor
		explicit OBJECTOBSERVER (
			OBJECT& object									//!< Object to observe
			) :
			OBSERVER_BASE(object)
			{}
			
		//! Destructor
		virtual ~OBJECTOBSERVER (
			) {}

		//! Called when the Access Control List has changed for this object			
		virtual void OnACLChanged (
			) {return;}
			
		//! Called when the ctor's 'object' is attached to an actual RVC object
		virtual void OnAttach (
			) { return;}
		
		//! Called when a child object is about to be closed
		virtual void OnChildClosing (
			const OBJECT&									//!< Object to be closed
			) {return;}
			
		//! Called when a child object is created
		//! NOTE: If this a component object, Eg: database object, the object that is created is not fully developed
		//! and methods for this object should not be used.
		virtual void OnChildCreate (
			const OBJECT&									//!< Created object
			) {return;}
			
		//! Called when a child object is about to be destroyed.
		//! If the object is opened in this method override, then the deletion of the object will fail
		virtual void OnChildDestroying (
			const OBJECT&									//!< Object to be deleted
			) {return;}
			
		//! Called when a child object is opened
		//! NOTE: If this a component object, Eg: database object, the object that is opened is not fully developed
		//! and methods for this object should not be used.
		virtual void OnChildOpen (
			const OBJECT&									//!< Opened object
			) {return;}
			
		//! Called when the ctor's 'object' is closed
		virtual void OnClose (
			) {return;}
			
		//! Called when a decendant object's descriptor has changed
		virtual void OnDecendentDescriptorChanged (
			const OBJECT&									//!< Object that had its decriptor changed
			) {return;}
			
		//! Called when the ctor's 'object' descriptor has changed
		virtual void OnDescriptorChanged (
			) {return;}
			
		//! Called when the ctor's 'object' is about to be destroyed (the object's dtor is called)
		virtual void OnDestroy (
			) { return;}
		
		//! Called when the ctor's 'object' is about to be detached from an actual RVC object
		virtual void OnDetach (
			) { return;}
		
		//! Called when a 'Undo' operation has begun on the project file
		virtual void OnUndoBegin (
			) {return;}
			
		//! Called when a 'Undo' operation has ended on the project file
		virtual void OnUndoEnd (
			) {return;}
			
		//! Called when a 'Undo' operation is possible on the project file
		virtual void OnUndoPossible (
			) {return;}
			
	protected:		

		virtual void OnNotifyRaw (
			UINT32,							//!< Reason
			void *							//!< data
			) {return;}
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		OBJECTOBSERVER (const OBJECTOBSERVER& rhs);
		OBJECTOBSERVER& operator= (const OBJECTOBSERVER& rhs);

		friend class OBJECT;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	};


//! Base class for method filters
//!
//! This class is used to define a common method for filters to call for filtering rvc objects.
//! This class must be inherited from.
//!
//!	See example: class OBJECT::FILTER_NEWERTAHNPARENT

class OBJECT::FILTER {
	public:
		//! Default constructor
		FILTER () {};

		//! Destructor
		virtual ~FILTER () {};

		//! Inherited filter method to determine validity of the rvc object passed in
		//! @return true to accept the rvc object, false to reject
		virtual bool Filter (
			const RVC::OBJECT& obj
			) = 0;
	};


//! Interface class to filter out not unique names based on the starting name
class OBJECT::UNIQUENAMEFILTER {
	public:
	
		//! Inherited filter method to determine if the name passed to it is unique
		//! @return '0' if the name is not unique, > 0 if the name is unique, < 0 if error
		virtual int Filter (
			const RVC::OBJECTNAME& name
			) = 0;
	};
	

class OBJECT::UNIQUECHILDNAMEFILTER : public OBJECT::UNIQUENAMEFILTER {
	public:
		UNIQUECHILDNAMEFILTER (
			const RVC::OBJECT& ParentObj,
			RVC::OBJTYPE ObjType
			);
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const RVC::OBJECT& m_ParentObj;
		RVC::OBJTYPE m_ObjType;
	
		virtual int Filter (const RVC::OBJECTNAME& name);
		
		UNIQUECHILDNAMEFILTER& operator= (const UNIQUECHILDNAMEFILTER&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	

//! Class used by methods that perform creation of top level objects outside of CopyObjectTo() methods
//! to assist the copying of link objects when the link should refer to the destination object but the 
//! name of the top level object has changed
class OBJECT::TOPLEVELNAMECHANGE {
	public:
		TOPLEVELNAMECHANGE (
			const OBJECT& TopSrcObject,
			const OBJECT& TopDestObject
			);
	
		~TOPLEVELNAMECHANGE (
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const OBJECT& m_TopSrcObject;

		TOPLEVELNAMECHANGE& operator= (const TOPLEVELNAMECHANGE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Class filter designed to select objects that are newer than their parent
class OBJECT::FILTER_NEWERTHANPARENT : public OBJECT::FILTER {
	public:

		virtual bool Filter (
			const OBJECT& obj
			) {
			OBJECT parent;
			return (obj.GetParent(parent) >= 0 && !obj.IsOlderThan(parent));
			}
	};

//! Class filter designed to select objects based on an OBJTYPESET
class OBJECT::FILTER_OBJTYPESET : public OBJECT::FILTER {
	public:
		//! Constructor
		FILTER_OBJTYPESET (
			const OBJTYPESET& ObjTypes
			) : m_ObjTypes(ObjTypes) { }

		//! Virtual filter method
		virtual bool Filter (
			const OBJECT& obj
			) { return (m_ObjTypes.Contains(obj.GetObjectType())); }

		void SetTypes (
			const OBJTYPESET& ObjTypes
			) { m_ObjTypes = ObjTypes; }
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		OBJTYPESET m_ObjTypes;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#endif		// INC_RVC_OBJECT_H
