/**
 * \file rvc/objtype.h
 * \brief Define RVC::OBJTYPE enumeration
 *
 * \if NODOC
 * $Id: objtype.h_v 1.33 2012/01/20 15:41:08 mju Exp $
 *
 * $Log: objtype.h_v $
 * Revision 1.33  2012/01/20 15:41:08  mju
 * Add terrainTileset.
 *
 * Revision 1.32  2011/11/14 20:58:56  mju
 * Add objtype_Remote.
 *
 * Revision 1.31  2010/11/19 22:49:41  bcolombe
 * Added RemoteTileset, RemoteKML, WMS, ArcIMS
 *
 * Revision 1.30  2010/05/20 20:11:17  mju
 * Add objtype_stat_DbField.
 *
 * Revision 1.29  2010/04/12 20:39:52  mju
 * Remove objtype_QuadSearchTree as replaced by quadSearchTree_Legacy.
 *
 * Revision 1.28  2010/04/12 20:34:07  mju
 * Remove objtype_QuadSearchTree2 as no longer used.
 *
 * Revision 1.27  2010/04/12 16:19:31  mju
 * Remove unused objtype_QuadSearchTree3 as replaced by ..._Set.
 *
 * Revision 1.26  2010/04/12 16:04:43  mju
 * Use named subtypes for quadSearchTree instead of numbers.
 *
 * Revision 1.25  2010/04/02 20:08:52  mju
 * Add quadSearchTree3.
 *
 * Revision 1.24  2009/11/12 16:35:04  mju
 * Add gaps to objtype enum so easier to use as index.
 *
 * Revision 1.23  2008/12/15 18:44:35  scowan
 * Changed file referenced to change when entry added.
 *
 * Revision 1.22  2007/06/29 15:10:05  scowan
 * Added Raster Theme object type.
 *
 * Revision 1.21  2007/02/26 17:48:50  scowan
 * Inserted digitizer objtype.
 *
 * Revision 1.20  2006/04/14 20:31:49  scowan
 * Added container object type.
 *
 * Revision 1.19  2005/12/12 18:13:34  scowan
 * Removed unused enum.
 *
 * Revision 1.18  2005/11/08 16:24:41  scowan
 * Added mosaic layout objtype.
 *
 * Revision 1.17  2005/10/13 20:19:34  scowan
 * Added a new object type.
 *
 * Revision 1.16  2005/02/24 18:30:14  scowan
 * Added refernce file object types.
 *
 * Revision 1.15  2004/03/10 22:36:50  mju
 * Add OBJTYPE_Display_Anim3D.
 *
 * Revision 1.14  2003/12/26 18:00:07  scowan
 * Added object type entry..
 *
 * Revision 1.13  2003/09/25 21:56:58  dwilliss
 * doxygen
 *
 * Revision 1.12  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.11  2003/05/13 17:24:03  scowan
 * Added new object type.
 *
 * Revision 1.10  2003/05/07 14:50:37  scowan
 * Added new object types.
 *
 * Revision 1.9  2003/04/09 19:18:19  scowan
 * Added shape object type.
 *
 * Revision 1.8  2002/05/10 20:01:43  dwilliss
 * Define the enum for OBJTYPE_DBIndex
 *
 * Revision 1.7  2001/11/01 17:00:01  scowan
 * Added new object type.
 *
 * Revision 1.6  2001/09/13 19:49:41  mju
 * Change INT32 to int.
 *
 * Revision 1.5  2001/07/05 13:03:57  mju
 * No ; needed at end of namespace.
 *
 * Revision 1.4  2001/06/14 21:52:16  mju
 * Add increment operator.
 *
 * Revision 1.3  2001/06/13 16:19:13  mju
 * Put in RVC namespace.
 *
 * Revision 1.2  2001/06/13 14:45:58  mju
 * Spell out most object type names.
 *
 * Revision 1.1  2001/06/13 14:05:13  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_OBJTYPE_H
#define INC_RVC_OBJTYPE_H

namespace RVC {

//! Enumerations for RVC object types.
// Note that gaps are *not* to be reused, but are shown so if use objtype as index will allow for
enum OBJTYPE {
	OBJTYPE_All = 0,								//!< Used to represent "all" objects in selection/filters
	OBJTYPE_File = 1,								//!< RVC file itself
	OBJTYPE_Folder = 2,							//!< Folder
	OBJTYPE_Raster = 3,							//!< Raster
	OBJTYPE_Vector = 4,							//!< Vector
	OBJTYPE_CAD = 5,								//!< CAD
	OBJTYPE_Database = 6,						//!< General database
	OBJTYPE_Style = 7,							//!< Drawing style
	OBJTYPE_Script_Query = 8,					//!< Generic database query string
	OBJTYPE_Filter = 9,							//!< Spatial filter description

	OBJTYPE_Georeference = 11,					//!< Georeference information

	OBJTYPE_HyperIndex = 13,					//!< Hyper-index information
	OBJTYPE_DatabaseForm = 14,					//!< Database display layout
	OBJTYPE_TiePoint = 15,						//!< Tie point list
	OBJTYPE_Histogram = 16,						//!< Raster histogram
	OBJTYPE_ContrastTable = 17, 				//!< Contrasting table

	OBJTYPE_ColorPalette = 19,					//!< Color table for rasters
	OBJTYPE_Database_Raster = 20,				//!< Database for raster cells
	OBJTYPE_FreqProcInfo = 21,					//!< Information to match frequency-domain rasters
	OBJTYPE_RasterTrend = 22,					//!< Trend Information
	OBJTYPE_SpotHeader = 23, 					//!< Header (leader) file from SPOT tape
	OBJTYPE_CameraCalib = 24,					//!< Camera calibration information
	OBJTYPE_Database_VectorPoint = 25,		//!< Database for point elements
	OBJTYPE_Database_VectorLine = 26,		//!< Database for line elements
	OBJTYPE_Database_VectorPolygon = 27,	//!< Database for polygon elements

	OBJTYPE_Database_CAD = 29,					//!< Database for CAD
	OBJTYPE_PrinterColorBalance = 30,		//!< Printer color balance
	OBJTYPE_PrintDefinitions = 31,			//!< Description for raster printer devices

	OBJTYPE_OutlineFont = 33,					//!< Legacy font definitions
	OBJTYPE_GenericRef = 34,					//!< Generic reference object, not user-viewable object, used for temporary objects

	OBJTYPE_XYDigitizerDefinitions = 36,	//!< XY Digitizer Defintions object
	OBJTYPE_Display_Group = 37, 				//!< Display group layout information
	OBJTYPE_Disp2DWindow = 38,					//!< Old display window layout information
	OBJTYPE_TIN = 39,								//!< TIN
	OBJTYPE_DispParm = 40,						//!< Display parameter structure
	OBJTYPE_Legend = 41,  						//!< Legend
	OBJTYPE_Text = 42,							//!< Generic Text
	OBJTYPE_Database_TIN = 43,					//!< Database for TIN

	OBJTYPE_Region = 45,							//!< Region
	OBJTYPE_Association = 46,					//!< External file association
	OBJTYPE_QuadSearchTree_Legacy = 47,		//!< Search Quad Tree - legacy structure
	OBJTYPE_DatabaseParm = 48,					//!< Database parameters
	OBJTYPE_Theme = 49,							//!< Theme Mapping parameters
	OBJTYPE_Metadata = 50,						//!< Metadata
	OBJTYPE_Table = 51,							//!< Table in database
	OBJTYPE_DatabaseInverseList = 52,		//!< Database table-to-element list
	OBJTYPE_EditParm = 53,						//!< Object Editor parameter structure
	OBJTYPE_Display_Layout = 54,				//!< Display layout information
	OBJTYPE_Script_SML = 55,					//!< Stand-alone SML Script
	OBJTYPE_Spectral = 56,						//!< Spectral
	OBJTYPE_RasterSet = 57,						//!< Raster Set

	OBJTYPE_Display_Sim3D = 59,				//!< Display 3D animation parameters
	OBJTYPE_Hyperspectral = 60,				//!< Hyperspectral
	OBJTYPE_DatabaseConstraint = 61,			//!< Database table constraint information
	OBJTYPE_HyperspectralHistogram = 62,	//!< Hyperspectral histogram
	OBJTYPE_HyperspectralSensorInfo = 63,	//!< Hyperspectral sensor
	OBJTYPE_SensorInfo = 64,					//!< Sensor calibration information for spectral bands
	OBJTYPE_LayoutTemplate = 65,				//!< Layout template
	OBJTYPE_Thumbnail = 66,						//!< Thumbnail container

	OBJTYPE_DBIndex = 68,						//!< Database index

	OBJTYPE_SpatialTree = 70,					//!< Spatial search tree
	OBJTYPE_HoughParms = 71,					//!< Hough Transformation Parameters
	OBJTYPE_Shape = 72,							//!< Geometric "Shape" object (Shapefile, Oracle Spatial, ...)
	OBJTYPE_Database_Shape = 73,				//!< Database for Geometric "Shape" object (Shapefile, Oracle Spatial, ...)
	OBJTYPE_ICMProfile = 74,					//!< ICM Profile objects
	OBJTYPE_QuadSearchTree_List = 75,		//!< Search Quad Tree - "list" structure
	OBJTYPE_MosaicLayout = 76,					//!< Mosaic Layout 
	OBJTYPE_Container = 77,						//!< Container object, cannot be copied, moved, etc...
	OBJTYPE_RasterTheme = 78,					//!< Raster Theme object
	OBJTYPE_QuadSearchTree_Set = 79,			//!< Search Quad Tree - "set" structure
	OBJTYPE_Stat_DbField = 80,					//!< Statistics for database field
	OBJTYPE_RemoteTileset = 81,				//!< Remote Tileset (not a real RVC object)
	OBJTYPE_RemoteKML = 82,						//!< Remote KML (not a real RVC object)
	OBJTYPE_WMS = 83,								//!< WMS (not a real RVC object)
	OBJTYPE_ArcIMS = 84,							//!< ArcIMS (not a real RVC object)
	OBJTYPE_Remote = 85,							//!< Generic Remote URL (not a real RVC object)
	OBJTYPE_TerrainTileset = 86,				//!< Terrain Tileset (not a real RVC object)
	OBJTYPE_LAST									//!< Make sure rvcsys / rvcfile.c gets updated if this is changed
	};

//! Increment operator for RVC::OBJTYPE.
inline RVC::OBJTYPE& operator++ (RVC::OBJTYPE& lhs) \
	{ lhs = static_cast<RVC::OBJTYPE>(static_cast<int>(lhs) + 1); return (lhs); }

}	//! End of RVC namespace
	
#endif		//!< INC_RVC_OBJTYPE_H
