/******************************************************************************
 *
 * \file sml/context.h
 * \brief SML CONTEXT
 *
 * \if NODOC
 * $Log: context.h_v $
 * Revision 1.18  2011/10/12 14:06:01  mju
 * Merge contextp hdr into here and remove unrelated hdrs.
 *
 * Revision 1.17  2011/10/12 12:45:53  mju
 * Remove redundant hdrs.
 *
 * Revision 1.16  2008/11/03 16:59:49  scowan
 * Moved getParent from sml.
 *
 * Revision 1.15  2008/11/03 16:50:36  mju
 * Move sml_AppInit here.
 *
 * Revision 1.14  2008/10/30 17:55:01  scowan
 * Moved exported fucntion to here.
 *
 * Revision 1.13  2008/10/23 20:37:58  scowan
 * Moved function prototypes from sml include file.
 *
 * Revision 1.12  2008/10/16 17:14:52  scowan
 * Moved most context creation code to context specific include file.
 *
 * Revision 1.9  2008/09/29 13:57:29  mju
 * Use mistatuscontext in get/setContext.
 *
 * Revision 1.8  2008/09/09 12:59:53  mju
 * Exclude internal version of callFunction from doxygen.
 *
 * Revision 1.3  2007/05/15 22:19:12  dwilliss
 * keep track of product code
 *
 * Revision 1.2  2007/05/08 15:45:34  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/07 15:43:38  dwilliss
 * Initial revision
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_CONTEXT_H
#define INC_SML_CONTEXT_H

#ifndef	INC_SML_SYMBOL_H
	#include <sml/symbol.h>
#endif

#ifndef	INC_SML_ARG_H
	#include <sml/arg.h>
#endif

#ifndef	INC_SML_CONTEXTTYPE_H
	#include <sml/contexttype.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_MICALLBACKLIST_H
	#include <mi32/micallbacklist.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_MTIMER_H
	#include <mi32/mtimer.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

#ifndef INC_MI32_MCB_H
	#include <mi32/mcb.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_SMLFUNCS_H
	#include <mi32/smlfuncs.h>		// Not needed for this hdr, but lots of stuff needs function declarations for proper import/export using DLL
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

#include <stdio.h>


#ifdef WIN32
#pragma warning (disable: 4018) /* '!=' : signed/unsigned mismatch */
#pragma warning (disable: 4251) //! struct needs dll-interface blah blah blah
#endif


#define NSTACK	2048
#define NFRAME	512


//=====================================================================================================================
// Forward Declarations

#ifndef GENERATING_DOXYGEN_OUTPUT
class CONSOLE;
class FILEPATH;
class MISTRING;
class MISTATUSCONTEXT;
class XMLDOC;
class XMLNODE;
class FILE_TEXT;
struct DISPLAYINFO;

#ifdef X_NATIVE
// Need to create a prototype for a method that takes
// a CWnd even though may not be compiling WIN32_NATIVE.
class CWnd;
struct _GUID;
#endif

struct ITypeInfo;
struct IDispatch;

namespace MGD {
	class CONTEXT;
	}

namespace RVC {
	class FILEHANDLE;
	class DESCRIPTOR;
	class DATABASEHANDLE;
	class OBJITEM;
	class CREDENTIALS;
	}

namespace SML {
	union ARG;
	struct CALLBACKSTRUCT;
	class FIELDDATA;
	class SYMBOL;
	class STRING;
	class DEVPRIVATE;
	class GEOSTRUCT;
	class CONTEXT;
	class RASTEROBJECT;
	class VECTOROBJECT;
	class CADOBJECT;
	class TINOBJECT;
	class SHAPEOBJECT;
	class ELEMDATA;
	class ENUMLIST;
	class SOURCEINFO;
	class PARSEPOS;
	class FLAG;
	class CLASS;
	class FUNC;
	class OBJECT;
	class ONEXITDATA;
	class MACHINEINSTRUCTION;
	class SMLDEBUG;
	class PARSER;
	class MEMPOOL;
	class SEGMENT;
	class PREPROCESSORSTACK;
	}

#endif	// GENERATING_DOXYGEN_OUTPUT

//=====================================================================================================================

#ifndef GENERATING_DOXYGEN_OUTPUT
int qqlex(void*, SML::PARSER*);
void qqerrorext(SML::CONTEXT*, int, const char* const*, const short*, int);
void qqerror(void*, const char*);
#endif

namespace SML {
typedef int (* FUNCINSTALLER)(CONTEXT*);
typedef PTRINT MACHINEINST;


//=====================================================================================================================
#ifndef GENERATING_DOXYGEN_OUTPUT
// Defined here because CONTEXT has private member which is an array of them.
class FRAME {
	public:
		SYMBOL		*symp;		//!<  symbol table entry of function/procedure
		MACHINEINST	*retpc;		//!<  Where to resume after return
		MACHINEINST	*retcs;		//!<  Return code segment
		ARG			*argn;		//!<  n-th argument on stack
		SYMBOL		*localvars; //!<  locaal variables
		int			nargs;		//!<  number of arguments
	};
#endif // GENERATING_DOXYGEN_OUTPUT

//=====================================================================================================================
#ifndef GENERATING_DOXYGEN_OUTPUT
/* The "DBTABLEINFO" returns one of these too, but ignores the "FieldNum" */
class FIELDDATA {
	public:
		INT32 IsTableData;		// always 0
		ELEMDATA *ElemData;
		int TableNum;
		int FieldNum;				//!<  Field number of this field
	};
#endif // GENERATING_DOXYGEN_OUTPUT

//=====================================================================================================================
//!	This class provides a way to attach private data to an SML::CONTEXT on a per-context basis.
//! Any data attached in this way is automatically deleted when the context is destroied.
//! To do so, first create a class derived from SML::DEVPRIVATE.  Then write a function like
//! the following.
//!
//! @code
//! static MYPRIVDATA* GetPrivData (
//!		SML::CONTEXT* context
//!		) {
//!		static int PrivateDataIndex = -1;
//!		MYPRIVDATA* pdp;
//!
//!		if (PrivateDataIndex < 0) {
//!		   PrivateDataIndex = context->PrivDataIndexAlloc();
//!		   if (PrivateDataIndex < 0) return (NULL);
//!		   }
//!		pdp = static_cast<MYPRIVDATA*>(context->GetPrivData(PrivateDataIndex));
//!		if (pdp == NULL) { 			// First time for this context
//!		   pdp = new MYPRIVDATA(context);
//!		   context->SetPrivData(PrivateDataIndex, pdp);
//!		   }
//!		return (pdp);
//!		}
//! @endcode
//!
//!
class DEVPRIVATE {
	public:
		virtual ~DEVPRIVATE() {
			}
	protected:
		//! Constructor (protected).
		//! The constructor is protected so that you can't create an instance of an SML::DEVPRIVATE, only derived classes.
		//! The derived class should make the constructor public.
		DEVPRIVATE() {
			}
	};

//=====================================================================================================================
#ifndef GENERATING_DOXYGEN_OUTPUT
//	This class is used internally by SML to keep track of things like
//
// @code
//		thing.color.red = 0
//	@endcode
//
//	If thing.color just put a copy of the color on the stack, then setting
// the red componant of that temporary copy of the color wouldn't do much.
// The class stack allows SML to keep track of where the temp color came
// from, so that if "thing" allows "color" to be set, the above example
// would modify the temporary copy of color, then set thing.color.
//
class CLASSSTACK {
	public:
		void *parent;
		void *ptr;
		CLASSMEMBER *member;
		bool bIsBound;
	};
#endif

//=====================================================================================================================

/* Flags for SML_InstallStaticEnum().  It gets passed an array of SMLFLAGs and we have
*	to tell it what if anything to free when the context is destroied
*/
#define SMLFLAG_FreeName			0x0001
#define SMLFLAG_FreeDesc			0x0002
#define SMLFLAG_FreeFlagArray		0x0004

//! Flags for GetDestObjectSymbol()
#define DESTOBJ_OutputCantBeInput		0x0001
#define DESTOBJ_DeleteExisting			0x0002
#define DESTOBJ_CreateEmpty				0x0004
#define DESTOBJ_DestIsFirstParm			0x0008
#define DESTOBJ_FunctionReturnsObject	0x0010


#define SMLRUN_StayResident 0x00000001	//!< Flag for SMLCONTEXT::Run().
#define SMLRUN_BreakAtLine0 0x00000002	//!< Flag for SMLCONTEXT::Run().


//=====================================================================================================================
//! Main class used by SML functions
class RVCSYSLIBCLASSEXPORT CONTEXT {
public:

	// Note, the constructor is private so that code outside the DLL
	// doesn't have to know how big an SMLCONTEXT is
	// Destructor is private so that you have to call Destroy() to destroy
	// the context.  Actual destruction may be delayed by pending callbacks.

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Internal use only
	//-------- Static public methods ---------------
#if defined(RVCSYSDLL)
	//! Function to allocate an SMLCONTEXT for an open database.
	//! The constructor is private so that code outside the DLL doesn't
	//! have to know how big to allocate the thing.
	//! This is actually a very low-level function and shouldn't be used
	//! directly.  Use DBQuery_CreateContextFromDBHandle() instead.

	static ERRVALUE Create (
		RVC::DATABASEHANDLE* dbhandle,		//!< Handle to open database
		CONTEXT*& context,					//!< SMLCONTEXT returned
		bool ReopenHandle = true
		);
#endif

	//! Function to allocate an SMLCONTEXT for a geospatial object.
	//! The constructor is private so that code outside the DLL doesn't
	//! directly.
	//! This is actually a very low-level function and shouldn't be used
	//! directly.  Use SML_CreateObjectContext() instead.
	//! Note: the credentials are only used for opening the initial object 
	//! 	pointed to by ObjItem
	static ERRVALUE Create (
		CONTEXT*& context,		//!< SMLCONTEXT returned
		const RVC::OBJITEM& ObjItem,		//!< Main object to open
		const RVC::CREDENTIALS* credentials,	//!< Credentials to use when opening ObjItem
		UINT32 flags = 0
		);

#if defined(RVCSYSDLL)
	//! Function to allocate an SMLCONTEXT for a geospatial object.
	//! The constructor is private so that code outside the DLL doesn't
	//! directly.
	//! This is actually a very low-level function and shouldn't be used
	//! directly.  Use SML_CreateObjectContext() instead.
	//! If the fid parameter is 0, the context is not associated with any
	//! object and useful as just a plain SML context
	DEPRECATED static ERRVALUE Create (
		CONTEXT*& context,		//!< SMLCONTEXT returned
		const RVC::OBJITEM& objitem,		//!< Handle to open RVC file
		UINT32 flags = 0
		);
#endif

	static ERRVALUE Create (
		CONTEXT*& context,		//!< SMLCONTEXT returned
		UINT32 flags = 0
		);

	#endif // GENERATING_DOXYGEN_OUTPUT

	//! Setup an alias for a class or function
	//! Existing scripts will be able to refer to the class or function using
	//! OldName, but will get a parser warning that the name has changed.
	//! Note: both names must be string literals or static.  The context
	//! only keeps the pointers
	void AddNameAlias (
		const char* NewName,
		const char* OldName
		);

	//! COM-like AddRef method
	//! An SMLCONTEXT is not a COM object, but uses AddRef/Release symantics.
	void AddRef (
		);

	//! COM-like AddRef method
	//! An SMLCONTEXT is not a COM object, but uses AddRef/Release symantics.
	//! Calling Release() may actually destroy the SML context if the caller
	//! was holding the last reference
	void Release (
		);

	//! Return the string for the n'th operator
	static char *GetOperator (
		int num
		);

	//! Return the help code for the n'th operator
	static char *GetOperatorHelp (
		int num
		);

	//! Return the description for the n'th operator
	static char *GetOperatorDescription (
		int num
		);

	//! Return the list of operators (for the Insert Operator dialog)
	static char **GetOperatorList (
		) ;

	//! convet an SML::STYPE into a string for display purposes.
	//! Note, these are not i18n-able on purpose.  SML keywords
	//! cannot be translated
	static char *STYPEtoString (
		STYPE stype
		);

	//! Get the STYPEFlags for a given STYPE.
	//!	This gets used in 2 places.
	//!	When checking parameters, it's used to see if a symbol is an
	//!		STYPE_ObjSym.  If it is but not the kind we want, we cast it.
	//! It's also used after a function call to see if the parameter
	//!		was an ObjSym and if it was a temp object, free it.
	//!	Not currently used for anything else, so the contents are not
	//!	that important except for the ones that have ObjSym set.
	static int STYPEFlags (
		STYPE stype
		);

	//! Convert STYPE_Xxxx to the token value as assigned by YACC
	static int STYPEToTokenVal (
		STYPE stype
		);

	//! Convert a token value as assigned by YACC to an STYPE_Xxxx
	static STYPE TokenValToSTYPE (
		int tval
		);

	//! Register a list of function installers to be installed for any context,
	//! This may be called on program startup.  
	static void RegisterFunctionList (
		const FUNCINSTALLER* FuncList
		);

	//! Register a list of function installers to be installed for any context,
	//! This may be called on program startup.  
	static void RegisterMinimalFunctionList (
		const FUNCINSTALLER* FuncList
		);

//--------- Get methods ------------

	//! Get the AbortOnError flag.
	//! If set (which it is by default, then any built-in function that gets an
	//! error and calls SetErrorCode() will cause the script to abort and popup
	//! an error dialog.  If cleared, the error will be noted but the script will
	//! go on and the script writer can check GetLastError() to see if there was
	//! an error.
	bool GetAbortOnError (
		) const;


	//! Get an argument from the argument list
	//! Can return an error if the argument is invalid
	//! in this case, the SML ErrorCode has already been set
	//! and you just have to return.
	//! Note, the args/numargs parameters are in the opposite
	//! order of the old function.  I thought that was a stupid order.
	int GetArg (
		ARG* args,
		int numargs,
		int num,
		OBJECT** pObject,
		bool UndefinedOK = false
		);

	//! Get the array index base (normally 1, but can be overridden)
	int GetArrayIndexBase (
		) const;

	//! Return the time in seconds between each cancel check
	double GetCancelCheckInterval (
		) const;

	//! Return whether or not string comparisons should be case sensitive
	bool GetCaseSensitiveStrings (
		) const;

	//! Get the size of the bytecode in memory
	int GetCodeSize (
		) const;

  //! Get the current Code Segment (CS) (const)
	const MACHINEINST* GetCS (
		) const;

  //! Get the current Code Segment (CS) (non-const)
	MACHINEINST* GetCSNonConst (
		) const;

	//! Returns the console handle
	CONSOLE *GetConsoleInterface (
		) const;

	//! Return the SML::FUNC* of the current function.
	//! This is only valid while running, not parsing.
	const SML::FUNC* GetCurrentFunction (
		) const;

	//! Return the name of the current function.
	//! This is only valid while running, not parsing.
	//! This should return a (const MIUNICODE *), but not all the
	//! places that need it know about const yet.
	MIUNICODE* GetCurrentFunctionName (
		) const;

	//! Return symbol for return value of a function returning an object
	//!
	//!		This is a magic function.  Given the arguments, it returns a symbol
	//!		pointing to the destination symbol.  If it has to, it makes a new
	//!		temporary symbol and returns that.  However, it first looks at the
	//!		next instruction in the code.  If it's an opOBJASGN, it returns
	//!		the next symbol on the stack (args[-1]), which will be the variable
	//!		that opOBJASGN will assign into.  It then adjusts the PC to skip the
	//!		assignment instruction and adjusts the frame pointer to make the
	//!		dest variable pop off with the rest of the function parameters when
	//!		the funtion returns.
	//!
	//! Allowable flags:
	//!		\li DESTOBJ_OutputCantBeInput
	//!				If set, looks to see if any input parameter is the same as
	//!				the destination and gives an error if it is.  This prevents
	//!				stupid things like trying to merge a vector with itself.
	//!
	//!		\li DESTOBJ_DeleteExisting
	//!				If symbol points to something, delete it. (just the object,
	//!				not the file) Must keep the same name/description, which are
	//!				in the object info.
	//!
	//!		\li DESTOBJ_CreateEmpty
	//!				Same as DESTOBJ_DeleteExisting, but then creates an empty
	//!				object with the same name, etc.
	//!
	//!		\li DESTOBJ_DestIsFirstParm
	//!				The destination is the first parameter to the function and
	//!				the SML::PARM for it has the PARM_ByRef flag.
	//!
	//!		\li DESTOBJ_FunctionReturnsObject
	//!				This is the old way of writing object functions in SML.
	//!				the return value is actually arg[-1].  All functions of
	//!				this type are deprecated in favor of DestIsFirstParm.
	//!
	//! \deprecated - Do not write any more functions that return vector or cad
	//!   objects.
	SYMBOL* GetDestObjectSymbol (
		int numargs,					//!< number of arguments passed to SML func
		ARG* args,					//!< SML function's argument list
		UINT8* argtype,				//!< SML functions's argument types
		STYPE OutSTYPE,			//!< STYPE of the object to be returned
		UINT32 flags					//!< Flags (see above)
		);

	//! Returns the display info set by previous call to SetDispInfo.
	//! SML doesn't use this information except to pass it through
	//! to the script is _context.DisplayInfo
	const DISPLAYINFO* GetDispInfo (
		) const;

	//! Returns the line of the last error
	int GetErrorLine (
		) const;

  	//! Returns true if we want scripts to exit on errors.
	//! We let the script change this and a
	//! few SML functions return error codes rather than
	//! causing the script to exit.
	int GetExitOnError (
		) const;

	//! Return the SYMBOL of a database field
	SYMBOL* GetFieldSymbol (
		int table, 		//!< Table number
		int field		//!< Field number
		);

	//! Return the SYMBOL of a database field
	//!
	//! This version only used by the parser.
	SYMBOL* GetFieldSymbol (
		SYMBOL* t,
		SYMBOL* f
		);

   //! Returns the current raster being processed by a "foreach" or NULL if none
	SYMBOL* GetForEachRastSym (
		) const;

	//! Get the name of a function group (don't free this)
	//! Maybe this should return const? Should it look up the
	//! name in messages.txt for us or should the caller?
	//! (Currently we expect the caller to do that)
	char* GetFuncGroupName (
		int num
		) const;

	//! Returns function types and names for the "Insert Function" dialogs
	int GetFunctionTypeList (
		char ***namep,
		int **typep
		) const;

	//!		Returns the last error code, message and position
	//!		where the error occured.
	//!
	//!		Calling this function also clears the error condition
	//!		If error occured in an include file, pos returned will be -1
	//!		to tell us not to try to move the cursor (not in the text widget)
	int GetLastError (
		MISTRING& msg,
		INT32 *pos = 0
		);

	//!		Returns the last error code, and (optionally) position
	//!		where the error occured.
	//!
	//!		Calling this function also clears the error condition by default.
	//!		You can pass false for bClearErrorState to prevent that.
	//!		If error occured in an include file, pos returned will be -1
	//!		to tell us not to try to move the cursor (not in the text widget)
	ERRVALUE GetLastError (
		INT32 *pos  = 0,
		bool bClearErrorState = true
		);
	
#ifdef RVCSYSDLL
	ERRVALUE GetLastError (
		INT32 *pos 
		);
#endif

	//! Retrieves the last STRING to be popped from the stack
	//!
	//! @note The string returned can be modified (I don't know why you would),
	//! 		but don't free it!
	STRING* GetLastPoppedString (
		) const;

	//! Returns the RVC::OBJITEM of the main object this query is for.
	//! Returns 1 if there is a main object, 0 if not
	int GetMainObject (
		RVC::OBJITEM& ObjItem
		) const;

	//! Will return true if the parser needs to emit opLINE codes for the
	//! debugger.  (Used to interleave source and assembly)
	int GetNeedLineNumbersInCode (
		) const;

	//! Returns the current number of breakpoints set in the debugger
	int GetNumBreakPoints (
		) const;

 	//! Get the number of function groups
	int GetNumFuncClasses (
		) const;

	//! For internal use only (should be private, but used by
	//! non-member funcs in qqobject.c)
	OBJECT* GetObjectList (
		) const;

	//! Get specification of an SML bytecode opcode.
	//! Bad name.  Should have been GetOpcodeData()
	const MACHINEINSTRUCTION* GetOpCodeData (
		int opcode
		) const;

  	//! Get the parent widget to use for popping up dialogs and such.
	//! We ran into a problem with WIN32_NATIVE code.  Since MDLGPARENT
	//! is just a typedef that normally resolves to Widget, the function
	//! signature said it returned a Widget.  Then if you compile for
	//! WIN32_NATIVE, the prototype says it should be a CWnd and the
	//! signatures don't match, giving an unresolved external.  To
	//! get around this, I added the (void*) version below and modified
	//! all code to use that instead.  Need to keep the other one around
	//! though for old code.
	MDLGPARENT GetParentWidget (
		) const;

	//! Get Parent Widget
	//! @see SMLCONTEXT::GetParentWidget()
	void* GetParentWidgetVoid (
		) const;

	int GetParserWarningLevel() const;

	//! Return the warnings from the most recient call to Parse()
	const MISTRING& GetParserWarnings() const;

	//! Return a list of fields accessed by the most reciently parsed query
	//! which are not indexed.
	//! If the query will be evaluated multiple times, you may decide to
	//! index the fields first, or possibly let the user decide.
	//! If you do end up indexing them, you should reparse the query to get
	//! it to use the index.
	const SIMPLE_ARRAY<FIELDDATA>& GetUnindexedFieldList() const;

	//! Set the level for parser warnings
	//!
	//! 	0 = none,
	//!		1 = Warn about deprecated function/class usage
	//!		2 = Warn about unindexed fields
	//!		3 = Warn about things that would cause errors on "strict" syntax checking
	//!
	void SetParserWarningLevel(int level);

	const MACHINEINST* GetPC (
		) const;

	//! Function used by the debugger only.
	//! If debugger becomes a friend class, this function can be made private
	int GetPCList (
		MACHINEINST ***plist
		);

	ERRVALUE GetPCList (
		SIMPLE_ARRAY<const MACHINEINST*> &plist
		);

   const SOURCEINFO* GetSourceInfo (
		const MACHINEINST* pc
		) const;

   const PARSEPOS* GetSourcePos (
		const MACHINEINST* pc
		) const;

	bool GetSourceLine (
		const MACHINEINST* pc,
		MISTRING& mistr
		) const;

	SYMBOL* GetPrimaryKeySymbol (
		SYMBOL* fk_sym
		);

 	//! Return the primary table for a segment.
	//! (assuming a pinmap query or computed field)
	int GetPrimaryTable (
		int segment
		) const;

	//! Get the XMLDOC that this script was parsed from, if any.
	//! Note: the document is owned by the context.  The caller may modify
	//! 	it, but should not free it.
	XMLDOC* GetXMLDoc (
		) const;


	//! Append a Unicode string to the print buffer
	int AppendPrintBuffer (
		const MIUNICODE*
		);

	//! Append an ISO-2022 string to the print buffer
	int AppendPrintBuffer (
		const char*		//! assumed to be ISO-2022
		);

	//! Return the contents of the print buffer in Unicode
	//!
	//! When running in an SML context, printf prints to a console
	//! window and there is no print buffer.  In all other contexts,
	//! printf functions append text to a buffer which the caller
	//! can retrieve.
	//!
	const MISTRING& GetPrintBufferMISTRING (
		) const;

	//! Get private data for this context.  Allocate an index with
	//! PrivDataIndexAlloc()
	DEVPRIVATE* GetPrivData (
		int pindex
		) const;

	//! Used by parser
	int GetProgBase (
		) const;

	//! Used by parser
	int GetProgPtr (
		) const;

	//! this function should be private but...
	//! for now needs to be called
	//! from a C style stub function.
	char *GetRCSID (
		SOURCEINFO *fni
		);

	//! Returns the list of registered classes
	CLASS* GetRegisteredClassList (
		) const;

	//! Returns the root database handle (may be null) which should be used
	//! to allocate a cursor if calling Evaluate with ELEMTYPE_DBCursor
	//! Note: for internal use only
	RVC::DATABASEHANDLE* GetRootElemDBHandle (
		) const;

 	//! return the element number we're evauating the query for.
	//!
	//! @note only for element queries
	int GetRootElemNum (
		) const;

	//! Returns the main ELEMTYPE_Xxxx if this is a query.
	//! (eg, polygon style query)
	int GetRootElemType (
		) const;

	//! Get the filename of the current script.
	const FILEPATH& GetScriptFilePath (
		) const;

	//! Gets the inode of the current script if it came from an RVC file.
	//!
	//! @return an inode or 0 if the script didn't come from an RVC file
	INT32 GetScriptInode (
		) const;

	//! Gets the RVC::OBJITEM of the script.
	const RVC::OBJITEM& GetScriptObjItem (
		) const;

#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Get the database handle for a given segment.
	//! Do NOT close this!
	//! For use by the editor and debugger only (outside RVCSYS)
	RVC::DATABASEHANDLE* GetSegmentDBHandle (
		int segment
		) const;
#endif

	//! Returns SegmentType() & QTYPE_ELEMTYPE_MASK.
	//! If segment -1 is passed (or omitted since -1 is the default) it will
	//! use the "current" segment.  This only works while a script is running
	int GetSegmentElemType (
		int segment = -1
		) const;

	//! Return the XMLNODE this segment was parsed from.
	//! Assumes SetSegmentXMLNode was called to set the segment earlier
	const XMLNODE* GetSegmentXMLNode (
		int segnum
		) const;

	//! Given an XMLNODE, return the segment that was parsed from it.
	//! This is used to identify scripts from an XML \<script\>
	//! tag so that we don't parse the same tag twice.
	//! Returns the segment number or -1 if no segment came from the
	//! given node.  Note that if node is 0, it will return the first
	//! node that did NOT come from an XML node.
	int GetSegmentByXMLNode (
		const XMLNODE* node
		) const;

 	//! Get the current STATUS bits
	UINT32 GetStatus (
		) const;

	//! Get the MISTATUSCONTEXT of the status line of the SML editor.
	//! @note returns NULL if there is no status line or SML editor
	MISTATUSCONTEXT* GetStatusContext (
		) const;

  //! Need to rename this.
  //!
  //! _I_ don't even rememeber what it is (DCW)
	SYMBOL *GetStdSym (
		) const;

	//! Get an SYMBOL by (char*) name.
   //!		Programmer accessable function to get a symbol from the
	//!		symbol table after evaluating the query.  Actually
	//!		returns a pointer to the symbol.
	SYMBOL *GetSymbol (
		const char *name
		) const;

	//! Get an SYMBOL by (MIUNICODE*) name.
   //!		Programmer accessable function to get a symbol from the
	//!		symbol table after evaluating the query.  Actually
	//!		returns a pointer to the symbol.
	SYMBOL *GetSymbol (
		const MIUNICODE *name
		) const;

	//! Returns symbol types and names for the "Insert Symbol" dialogs
	int GetSymbolTypeList (
		char ***namep,
		int **typep
		) const;

 	//! Get the symbol list for a group of functions
	SYMBOL* GetSymbolListFuncs (
		int group = 0
		) const;

	//! Gets the main symbol list
	SYMBOL* GetSymbolListGlobal (
		int level = 0
		) const;

	//! Gets the symbol list at the current scope
	SYMBOL* GetSymbolListLocal (
		) const;

	//! Gets the symbol list for permanent symbols
	SYMBOL* GetSymbolListPerm (
		) const;


	//! Get the list of symbols for a function group
	//!
	//!	This function gives you a list of the functions in a particular
	//!	function group.
	//!
	//! @return the number of strings in the array.
	//!
	//! @param funcgroup is the group number of the function group to get.
	//! You can find the number of function groups by calling
	//! SMLCONTEXT::GetNumFuncClasses().
	//!
	//! @param names Gets filled in with a list of function names
	//! @param bIncludeAliases If true (default), include aliases in the
	//!	list of symbols.
	//!
	//! @see SMLCONTEXT::GetNumFuncClasses()
	//! @see SMLCONTEXT::GetFunctionTypeList()
	//!
	int GetSymbolNameList (
		int funcgroup,
		MISTRINGLIST& names,
		bool bIncludeAliases = true
		) const;

	//! Get the SYMBOL for a database table
	SYMBOL* GetTableSymbol (
		int table		//!< Table number
		);

	//! Get the ProductCode for the script being executed or parsed.
	//! This is used to limit the set of available classes to only those
	//! available in the given product
	MIPRODUCTCODE GetTargetProductCode (
		) const;


//--------- Set methods ------------

	//! Set the AbortOnError flag.
	//! If set (which it is by default, then any built-in function that gets an
	//! error and calls SetErrorCode() will cause the script to abort and popup
	//! an error dialog.  If cleared, the error will be noted but the script will
	//! go on and the script writer can check GetLastError() to see if there was
	//! an error.
	void SetAbortOnError (
		bool value
		);

	//! Set an error code for a function argument.
	//! Parameter numbers start at 0.
	//! If errcode is omitted, EBadFuncParm will be used
	//! If funcname is omitted, the current function name will be used
	//! Returns the error code passed in
	ERRVALUE SetParmError (
		int parmnum,				//!< Parameter, 0-based
		int errcode = EBadFuncParm,
		const MIUNICODE* funcname = 0
		);

	//! Set the time in seconds between each cancel check
	void SetCancelCheckInterval (
		double seconds
		);

 	//! Set the function to check cancel
	void SetCancelFunc (
		int (*func)(CONTEXT *, void *),
		void *data
		);

	//! Set whether or not string comparisons are case sensitive
	//! The default is true.
	//! Note, this is NOT for symbol name comparisons
	void SetCaseSensitiveStrings (
		bool sensitive = true
		);

	#if defined (RVCSYSDLL)
	//! Set the console handle. This is done by the editor
	//! This version can go away after the 30-Dec-06 build
	DEPRECATED void SetConsole (
		void *console
		);
	#endif

	//! Set the console handle. This is done by the editor
	void SetConsole (
		CONSOLE *console
		);

	void SetDebugger (
		SMLDEBUG* debug
		);

	//! Set display info.
	//! This has to be done outside the DLL
	void SetDispInfo (
		const DISPLAYINFO& info
		);

	//! Set an error condition (and maybe abort the script).
	//! By default, this will also cause the script to abort, but the script
	//! author can override that by setting _context.AbortOnError = 0 and
	//! use GetLastError() to check for errors.
	int SetErrorCode (
		int code
		);

	//! Set an error condition (and abort script).
	//! This WILL cause the script to abort
	int SetErrorCodeAndAbort (
		int code
		);

	//! Used for parse errors (qqerrorext) 
	void SetErrorMessageForce (
		MIUNICODE *s
		);

	//! Set the ProductCode for the script being executed or parsed.
	//! This is used to limit the set of available classes to only those
	//! available in the given product
	void SetTargetProductCode (
		MIPRODUCTCODE ProductCode
		);

	//! Called by the editor when debugger is activated/deactivated
	void SetNeedLineNumbersInCode (
		int val
		);

	//! Set the parent widget to use for popping up dialogs and such
	void SetParentWidget (
		MDLGPARENT widget
		);

#if !defined(WIN32_NATIVE) || defined(RVCSYSDLL)
	//! CWnd version of SetParentWidget() for use in WIN32_NATIVE apps
	//! We need to create a prototype for this method so that we can
	//! export it from the DLL. Even though the DLL is never compiled
	//! for WIN32_NATIVE, it's used from WIN32_NATIVE.
	//! Note, however that if we _are_ compilng for WIN32_NATIVE,
	//! The above method would resolve to the same as this one
	//! (since an MDLGPARENT will be a (CWnd*) in that case.
	void SetParentWidget (
		CWnd* widget
		);
#endif

  	//! Turns ownership of the pointer over to the context.  Pointer
	//! will be deleted when the context is destroied.
	void SetPrivData (
		int pindex,
		DEVPRIVATE* pdata
		);

	//! Set the script filename
	//! Use if the script is not in an RVC object
	void SetScriptFilePath (	
		const FILEPATH& filepath
		);

	void SetScriptObjItem (
		const RVC::OBJITEM& objitem
		);

	//! Set segment to be parsed or executed.
	ERRVALUE SetSegment (
		int segnum
		);

	//! Keep track of the XMLNODE this segment was parsed from.
	//! This is used to identify scripts from an XML \<script\>
	//! tag so that we don't parse the same tag twice.
	void SetSegmentXMLNode (
		int segnum,
		const XMLNODE* node
		);

	//! Set one or more status bits
	UINT32 SetStatusBits (
		UINT32 bits
		);

	//! Set the status context.
	void SetStatusContext (
		MISTATUSCONTEXT* pStatusContext
		);

	//! Set the XMLDOC that this script was parsed from, if any.
	//!	Note, the SML context then "owns" the document pointer
	//! 	and will delete it when it's done.
	void SetXMLDoc (
		XMLDOC* doc
		);

	//! Check to see if a class is derived from another class.
	//! (internal use by the parser)
	bool TestClass (
		const char* GivenClassName,
		const CLASS* NeededClass
		) const;

	//! Check to see if a class is derived from another class.
	//! (internal use by the parser)
	bool TestClass (
		const CLASS* GivenClass,
		const CLASS* NeededClass
		) const;

	// ------------ Other methods (Alphabetical) -------------

	//! Add symbol referring to object using 'MIUNICODE*' name.
	ERRVALUE AddObjectSymbol (
		const MIUNICODE *name,
		const RVC::OBJITEM& objitem,
		SYMBOL **symp,
		UINT32 flags = 0,
		const RVC::CREDENTIALS* Credentials = 0
		);

	//! Add symbol referring to object using 'char*' name.
	ERRVALUE AddObjectSymbol (
		const char *name,
		const RVC::OBJITEM& objitem,
		SYMBOL **symp,
		UINT32 flags = 0,
		const RVC::CREDENTIALS* Credentials = 0
		);

	//! Add an OnExit entry to the OnExitList
	void AddOnExit (
		ONEXITDATA* data
		);

	//! Add a symbol to the context (MIUNICODE*) name
	//! @return the symbol added
	SYMBOL *AddSymbol (
		const MIUNICODE *name,
		STYPE stype
		);

	//! Add a symbol to the context (char*) name.
	//! @return the symbol added
	SYMBOL *AddSymbol (
		const char *name,
		STYPE stype
		);

	//! Add a class symbol to the context (MIUNICODE*) name.
	//!
	//! This would let you add, for example, a COLOR symbol for use
	//! in a query.
	//!
	//! @return the symbol added
	SYMBOL *AddClassSymbol (
		const MIUNICODE *name,	//!< Symbol name
		char *classname, 		//!< SML class name (SMLCLASSNAME_...)
		void *ptr 				//!< Pointer to the data the symbol represents
		);

	//! Add a class symbol to the context (char*) name.
	//! See details in the (MIUNICODE*) version of this method
	//! @return the symbol added
	SYMBOL *AddClassSymbol (
		const char *name,		//!< Symbol name
		char *classname, 		//!< SML class name (SMLCLASSNAME_...)
		void *ptr 				//!< Pointer to the data the symbol represents
		);

	//! Add a breakpoint at the current offset.
	//! (offset = pc - cs)
	int BreakPointAdd (
		int offset
		);

	//! Remove a breakpoint from the current offset.
	//! (offset = pc - cs)
	int BreakPointRemove (
		int offset
		);

 	//! Toggle a breakpoint at the current offset.
	//! (offset = pc - cs)
	int BreakPointToggle (
		int offset
		);

	//! Add a callback.
	//!
	//! For a list of possible callback reasons
	//! @see SML::CALLBACKSTRUCT
	int CallbackAdd (
		McbCallbackFunc func,
		void *cbdata,
		UINT32 reason
		);

	//! Add a callback to the Global callback list.
	//!
	//! For a list of possible callback reasons
	//! @see SML::CALLBACKSTRUCT
	static int CallbackAddGlobal (
		McbCallbackFunc func,
		void *cbdata,
		UINT32 reason
		);

	//! Remove a callback.
	//!
	//! For a list of possible callback reasons
	//! @see SML::CALLBACKSTRUCT
	int CallbackRemove (
		McbCallbackFunc func,
		void *cbdata,
		UINT32 reason
		);

	//! Remove a callback from the Global callback list.
	//!
	//! For a list of possible callback reasons
	//! @see SML::CALLBACKSTRUCT
	static int CallbackRemoveGlobal (
		McbCallbackFunc func,
		void *cbdata,
		UINT32 reason
		);

	//! Call the callback list.
	//!
	//! For a list of possible callback reasons
	//! @see SML::CALLBACKSTRUCT
	int CallCallbacks (
		CALLBACKSTRUCT *cbs
		);

	//! Call the destructors symbols pointing to class data
	//!
	//! This calls SYMBOL::DisposeOFContents() for all
	//! STYPE_ClassVar symbols
	void CallDestructors (
		);

#if defined(RVCSYSDLL) && !defined(GENERATING_DOXYGEN_OUTPUT)
	//! Call a function pointed to by a given symbol.
	//!
	//! This is used internally by SML implementation only.
	//! \deprecated.  Call the version that returns ERRVALUE
	DEPRECATED void CallFunction (
		SYMBOL* func,			//!< SML::SYMBOL of function to call
		ARG* args,				//!< List of arguments to pass
		int numargs,			//!< Number of arguments in "args"
		UINT8* tlist,			//!< SML::STYPEs of the args
		ARG* ret,				//!< Return value
		UINT32 flags			//!< 1 means opCALL, 0 means callback
		);
#endif

	//! Call a function pointed to by a given symbol.
	//!
	//! This is used internally to call both user-defined
	//! and built-in SML functions.  One possible use for this function
	//! would be to see if a query has some user-defined function in it
	//! and if so, call it once before drawing.
	//! For example:
	//! @code
	//!    UINT8 type = SML::STYPE_Class;
	//!    SML::ARG arg, ret;
	//!    SML::SYMBOL* func;
	//!    arg.ptr = layer;				// Layer about to be drawn
	//!
	//!    func = context->GetSymbol("OnDrawBegin");
	//!    if (func) context->CallFunction(func, args, 1, types, &ret);
	//!    // Do the drawing here
	//!    func = context->GetSymbol("OnDrawEnd");
	//!    if (func) context->CallFunction(func, args, 1, types, &ret);
	//! @endcode
	ERRVALUE CallFunction (
		SYMBOL* func,			//!< SML::SYMBOL of function to call
		ARG* args,				//!< List of arguments to pass
		int numargs,			//!< Number of arguments in "args"
		UINT8* tlist,			//!< SML::STYPEs of the args
		ARG* ret					//!< Return value
		);

	//! Calls the named function.
	//!
	//! Looks up a built-in or user-defined function by name and calls
	//! it with no parameters.  Also sets the current element like
	//! SML::CONTEXT::Evaluate().
	//!
	//! There's a more flexible version of this function which takes
	//! a list of parameters to be passed to the function.
	//!
	//! @note This only works for procedures (no return value) or
	//! functions that return numeric values.  It call any other type
	//! of function, you must use the other version of CallFunction().
	//!
	//! @return The value returned by the function or 0 if it was a
	//! 		procedure.
	double CallFunction (
		const char *name,
		ELEMTYPE elemtype,
		INT32 element = 0
		);


	//! Clears the buffer used by printf functions
	//!
	//! When running in an SML context, printf prints to a console
	//! window and there is no print buffer.  In all other contexts,
	//! printf functions append text to a buffer which the caller
	//! can retrieve.
	void ClearPrintBuffer (
		);

 	//! Clear one or more status bits
	UINT32 ClearStatusBits (
		UINT32 bits
		);

	//! Closes all open objects.  This is done automatically when
	//! the script exits.
	void CloseObjects (
		);

	//! Create an instance of a class pointer
	//! You should use the SMLCLASSNAME_Xxx macros for the class name.  This
	//! makes them easier to search for and prevents mistakes due to typos
	//! (or at least lets the compiler catch them).
	//! Any SML function or class member that returns a copy of something that
	//! it expects SML to delete should use this method to allocate the thing
	//! being returned, unless you know for a fact that SML expects it to be
	//! allocated with new/delete.  That way, as the older classes get converted
	//! over to be allocated with new/delete, you won't have to change anything.
	void* ConstructClassInstance (
		const char* ClassName
		);

	//! Returns true if the current function is a class method.
	//! This is for use in built-in functions.  If it returns
	//! true, then args[-1] is a pointer to "this".  This is useful
	//! when you have an existing function that takes a pointer to a
	//! class as its first member and want to make it do double duty
	//! as a class method.
	//! For example, the SML function ViewRedraw() takes a view (args[0]
	//! and a flag (args[1]).  As a class method, it would take the same
	//! parameters but view would be args[-1] and the flags args[0])
	//! Note that this is only valid for functions which match the prototype
	//! 	whatever FuncName(int numargs, SML::ARG* args, UINT8* type, SML::CONTEXT* context)
	bool CurrentFunctionIsClassMethod (
		) const;

	//! Destroy the context.  Will free it as well, so don't access
	//! the context after calling this function.  If we were using
	//! constructor/desctructors for this, this would be the destructor.
	//! However, If any widget destroy callbacks have been registered,
	//! the actual context will stick around until the last of them
	//! has been called.
	//! \deprecated Use Release() instead
	DEPRECATED int Destroy (
		);

	//! Evaluate a query for a given element of a given type
	//! Might be better to call the other version of this method
	//! which does not take an elemtype.
	int Evaluate (
		int segnum,
		INT32 element,
		int ElemType,
		void *valp
		);

	//! Evaluate a query for a given element
	//! Uses the element type given when the segment was created.
	int Evaluate (
		int segnum,
		INT32 element,
		void *valp
		);


	//! Allocate an SML::GEOSTRUCT
	//!
	//! Code outside rvcsys.dll cannot construct SML::GEOSTRUCTs
	//! since the size of the struct is private
	GEOSTRUCT* GeorefAlloc (
		);

	//! Copy an SML::GEOSTRUCT
	GEOSTRUCT* GeorefCopy (
		const GEOSTRUCT* src
		);

	//! Free an SML::GEOSTRUCT
	void GeorefFree (
		GEOSTRUCT* georef
		);

	//! Set the usage for this script.
	//! Allows parser to warn or give an error if user tries to use a script
	//! for the wrong thing
	USAGE GetUsage (
		int Segment
		) const;

 	//! Returns true if there is a breakpoint at a particular offset
	bool HasBreakPoint (
		int offset
		) const;

	//! Returns true if we should call TestCancleOnce() occasionally
	//! in grunt loops (to keep the server happy)
	bool HasCancelFunc (
		) const;

	//! Install a class definition.
	int InstallClass (
		CLASS* staticClass
		);

	//! Installs a list of enum values which can be used in SML::CLASSMEMBERs.
	//! To define a member which is an enum, you make the member's type as
	//! if it were a numeric member, but give it STYPE_Enum and set the
	//! ResourceName to the name the enum was registered as
	//!
	//! Possible flags:
	//!    SMLFLAG_FreeFlagArray  - Free the smlflags when the context is destroyed
	//!    SMLFLAG_FreeName		  - Free the name parameter of all the flags (not the name parameter, that's const and it makes a copy)
	//!    SMLFLAG_FreeDesc		  - Free the desc parameter of all the flagss
	int InstallEnum (
		const char* name,
		FLAG* smlflags,
		UINT32 flags
		);

	//! Installs a list of enum values which must be generated at runtime.
	//! This is useful for installing things like distance units which
	//! must be read from units.txt
	//!
	//! Possible flags:
	//!    SMLFLAG_FreeFlagArray  - Free the smlflags when the context is destroyed
	//!    SMLFLAG_FreeName		  - Free the name parameter of all the flags (not the name parameter, that's const and it makes a copy)
	//!    SMLFLAG_FreeDesc		  - Free the desc parameter of all the flags
	int InstallEnum (
		const char* name,
		int (*func)(CONTEXT* , void*, FLAG**),
		void* funcdata,
		UINT32 flags
		);

	//! Install a list of functions
	//! (For a new method of installing functions which has not been
	//! completely implemented yet.  Use InstallFunctions() for now)
	int InstallFunctionList (
		const FUNCINSTALLER *funclist
		);

	//! Install a list of functions.  The vfuncs parameter is an array
	//! of SML::FUNCs with the last member of the array having NULL for
	//! the function name
	int InstallFunctions (
		const char* symclass,
		const FUNC* vfuncs
		);

	//! Install functions which should only be available in SML (not query).
	//! This just calls all MsmlCR_InstallFuncsSML callbacks.
	//! It is done for you by the various
	void InstallSMLOnlyFunctions (
		);

	//! Returns true if the contest is being destroied
	//! or has been destoied and is waiting for the last widget
	//! destroy callback to actually free itself.
	bool IsBeingDestroied (
		) const;

	//! Dump a disassembled listing of the bytecode to a file
	void List (
		FILE *file
		);

	//! Dump a disassembled listing of the bytecode to an MISTRING
	void List (
		MISTRING&
		);

	//! Function used by List() and by the debugger.
	//! (If debugger becomes a friend class, this function can become private)
	int ListLine (
		const MACHINEINST* c,
		bool bIgnoreBad,
		bool bMarkIt,
		MISTRING& buf
		);

	//! Function used by List() and by the debugger.
	//! (If debugger becomes a friend class, this function can become private)
	int ListLine (
		const MACHINEINST* c,
		bool bIgnoreBad,
		bool bMarkIt,
		MIUNICODE* buf,
		int len
		);

	//! Lookup the array of SML::FLAGs given the name of an enum list
	//! Mostly internal.  May become private
	FLAG* LookupEnumList (
		const char* name
		);

#ifdef RVCSYSDLL
	//! non-const version of above needed for historical purposes
	FLAG* LookupEnumList (
		char* name
		);
#endif

	//! Allocate any kind of SML::OBJECT
	OBJECT* ObjectAlloc (
		STYPE stype
		);

	//! Allocate an SML::RASTEROBJECT
	int ObjectAlloc (
		RASTEROBJECT**
		);

	//! Allocate an SML::VECTOROBJECT
	int ObjectAlloc (
		VECTOROBJECT**
		);

	//! Allocate an SML::CADOBJECT
	int ObjectAlloc (
		CADOBJECT**
		);

	//! Allocate an SML::TINOBJECT
	int ObjectAlloc (
		TINOBJECT**
		);

	//! byte-code optimizer.
	//! This is automatically called by Parse().
	//! However, if you cal GetUnindexedFieldList() after Parse() and decide
	//! to index something, you should call Optimize() again to get the query
	//! to actually use the index.
	int Optimize (
		);

	//! Decrement the number of outstanding XtDestroy callbacks
	//! which refer to this context
	void OutstandingDestroysDec (
		);

	//! Return the number of outstanding XtDestroy callbacks
	//! which refer to this context
	int OutstandingDestroysGet (
		) const;

	//! Increment the number of outstanding XtDestroy callbacks
	//! which refer to this context
	void OutstandingDestroysInc (
		);

	//! Parse a query
	int Parse (
		int segnum,
		const MIUNICODE *str,
		UINT32 flags = 0
		);

 	//! Allocate a private data index.  This should only be called once
	//! per module and kept in a static variable.  You use when you call
	//! GetPrivData() and SetPrivData()
	int PrivDataIndexAlloc (
		);

	//! Do garbage collection on string pool
	//!
	//! Runs through the pool of strings and frees any which are no
	//! longer referenced by any symbols or code.  Since strings are
	//! refcounted, this \e should be unnecessary, but since the refcount
	//! isn't automatic, its error prone.  This prevents leaks.
	void RecycleStrings (
		bool FreeStringsInCodeToo = false
		);

	//! Registers the function to call to close an object type
	//! Would be done through virtual destructors on SML::OBJECT derived
	//! classes if we could do that.
	void RegisterObjectCloseFunc (
		STYPE stype,
		int (*func)(CONTEXT *, OBJECT *)
		);

#ifndef GENERATING_DOXYGEN_OUTPUT
#ifdef RVCSYSDLL
	//! Scans all functions and classes to make sure the classes their
	//! members and parameters refer to are known.  If the MissingClassFile
	//! paremeter is given, any errors will be written out to that file.
	int ResolveClassMembers (
		);
#endif
#endif

	//! Scans all functions and classes to make sure the classes their
	//! members and parameters refer to are known.  Any errors are written out
	//! to the MissingClassFile 
	int CheckForClassMemberErrors (
		FILE_TEXT& MissingClassFile
		);

	//!	Run is used for normal SML script.  Not currently SML Layers in display
	//!	Never for any kind of query or geoformula
	//!
	//!	Possible flags:
	//!		SMLRUN_StayResident	-- Don't dispose of variables allocated
	//!				within the script when the script exits.  They will be
	//!				disposed of when the context is destroied.
	int Run (
		int segnum,
		UINT32 flags = 0
		);

	//! Create a query segment.  A context can have multiple segments,
	//! each with its own code.  Keep in mind that the symbol table
	//! is shared between all segments in a context.
	//!
	//! @return a segment number or error code < 0
	//! @note 0 is a valid segment number.
	int SegmentCreate (
		USAGE usage,		//!< Usage
		ELEMTYPE ElemType = ELEMTYPE_Empty,			//!< ELEMTYPE_Xxxxx
		int table = -1			//!< Base table for pinmap queries
		);

	//! Destroy a segment
	int SegmentDestroy (
		int segment
		);

 	//! Used internally to setup the database 
	int SetupObjectDatabase (
		OBJECT *obj,
		RVC::FILEHANDLE* filehandle,
		RVC::DATABASEHANDLE* dbhandle,
		ELEMDATA *ElemData,
		ELEMTYPE ElemType,
		const RVC::CREDENTIALS* Credentials,
		bool fromcb = false
		);

	//! Set the usage for this script.
	//! Allows parser to warn or give an error if user tries to use a script
	//! for the wrong thing
	void SetUsage (
		int Segment,
		USAGE usage
		);

	//!	Called internally to make the script exit and notify the parent.
	//! If called from within a method, we have to set bCanDestroyContext
	//! to false or the calling context may be destroied before we return
	//! to it.
	int Stop (
		bool bCanDestroyContext
		);

	//!	Called internally to make the script exit and notify the parent.
	//! Old version without parameters, still need to export for backward
	//! compatibility.  Forces CanDestroyContext to true.  This version
	//! CAN NOT be called from within another method in SML::CONTEXT.
	int Stop (
		);

	//! Allocate an SML::STRING (char*).
	//! Assumes ISO-2022
	STRING* StringAlloc (
		const char* str
		);

	//! Allocate an SML::STRING (UINT8*).
	//! Assumes UTF8
	STRING* StringAlloc (
		const UINT8* str
		);

	//! Allocate an SML::STRING (MISTRING).
	STRING* StringAlloc (
		const MISTRING& str
		);

	//! Allocate an SML::STRING (STRUTF8).
	STRING* StringAlloc (
		const STRUTF8& str
		);

	//! Allocate an SML::STRING (MIUNICODE*).
	STRING* StringAlloc (
		const MIUNICODE* str
		);

	//! Free an SML::STRING
	void StringFree (
		STRING*
		);

	SYMBOL* TempObjectSymbolAlloc (
		STYPE stype
		);

	void TempObjectSymbolFree (
		SYMBOL*
		);

	//! Call the test_cancel function.
	//!
	//! @return 1 if we should cancel
	//! @note If HasCancelFunc() returns 0, do NOT call this function.
	//! For effeciancy, this function won't check and would end up
	//! calling through a NULL pointer.
	int TestCancelOnce (
		);

	void VMExecute (
		MACHINEINST* pc
		);

	int VMReset (
		);


	//! Allocate memory.
	//! Very similar to MmAlloc(), but this one keeps track of the allocated
	//! memory and it gets freed when the context is destroied.
	//! It is also optimized to handle lots of requests for little chunks
	//! of memory.
	//! IMPORTANT!  Memory allocated with this method MUST be freed with
	//! the MemFree() method. Freeing it with MmFree() or free() will result
	//! in a painful death.
	//! This should NOT be used to allocate anything returned by an SML
	//! function or class member.
	ERRVALUE MemAlloc (
		void** pRet,
		UINT32 size
		);

	//! Allocate and clear memory.
	//! Very similar to MmAllocC(), but this one keeps track of the allocated
	//! memory and it gets freed when the context is destroied.
	//! It is also optimized to handle lots of requests for little chunks
	//! of memory.
	//! IMPORTANT!  Memory allocated with this method MUST be freed with
	//! the MemFree() method. Freeing it with MmFree() or free() will result
	//! in a painful death.
	ERRVALUE MemAllocC (
		void** pRet,
		UINT32 size
		);

	//! Free memory.
	//! IMPORTANT!  This function must only be used to free memory allocted
	//! with the SML::CONTEXT::MemAlloc() and SML::CONTEXT::MemAllocC() methods.
	//! Attempting to free other memory will result in a painful death.
	//! Attempting to free a NULL pointer is ignored.
	//! Unlike MmFree, this does NOT set the pointer to NULL for you.
	void MemFree (
		void* ptr
		);

private:
	#ifndef GENERATING_DOXYGEN_OUTPUT
	CONTEXT();
	~CONTEXT();

	ERRVALUE CallFunctionLow (
		SYMBOL* func,		//!< SYMBOL of function to call
		ARG* args,			//!< List of arguments to pass
		int numargs,			//!< Number of arguments in "args"
		UINT8* tlist,			//!< STYPEs of the args
		ARG* ret,			//!< Return value
		UINT32 flags			//!< 1 means opCALL, 0 means callback
		);
	class REENTER;

	class MEMMGR {
		public:
			MEMMGR();
			~MEMMGR();

			ERRVALUE Alloc(void**, UINT32);

			//! Since the pointer allocated by the above function known
			//! the pool it was allocated from, we don't actually need
			//! a context to free from.  This is good because it means
			//! we can free from destructors of classes that don't know
			//! their context.
			static void Free(void*);

		private:
			MEMPOOL* m_pool[16];
		};

	// Check Virtual Machine -- done once on initialization to make sure we
	// didn't muck up the opcode table.
	int CheckVM (
		);

	//! Helper function called by Destroy().  To Balance InitLow()
	void DestroyLow (
		);

	int _Evaluate (
		int segnum,
		INT32 element,
		int ElemType,
		void *valp
		);

	void FreeOnExitData (
		);

	int GetAllFunctions (
		MISTRINGLIST& list,
		bool bIncludeAliases = true
		) const;

	int GetOpcodeByteCount (
		const MACHINEINST* pc
		);

	int InstallFunction (
		const FUNC* func,
		SYMBOL **psp,
		int funcgroup
		);

	//! Used by constructors to install the base function groups and classes
	int InstallQueryFunctions (
		);

	void DestroyPriv();

	//! these functions should be private but for now need to be called
	//! from a C style stub function.  Once those stubs are retired, they
	//! can become private.  If we find any that are not used within the
	//! DLL, they can just go away at that point. (Although perhaps some
	//! of them should stay public and be documented?)

public:		//! (But don't document them)

	//! Destroy an instance of a class pointer
	void DestroyClassInstance (
		void* ptr,
		const CLASS* Class,
		int NumItems = 1
		);

	//! Create an instance of a class pointer
	void* ConstructClassInstance (
		const CLASS* Class
		);

	//! Copy an instance of a class pointer
	void* CopyClassInstance (
		void* ptr,
		const CLASS* Class
		);

	//! Will become private once wrapper function no longer needed
	CLASS* FindNamedClass (
		const char* name
		) const;

	//! [ private ]
	//! Will become private once wrapper function no longer needed
	void FreeEnum (
		ENUMLIST* smlenum
		);

	//! Will become private once wrapper function no longer needed
	SYMBOL* FreeSymbols (
		SYMBOL* sp,
		int freelocal
		);

	//! Will become private once wrapper function no longer needed
	void FreeUserFuncs (
		int segnum
		);

	//! Does all the work of _SML_CreateContextFromDBHandle except
	//! the actual allocation.  I could have named this Init too
	//! and let function overloading take care of the rest, but
	//! this way I'm not depending on the number of parameters staying
	//! the same.
	int InitFromDBHandle (
		RVC::DATABASEHANDLE* dbhandle,
		bool ReopenHandle = true
		);

	//! Helper function used by both of the Init functions
	//! (in qqinit.c)
	int InitLow (
		);

	//! Returns true if the script is running
	bool IsRunning (
		) const;

	//! See if class or function name has been aliased.
	//! Returns the new name of the class or function given the old name
	const char* GetAliasNewName (
		const char* OldName
		) const;

	//! See if class or function name has been aliased.
	//! Returns the old name of the class or function given the new name
	const char* GetAliasOldName (
		const char* NewName
		) const;

	//! See if class or function name has been aliased.
	//! Returns the old name of the class or function given the new name and index
	const char* GetAliasOldName (
		const char* NewName,
		int index
		) const;

	void ParserReset (
		);

	int _Run (
		int segnum,
		UINT32 flags = 0
		);


	int SetupDatabase (
		RVC::FILEHANDLE* filehandle,
		RVC::DATABASEHANDLE* dbhandle,
		const RVC::CREDENTIALS* Credentials,
		bool fromcb
		);

	int SetupParentObjectSymbol (
		const RVC::OBJITEM& ParentObjItem,
		const RVC::CREDENTIALS* Credentials = 0
		);

	//! helper function used by both flavors of SetErrorCode()
	//! Named SMLSetErrPos() to avoid confusion with SetErrPosn()
	void SMLSetErrorPos (
		);

	//! Will become private once wrapper function no longer needed
	//!
	//! Typically, you should call AddSymbol() instead
	//! @see SMLCONTEXT::AddSymbol()
	//!
	SYMBOL* SymbolAlloc (
		const MIUNICODE* s
		);

	ERRVALUE SymbolSetDescriptor (
		SYMBOL* sym,
		const RVC::DESCRIPTOR& Descriptor
		);

	//! Force strict syntax checking.
	void SetStrictSyntax (
		bool strict = true
		);

	void SetErrorLow (
		int code,
		bool bAbort = true
		);

	//! Will become private once wrapper function no longer needed
	SYMBOL* SymbolAlloc (
		const char* s
		);

	//! Will become private once wrapper function no longer needed
	SYMBOL* SymbolInstall (
		const char* s,
		int t,
		double d,
		SYMBOL::SCOPE scope
		);

	//! Will become private once wrapper function no longer needed
	SYMBOL* SymbolInstall (
		const MIUNICODE* s,
		int t,
		double d,
		SYMBOL::SCOPE scope
		);

	//! Will become private once wrapper function no longer needed
	void SymbolTableFree (
		int FreePerm
		);

	//! Will become private once wrapper function no longer needed
	void SymbolTableInit (
		);
	//!
	//! Will become private once wrapper function no longer needed
	void SymbolTablePop (
		);

	//! Will become private once wrapper function no longer needed
	void SymbolTablePush (
		SYMBOL* sym
		);

	//! Non member functions which need access to private data
	//! These are required by the YACC parser.  qqparse(), whic
	//! is generated by YACC is a friend of SML::PARSER
	friend  int ::qqlex(void*, SML::PARSER*);
	friend  void ::qqerrorext(SML::CONTEXT*, int, const char* const*, const short*, int);
	friend  void ::qqerror(void*, const char*);
	//! These two classes are derived from CONTEXT, but we want
	//! to give them access to private stuff too.
	//! Eventually, make the stuff they need protected instead of
	//! private and remove the "friend" lines.  But not until it's
	//! safe to rearange the members, because making some protected
	//! will change the order, breaking old code which accessed the
	//! members directly from outside the dll. (6.2 release doesn't
	//! do this, but may still need to send current rvcapp.dll in
	//! patches to pre 6.2 systems)
	friend class PARSER;
	friend class VM;
	friend class REENTER;

	friend class STRING;	//!< Constructor/destructor needs private data access
	friend class OBJECT;	//!< Constructor/destructor needs private data access
	friend class STRGC;	//!< String garbage collection needs m_strings
	friend class AUTOCLASS;
	friend class SYMBOL;

private:
	//! Does all the work of SML_InitExt except the actual
	//! allocation.
	//! Only used by a deprecated version of Create
	int Init (
		const RVC::OBJITEM* pParentObjItem,
		const RVC::CREDENTIALS* Credentials,
		UINT32
		);

#define SML_NUM_REGISTERS 8
	ARG *m_sp;
	ARG m_registers[SML_NUM_REGISTERS];
	FRAME *m_fp;
	int m_progp;
	const MIUNICODE *m_pSource;				//!<  source to parse (only valid while parsing)
	MACHINEINST *m_cs, *m_pc;
	MISTRING m_message;
	int  m_ParsePos;			//!<  character position currently being parsed
	int  m_ParseLine;			//!<  line of this input file currently being parsed
	int  m_LineCount;			//!<  line of input currently being parsed (total)
	MISTRING m_PrintBuf;

	int  m_ErrorCode;
	SYMBOL *m_pLastSym;			//!<  last symbol - used for assignment
	SYMBOL *m_symlist[10];
	SYMBOL **m_funcs;			//!<  Array of built-in function classes
	char **m_pFuncClass;			//!<  Array of built-in function class names
	int m_NumFuncClasses;		//!<  Number of built-in function classes
	SIMPLE_ARRAY<SEGMENT*> m_SegTable;
	int m_CurrentSegNum;
	int m_TablePtr;			//!< Not really a pointer
	int m_NextTopen;
	bool m_bReturning;
	bool m_bHalting;
	bool m_Brk;
	bool m_bParseBegin;	//!<  flags used in parsing
	bool m_bParseEnd;
	bool m_bParseForEach;	//!<  parsing a foreach loop
	bool m_bEOF;
	bool m_bOnlyNeedName;	//!<  parsing: treat next symbol as just a string
	bool m_bDatabaseRoot;	//!<  elements are record numbers in main table
	bool m_bNoExitError;
	bool m_bNewLine;			//!<  Set when parsing if a newline has been hit
	bool m_bDestroied;		//!<  Context destroyed but not free'd because of outstanding destroy callbacks
	bool m_bNeedLineNumbersInCode;
	bool m_bNeedTableName;
	bool m_bNeedFieldName;
	bool m_bNeedElemType;
	bool m_bParsing;
	bool m_bInStopCallback;		//!<  So destroy context won't actually destroy
	bool m_bDestroiedInStopCallback;		//!<  Tried to destroy in stop callback, do it after
	bool m_bClassesResolved;	//!<  Already done, Installing new classes must do it again
	bool m_bForceLocalScope;
	bool m_bStartOfLine;
	ELEMDATA *m_pElemData, *m_pCurElemData, *m_pRootElemData;
	ELEMTYPE	m_RootElemType, m_CurElemType;
	void *m_pConsole;
	bool m_bConsoleIsOld;
	MDLGPARENT m_parent;				//!<  Really a Widget under X
	STRING *m_pStrList;
	MILIST<STRING*> m_ConstStrings;	//!<  Linked list containing strings in code
	int *m_pSrcPos;
	MISTATUSCONTEXT *m_pStatusContext;
	int (*m_CancelFunc)(CONTEXT *, void *);
	void *m_CancelData;
	SYMBOL *m_pStdSym; //!< , *foreachsym;
	int m_LastParse;	//!<  Last line number parsed
	SIMPLE_ARRAY<SML::DEVPRIVATE*> m_DevPrivatePool;
	int m_scope, m_vartype;
	INT32 m_SeqNum;			//!<  Incremented for every call to DBQuery_Evaluate
	SYMBOL *m_pForEachRastSym;	//!<  Current foreach
	FILEPATH m_pScriptFilepath;
	INT32 m_ScriptInode;						//!<  So the script can know where it is
	RVC::OBJITEM m_ScriptObjItem;
	CLASS *m_pRegisteredClasses;
	OBJECT* m_ObjListHead;
	OBJECT* m_ObjListTail;
	MTIMER m_CancelCheckTimer;
	UINT32 m_CancelCheckInterval;			//!<  How often to check cancel in ticks (MtimerTicksPerSecond() * seconds)
	UINT32 m_NextCancelCheckTick;			//!<  Check cancel if MtimerTickCount() is > than this
	int m_LastPoppedType;
	MICALLBACKLIST m_CallbackList;
	int m_fid;								//!<  RVC File handle to close when destroying context (for CreateObjectContext)
	UINT32 m_status;
	int m_ArrayIndexBase;				//!<  Do arrays start at 1 or 0 ? (default is 1)
	SIMPLE_ARRAY<int> m_BreakPoints;
	SMLDEBUG* m_Debugger;
	void (*m_DebugCallback)(CONTEXT *context, void *DebugData);
	void *m_DebugData;
	int m_OutstandingDestroys;
	int m_RefCount;
	SIMPLE_ARRAY<ONEXITDATA*> m_OnExitList;
	int *m_pPCLine;
	SEGMENT *m_pCurSeg;
	SYMBOL *m_PredefSymTable;			//!<  Table of predefined symbols (don't delete)
	SOURCEINFO *m_pParseFile;					//!<  Filename to report error from -- DON'T FREE
	int m_ParseFileNum;
	SYMBOL *m_pPermSymbols;				//!<  Symbols that don't get free'd between parses
	DISPLAYINFO *m_pDispInfo;				//!<  Must be filled in outside the DLL
	/*--- Anything added after this point needs a function */
	/*    to access it if needed outside of rvcapp.dll  ---*/
	SYMBOL *m_pDefFunc;				//!<  Function being defined (parseer use only)
	int m_ClassPtrStackDepth;
	CLASSSTACK m_ClassPtrStack[20];
	ENUMLIST *m_EnumList;
	STRING *m_pLastPoppedString;
	CLASS *m_pLastPoppedClassType;
	void *m_pLastPoppedClassPtr;
	bool m_bLastPoppedClassIsBound;

	PREPROCESSORSTACK *m_pCPPStack;
	MEMMGR m_Mem;
	CONTEXTTYPE m_AutoClassTypesInstalled;
	MIPRODUCTCODE m_ProductCode;

#ifdef WIN32
	int LoadActiveXModule(const MIUNICODE* name);
	int InstallOLEType(IDispatch*, _GUID* pclsid, ITypeInfo*);
#endif
	static int s_RefCount;		//!< Number of context's currently open
	static SIMPLE_ARRAY<void*> s_hPlugins;
	int LoadPlugins();
	int UnloadPlugins();

	static MICALLBACKLIST s_CallbackList;	//!< List to add callbacks to for context create/destroy

	bool m_bCaseSensitiveStrings;
	bool m_bCurrentFuncIsClassMethod;
	bool m_bAbortOnError;
	UINT32 m_OptimizationFlags;

	protected:
		//! Actually only used by SML::PARSER which is a derived interface-only
		//! class and really only exists to hide the internal workings of SML
		CLASS* m_LastClass;
		CLASS* m_CurrentClass;
		SYMBOL *m_CallStack[20], *m_CurFunc;	//!<  Used for parsing only
		SYMBOL *m_CurOverload;	//!< used for parsing only
		int m_CallStackDepth;
		mutable SYMBOL *m_LastTable;
		struct LOOPDATA;

		LOOPDATA *m_LoopData;
		int m_numparms;
		int m_defparms;
		bool m_bInStr;
		int m_lastToken;
		int m_thisToken;
		int m_prevToken;

		MISTRING m_ParserWarnings;
		int m_ParserWarningLevel;
		SIMPLE_ARRAY<FIELDDATA> m_UnindexedFields;
		bool m_bStrictSyntax;
		SIMPLE_ARRAY<const void*> m_AlreadyWarned;	//!<
		SYMBOL* m_PrevTableSym;
		XMLDOC* m_XMLDoc;

		SYMBOL* m_ParserCurFuncProto;		// Prototype for func we're defining
		SYMBOL* m_ParserCurObject;
		SYMBOL* m_ParserCurTable;
		ELEMTYPE m_ParserCurElemType;
		UINT32 m_RegisterUsage;		// bits indicating which registers are allocated
		bool m_bForceNextToken;		// used while parsing
		int m_NextToken;		// used while parsing
		SIMPLE_ARRAY<INT32> m_ScopeStack;	// used while parsing
		INT32 m_ScopeRules;

		struct NAMEALIAS {
			NAMEALIAS(const char* o, const char* n) :
				OldName(o),
				NewName(n)
				{
				}
			const char* OldName;
			const char* NewName;
			};

		SIMPLE_ARRAY<NAMEALIAS> m_Aliases;

		struct UNRESOLVED {
			const char* name;
			void** ptr;
			bool bIsEnum;
			};

		MILIST<UNRESOLVED> m_UnresolvedList;

		void AddParserWarning (
			const MISTRING&,
			int WarningLevel = 1
			);

		void AddParserWarning (
			TEXTID id,
			int WarningLevel = 1
			);

		void AddDeprecatedWarning (
			const void* whatever,	//!< Just to keep track of what we warned about
			const char* name,	//!< function, class, or class.member name
			const char* desc	//!< Will be "RetireInX.X"
			);
#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

class CONTEXT_REFCOUNT {
	public:
		CONTEXT_REFCOUNT (
			CONTEXT* context
			) :
			m_context(context)
			{
			if (m_context != 0) m_context->AddRef();
			}

		~CONTEXT_REFCOUNT (
			) {
			if (m_context != 0) m_context->Release();
			}
	private:
			CONTEXT* m_context;
	};

//=====================================================================================================================

RVCSYSLIBEXPORT int CreateContextFromDBHandle (
	RVC::DATABASEHANDLE* dbhandle, 						//!< Handle of open database
	SML::CONTEXT *& context, 		//!< (SML::CONTEXT*) returned
	UINT32 flags					//!< XXX Document flags
	);

} // End of SML namespace
//=====================================================================================================================

typedef SML::CONTEXT SMLCONTEXT;
typedef SML::CONTEXT_REFCOUNT SMLCONTEXT_REFCOUNT;
typedef SML::DEVPRIVATE SMLDEVPRIVATE;
typedef SML::FIELDDATA SMLFIELDDATA;

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

//! Called once at program startup to register all the functions and classes
//! so that they'll be available in every context.
void SML_AppInit (
	);

//! Return the MDLGPARENT for the context
//!
//! This is the same as SML::CONTEXT::GetParent() except that for
//! technical reasons, that function is exported as returning a Widget
//! (it returns an MDLGPARENT which is defined as a Widget when rvcapp
//! is compiled).  This function exists outside the DLL so if compiled
//! with WIN32_NATIVE defined and linked to libmiwin.lib, actually
//! returns a CWnd
//!
MDLGPARENT SML_GetParent (
	SML::CONTEXT *context
	);

//! Create basic SML::CONTEXT.
//! Creates one segment (segment 0) and adds the base SML functions
//! (the ones listed in SML_SMLFuncList() in mi32/smlfuncs.h
int SML_InitExt (
	SML::CONTEXT **pContext,		//<! pointer to context RETURNED
	SML::USAGE Usage = SML::USAGE_StandAlone
	);

#if !defined(GENERATING_DOXYGEN_OUTPUT)
//! Create an SMLCONTEXT for use with Pinmapping
//! This function creates an SMLCONTEXT for a single database.
//! Note: This is for internal use by RVC::DATABASE only
int DBQuery_CreateContextFromDBHandle (
	RVC::DATABASEHANDLE* dbhandle,
	SML::CONTEXT *& context, 	//!< Context returned
	UINT32 flags			//!< ignored
	);
#endif // GENERATING_DOXYGEN_OUTPUT

//! Create context with associated object specified by RVC::OBJITEM.
int SML_CreateObjectContext (
	const RVC::OBJITEM& item,
	SML::CONTEXT **pcontext 	//!< Context returned
	);

#endif		// INC_SML_CONTEXT_H
