/**
 * \file soa/uiserverlist.h
 * \brief SOA User Interface for Server List
 *
 * \if NODOC
 * $Id: uiserverlist.h_v 1.3 2007/08/21 20:33:51 honerkamp Exp $
 *
 * $Log: uiserverlist.h_v $
 * Revision 1.3  2007/08/21 20:33:51  honerkamp
 * Addd search option to server selection
 *
 * Revision 1.2  2007/04/09 16:05:34  mju
 * Add dlgDownloadServerList.
 *
 * Revision 1.1  2007/02/09 22:11:25  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SOA_UISERVERLIST_H
#define	INC_SOA_UISERVERLIST_H

#ifndef	INC_SOA_SERVERLIST_H
#include <soa/serverlist.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_EDITURL_H
#include <mgui/editurl.h>
#endif

namespace SOA {
//=====================================================================================================================

//! Dialog to allow user to add item to server list.
class DLG_SERVERLIST_ADDITEM : public MGUI::DLGSHELL {
	public:

		//! Delegate for handling notification of item having been added.
		//! Item is always added to end of list.
		//! Parameters are:
		//!    Pointer to parent, NULL if added to root list
		//!    Pointer to item (in list) actually inserted
		typedef fastdelegate::FastDelegate<void(const SERVERLIST::ITEM*, const SERVERLIST::ITEM*)> DELEGATE_ONITEMADDED;

		//! Constructor.
		DLG_SERVERLIST_ADDITEM (
			SERVERLIST& ServerList,				//!< Reference to server list
			const MISTRING& listname,			//!< Name of list
			const MISTRING& title,				//!< Dialog title
			const MISTRING& prompt,				//!< Prompt
			const SERVERLIST::ITEM& item		//!< Item to add
			);

		virtual ~DLG_SERVERLIST_ADDITEM ();

		//! Set delegate to call when item actually added.
		//! Will be called if user adds folders, as well as when OK pressed
		void SetDelegateOnItemAdded (
			DELEGATE_ONITEMADDED DelegateOnItemAdded
			) { m_DelegateOnItemAdded = DelegateOnItemAdded; }

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		typedef DLG_SERVERLIST_ADDITEM THISCLASS;
		SERVERLIST& m_ServerList;
		MISTRING m_ListName;
		MISTRING m_Prompt;
		SERVERLIST::ITEM m_Item;
		DELEGATE_ONITEMADDED m_DelegateOnItemAdded;

		MGUI::CTRL_LABEL m_LblPrompt;
		MGUI::CTRL_LABEL m_LblURL;
		MGUI::CTRL_EDIT_URL m_EditURL;
		MGUI::FORM_EDIT_STRING m_EditName;
		MGUI::FORM_COMBOBOX m_CbxCreateIn;
		MGUI::CTRL_PUSHBUTTON m_BtnNewFolder;
		bool m_IsModifiedURL;

		SERVERLIST::ITEM* AddItem (SERVERLIST::ITEM *pParent, const SERVERLIST::ITEM& NewItem, SERVERLIST::ITEM *pListItem = 0);
		ERRVALUE CheckURL ();
		void EnableControls ();
		void OnCmdNewFolder ();
		void OnEditURL ();
		void OnEnterURL ();

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOK ();

		DLG_SERVERLIST_ADDITEM (const DLG_SERVERLIST_ADDITEM&);
		DLG_SERVERLIST_ADDITEM& operator= (const DLG_SERVERLIST_ADDITEM&);
		#endif
	};

//=====================================================================================================================

//! Display dialog while downloading catalog.
ERRVALUE DlgDownloadServerList (
	MDLGPARENT dlgparent,			//!< Parent for status dialog
	const MISTRING& msg,				//!< Message to show in status window
	const MISTRING& url,				//!< URL to catalog
	SERVERLIST::ITEM& result,		//!< Item to contain result if successful
	std::vector<MISTRING> *additionaltags=0 //!< Additional xml tags to parse out of the xml.
	);

//=====================================================================================================================
}	// End namespace SOA


#endif	// INC_SOA_UISERVERLIST_H
