/*
 *	MICROIMAGES Xtrans.h
 *
 *	X11 wrappers around our own transport routines
 *
 * These are mainly needed to build the X libs, not for
 * general usage.
 */

#ifndef XTRANS_H
#define XTRANS_H

#include <direct.h>
#include <stdlib.h>
#include <io.h>
#include <X11/Xw32defs.h>

#ifndef XLIBEXPORT
#ifdef WIN32
#ifdef XLIBDLL
#define XLIBEXPORT __declspec(dllexport)
#else
#define XLIBEXPORT __declspec(dllimport)
#endif
#else
#define XLIBEXPORT 
#endif
#endif


typedef struct {
	char data[32];
	} Xtransaddr;

typedef struct _XtransConnInfo {
	int fd;
	int family;
	int addrlen;
	Xtransaddr addr;
	} *XtransConnInfo;

typedef struct iovec {
	char*	iov_base;
	int	iov_len;
	} iovec_t;

typedef int BytesReadable_t;

#define TRANS_TRY_CONNECT_AGAIN	2

#define TRANS_ACCEPT_BAD_MALLOC	3

#define TRANS_CLOSEONEXEC	1
#define TRANS_NONBLOCKING	2

#ifdef __cplusplus
extern "C" {
#endif


XtransConnInfo _X11TransOpenCOTSClient (
	const char* address
	);

// done
int _X11TransConnect (
	XtransConnInfo trans_con,
	const char* address
	);

// done
int _X11TransDisconnect (
	XtransConnInfo trans_con
	);


//done
XLIBEXPORT int _X11TransGetHostName (
	char* name,
	int len
	);

// done
XLIBEXPORT int _X11TransGetPeerAddr (
	XtransConnInfo trans_con,
	int* pFamily,
	int* pAddrLen,
	Xtransaddr** pAddr
	);

// done
int _X11TransGetMyAddr (
	XtransConnInfo cip, 
	int *pFamily,
	int *pAddrLen,
	Xtransaddr** pAddr
	);

// done
int _X11TransConvertAddress (
	int* pFamily,
	int* pAddrLen,
	Xtransaddr** pAddr
	);

// done
int _X11TransClose (
	XtransConnInfo trans_con
	);


	// Possible options:
	// 	TRANS_CLOSEONEXEC
	// 	TRANS_NONBLOCKING
// done
int _X11TransSetOption (
	XtransConnInfo trans_con,
	int option,
	int value
	);

// done
XLIBEXPORT int _X11TransGetConnectionNumber (
	XtransConnInfo trans_con
	);

// done
int _X11TransFreeConnInfo (
	XtransConnInfo trans_con
	);

// done
int _X11TransBytesReadable (
	XtransConnInfo cip, 
	BytesReadable_t* pPend
	);

// done
XLIBEXPORT int _X11TransRead (
	XtransConnInfo cip, 
	void* buf, 
	unsigned int len
	);

// done
XLIBEXPORT int _X11TransWrite (
	XtransConnInfo cip, 
	const void* buf, 
	unsigned int len
	);


// done
XLIBEXPORT int _X11TransWritev (
	XtransConnInfo cip, 
	struct iovec* iov, 
	unsigned int iovcnt
	);

// done
XLIBEXPORT int _X11TransReadv (
	XtransConnInfo cip, 
	struct iovec* iov, 
	unsigned int iovcnt
	);

XLIBEXPORT int _X11TransSelect (
	int nfds,
	struct fd_set* readfds,
	struct fd_set* writefds,
	struct fd_set* exceptfds,
	const struct timeval* timeout
	);

XLIBEXPORT int _X11TransFDIsSet (
	int s,
	struct fd_set* fd
	);

XLIBEXPORT int _X11TransGetLastError (
	);

XLIBEXPORT void _X11TransSetLastError (
	int error
	);

#define WSAGetLastError _X11TransGetLastError
#define WSASetLastError _X11TransSetLastError

#undef FD_ISSET
#define FD_ISSET _X11TransFDIsSet
#define gethostname _X11TransGetHostName

#define _XimXTransOpenCOTSClient _X11TransOpenCOTSClient
#define _XimXTransConnect _X11TransConnect
#define _XimXTransClose _X11TransClose
#define _XimXTransGetConnectionNumber _X11TransGetConnectionNumber
#define _XimXTransDisconnect _X11TransDisconnect
#define _XimXTransRead _X11TransRead
#define _XimXTransWrite _X11TransWrite


#undef sleep
XLIBEXPORT void sleep (
	long n
	);

/* In ConnDis.c */
int _XConnectDisplay (
    char *display_name,
    char **fullnamep,
    int *dpynump,
    int *screenp,
    char **auth_namep,
    int *auth_namelenp,
    char **auth_datap,
    int *auth_datalenp
	);


#ifdef __cplusplus
	}
#endif

#endif // XTRANS_H
